/*

  array.c
  
  Gestion d'un tableau dynamique de taille croissante
   l'aide d'une unique allocation
  
*/

#define _ARRAY_C

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "common.h"
#include "error.h"
#include "memory.h"
#include "array.h"


/*#define DEBUG*/

#define DATA_TO_ARRAY(data) ((ARRAY *)((char *)data - sizeof(ARRAY)))
#define ARRAY_TO_DATA(array) ((char *)array + sizeof(ARRAY))


PUBLIC void ARRAY_create_with_size(void *p_data, size_t size, long inc)
{
  ARRAY *array;
  
  #ifdef DEBUG
  printf("ARRAY %d\n", size);
  #endif
  MEM_alloc(&array, sizeof(ARRAY));
    
  array->count = 0;
  array->max = 0;
  array->size = size;
  array->inc = inc;
  
  *((void **)p_data) = ARRAY_TO_DATA(array);
}


PUBLIC void ARRAY_delete(void *p_data)
{
  void **data = (void **)p_data;
  ARRAY *alloc = DATA_TO_ARRAY(*data);

  if (!*data)
    return;

  #ifdef DEBUG
  printf("ARRAY\n");
  #endif
  MEM_free(&alloc);
  
  *data = NULL;
}


PUBLIC long ARRAY_count(void *data)
{
  return DATA_TO_ARRAY(data)->count;
}


PRIVATE void *add(void *p_data, boolean zero)
{
  long inc;
  void **data = (void **)p_data;
  ARRAY *array = DATA_TO_ARRAY(*data);
  char *ptr;
  
  /*
  inc = (1024 / array->size);
  if (!inc) inc = 1;
  */
  
  inc = array->inc;
  
  array->count++;

  if (array->count > array->max)
  {
    array->max += inc;
    #ifdef DEBUG
    printf("ARRAY\n");
    #endif
    MEM_realloc(&array, sizeof(ARRAY) + array->max * array->size);
    *data = ARRAY_TO_DATA(array);
  }
  
  ptr = (char *)array + sizeof(ARRAY) + array->size * (array->count - 1);
  
  if (zero) memset(ptr, 0, array->size);
  
  return ptr;
}


PUBLIC void *ARRAY_add(void *p_data)
{
  return add(p_data, FALSE);
}


PUBLIC void *ARRAY_add_void(void *p_data)
{
  return add(p_data, TRUE);
}


PUBLIC void ARRAY_remove_last(void *p_data)
{
  void **data = (void **)p_data;
  ARRAY *array = DATA_TO_ARRAY(*data);

  if (!array->count)
    return;
    
  array->count--;
}


PUBLIC void *ARRAY_get(void *data, int pos)
{
  ARRAY *array = DATA_TO_ARRAY(data);
  
  return (char *)data + array->size * pos;
}
