/*

  buffer.c
  
  Gestion d'un buffer de chaines
  
*/


#define _BUFFER_C

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "common.h"
#include "error.h"
#include "memory.h"
#include "buffer.h"

/*#define DEBUG*/

#define DATA_TO_BUFFER(data) ((BUFFER *)((char *)data - sizeof(BUFFER)))
#define BUFFER_TO_DATA(buffer) ((char *)buffer + sizeof(BUFFER))


PUBLIC void BUFFER_create(void *p_data)
{
  BUFFER *buf;

  #ifdef DEBUG
  printf("BUFFER\n");
  #endif
  MEM_alloc(&buf, sizeof(BUFFER));
  
  buf->length = 0;
  buf->max = 0;
  
  *((void **)p_data) = BUFFER_TO_DATA(buf);
}


PUBLIC void BUFFER_delete(void *p_data)
{
  void **data = (void **)p_data;
  BUFFER *buf = DATA_TO_BUFFER(*data);

  #ifdef DEBUG
  printf("BUFFER\n");
  #endif
  MEM_free(&buf);
  *data = NULL;
}

PUBLIC long BUFFER_length(void *data)
{
  return DATA_TO_BUFFER(data)->length;
}


PUBLIC boolean BUFFER_need(void *p_data, long size)
{
  void **data = (void **)p_data;
  BUFFER *buffer = DATA_TO_BUFFER(*data);
  
  buffer->length += size;
  if (buffer->length > buffer->max)
  {
    buffer->max += BUFFER_INC;
    #ifdef DEBUG
    printf("BUFFER\n");
    #endif
    MEM_realloc(&buffer, sizeof(char) * buffer->max + sizeof(BUFFER));
    *data = BUFFER_TO_DATA(buffer);
  }
  
  return FALSE;
}


PUBLIC long BUFFER_add(void *p_data, void *string, size_t len)
{
  void **data = (void **)p_data;
  BUFFER *buffer = DATA_TO_BUFFER(*data);
  long pos;

  pos = buffer->length;
  BUFFER_need(p_data, len);

  memcpy(*data + pos, string, len);
  return pos;
}


/*
PUBLIC long BUFFER_add_string(void *p_data, char *string, short len)
{
  long pos;

  pos = BUFFER_add(p_data, &len, sizeof(len));
  BUFFER_add(p_data, string, len);

  return pos;
}
*/

PUBLIC void BUFFER_load_file(void *p_data, const char *name)
{
  void **data = (void **)p_data;
  BUFFER *buffer = DATA_TO_BUFFER(*data);

  int fd;
  struct stat info;
  long len;
  BUFFER *new_buffer;

  fd = open(name, O_RDONLY);
  if (fd < 0)
    THROW(E_OPEN, name);

  fstat(fd, &info);

  len = info.st_size;

  MEM_alloc(&new_buffer, len * sizeof(char) + sizeof(BUFFER));

  if (read(fd, BUFFER_TO_DATA(new_buffer), len) != len)
  {
    MEM_free(&new_buffer);
    THROW(E_READ, name);
  }

  if (*data)
  {
    MEM_free(&buffer);
  }

  new_buffer->length = len;
  new_buffer->max = len;
  *data = BUFFER_TO_DATA(new_buffer);
}
