/***************************************************************************

  compile.h

  Compiler initialization, reserved keywords table and subroutines table.

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef _COMPILE_H
#define _COMPILE_H

#include "array.h"
#include "buffer.h"
#include "table.h"
#include "memory.h"

#include "limit.h"
#include "read.h"

#include "class.h"

typedef
  struct {
    char *name;                /* nom du fichier source */
    long line;                 /* numro de ligne courante */
    char *source;              /* contenu du fichier source */
    unsigned verbose    : 1;   /* compilation prolixe */
    unsigned debug      : 1;   /* informations de dboguage */
    unsigned is_module  : 1;   /* si le source est une classe ou un module */
    unsigned is_form    : 1;   /* if the source is a class form */
    unsigned declared   : 1;   /* module dclar */
    unsigned _reserved  : 11;  /* rserv */
    char *output;              /* fichier gnr */
    PATTERN *pattern;          /* analyse lexicale du source */
    PATTERN *current;          /* position de traduction courante */
    PATTERN *end;              /* fin de traduction */
    FUNCTION *func;            /* fonction en cours de compilation */
    CLASS *class;              /* classe en cours de compilation */
    char *form;                /* nom du fichier formulaire */
    }
  PACKED
  COMPILE;


typedef
  struct {
    char *name;
    short flag;
    short value;
    short priority;
    short code;
    }
  PACKED
  COMP_INFO;

typedef
  struct {
    char *name;
    ushort opcode;
    ushort optype;
    short min_param;
    short max_param;
    }
  PACKED
  SUBR_INFO;

typedef
  struct {
    short index;
    char *name;
    }
  PACKED
  CONST_INFO;

#ifndef _COMPILE_C

EXTERN COMP_INFO COMP_res_info[];
EXTERN SUBR_INFO COMP_subr_info[];
EXTERN CONST_INFO COMP_const_info[];

EXTERN TABLE *COMP_res_table;
EXTERN TABLE *COMP_subr_table;
EXTERN TABLE *COMP_const_table;

EXTERN COMPILE COMP_current;

EXTERN char *COMP_project;

#endif

#define JOB (&COMP_current)

PUBLIC SUBR_INFO *SUBR_get(char *subr_name);
PUBLIC SUBR_INFO *SUBR_get_from_opcode(ushort opcode, ushort optype);
PUBLIC CONST_INFO *CONST_get(char *cst_name);
PUBLIC CONST_INFO *CONST_get_from_index(short index);

PUBLIC void COMPILE_init(void);
PUBLIC void COMPILE_load(void);
PUBLIC void COMPILE_exit(void);
PUBLIC void COMPILE_begin(char *file);
PUBLIC void COMPILE_end(void);

#endif
