/*

  error.c
  
  Gestion des erreurs
  
*/

#define _ERROR_C

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "common.h"
#include "error.h"


PUBLIC boolean ERROR_flag;

PRIVATE int ERROR_niv;
PRIVATE ERROR ERROR_buf[ERROR_BUF_MAX];

PRIVATE char *ERROR_msg[] =
{
  /* E_OK     	  */ "OK !?",
  /* E_MESSAGE 	  */ "%s",
  /* E_MEMORY 	  */ "Out of memory",
  /* E_OPEN   	  */ "Cannot open file '%s'",
  /* E_READ   	  */ "Cannot read file '%s'",
  /* E_SYNTAX     */ "Syntax error"
};

PRIVATE char ERROR_msgbuf[ERROR_MSG_MAX];
PRIVATE char *ERROR_msgptr = ERROR_msgbuf;


PUBLIC ERROR *ERROR_get(void)
{
  if (ERROR_niv == ERROR_BUF_MAX)
    ERROR_panic("Too many error handlers imbricated");
   
  return &ERROR_buf[ERROR_niv++];
}


PUBLIC void ERROR_free(void)
{
  if (ERROR_niv == 0) return;
  
  ERROR_niv--;
}


void THROW(char *fmt, ...)
{
  va_list args;
  int len;
  long code = (long)fmt;

  va_start(args, fmt);

  if (ERROR_niv == 0)
  {
    vfprintf(stderr, ERROR_msg[code], args);
    exit(1);
  }

  /*
  len = sprintf(ERROR_msgptr, "%s: ", func);
  ERROR_msgptr += len;
  */

  if (code >= 0 && code <= 255)
    len = vsprintf(ERROR_msgptr, ERROR_msg[code], args);
  else
    len = vsprintf(ERROR_msgptr, fmt, args);

  ERROR_msgptr += len;
  *ERROR_msgptr++ = '\n';

  if (ERROR_msgptr >= &ERROR_msgbuf[ERROR_MSG_MAX])
    ERROR_panic("Message buffer overflow");

  longjmp(ERROR_buf[ERROR_niv - 1].env, 1);
}

PUBLIC void PROPAGATE()
{
  if (ERROR_niv == 0)
    ERROR_panic("Can't propagate error. No more error handler");

  longjmp(ERROR_buf[ERROR_niv - 1].env, 1);
}


PUBLIC void ERROR_panic(char *error)
{
  fflush(NULL);
  fprintf(stderr, "** INTERNAL ERROR: %s.\n", error);
  fprintf(stderr, "Program aborting... Sorry...\n");
  exit(1);
}

PUBLIC void ERROR_print(void)
{
  *ERROR_msgptr = 0;
  fputs(ERROR_msgbuf, stderr);
}
