/***************************************************************************

  main.c

  Welcome to the compiler !

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __MAIN_C

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <dirent.h>

#include <unistd.h>
#include <getopt.h>

#include "common.h"
#include "error.h"
#include "str.h"
#include "file.h"

#include "compile.h"

#include "reserved.h"
#include "read.h"
#include "form.h"
#include "trans.h"
#include "header.h"
#include "output.h"


PRIVATE struct option Long_options[] =
{
  { "debug", 0, NULL, 'g' },
  { "version", 0, NULL, 'V' },
  { "help", 0, NULL, 'h' },
  { "verbose", 0, NULL, 'v' },
  { "all", 0, NULL, 'a' },
  { 0 }
};

PRIVATE char **path_list;
PRIVATE int path_current;

PRIVATE boolean main_debug = FALSE;
PRIVATE boolean main_verbose = FALSE;
PRIVATE boolean main_compile_all = FALSE;


PRIVATE void get_arguments(int argc, char **argv)
{
  int opt;
  int index = 0;

  for(;;)
  {
    opt = getopt_long(argc, argv, "gvaVh", Long_options, &index);
    if (opt < 0) break;

    switch (opt)
    {
      case 'V':
        printf("gbc-" VERSION "\n");
        exit(0);

      case 'g':
        main_debug = TRUE;
        break;

      case 'v':
        main_verbose = TRUE;
        break;

      case 'a':
        main_compile_all = TRUE;
        break;

      case 'h': case '?':
        printf(
          "\n"
          "GAMBAS Compiler version " VERSION " " __DATE__ " " __TIME__ "\n"
          "(c) 2000-2002 Benot Minisini\n\n"
          "This program is free software; you can redistribute it and/or \n"
          "modify it under the terms of the GNU General Public License as \n"
          "published by the Free Software Foundation; either version 1, or \n"
          "(at your option) any later version.\n\n"
          "This program is distributed in the hope that it will be useful,\n"
          "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
          "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n"
          "GNU General Public License for more details.\n\n"
          "Usage: gbc [options] <project file>\n\n"
          "Options:\n"
          "  -g  --debug                add debugging information\n"
          "  -v  --verbose              verbose output\n"
          "  -a  --all                  compile all\n"
          "  -V  --version              display version\n"
          "  -h  --help                 display this help\n"
          "\n"
          );

        exit(0);

      default:
        exit(1);

    }
  }

  if (optind == argc)
  {
    fprintf(stderr, "gbc: no project file.\n");
    exit(1);
  }

  if (optind < (argc - 1))
  {
    fprintf(stderr, "gbc: too many arguments.\n");
    exit(1);
  }

  /*COMP_project = STR_copy(FILE_cat(argv[optind], "Gambas", NULL));*/
  COMP_project = STR_copy(argv[optind]);

  if (!FILE_exist(COMP_project))
  {
    fprintf(stderr, "gbc: project file not found: %s\n", COMP_project);
    exit(1);
  }
}


PRIVATE void compile_file(char *file)
{
  time_t time_src, time_form, time_output;

  COMPILE_begin(file);

  if (!main_compile_all)
  {
    if (FILE_exist(JOB->output))
    {
      time_src = FILE_get_time(JOB->name);
      time_output = FILE_get_time(JOB->output);

      if (time_src <= time_output)
      {
        if (JOB->form == NULL)
          goto _FIN;

        time_form = FILE_get_time(JOB->form);
        if (time_form <= time_output)
          goto _FIN;
      }
    }
  }

  printf("Compiling %s...\n", JOB->name);

  JOB->debug = main_debug;
  JOB->verbose = main_verbose;

  COMPILE_load();

  FORM_do();
  READ_do();

  #ifdef DEBUG
  TABLE_print(JOB->class->table, TRUE);
  #endif

  HEADER_do();
  TRANS_code();

  #ifdef DEBUG
  TABLE_print(JOB->class->string, FALSE);
  #endif

  OUTPUT_do();

_FIN:
  COMPILE_end();
}


PRIVATE void path_add(char *path)
{
  *((char **)ARRAY_add(&path_list)) = STR_copy(path);
}


PRIVATE void path_init(char *first)
{
  ARRAY_create(&path_list);

  if (*first)
    chdir(first);

  path_add(FILE_get_current_dir());

  path_current = 0;
}


PRIVATE void path_exit(void)
{
  ARRAY_delete(&path_list);
}


PRIVATE long path_count(void)
{
  return ARRAY_count(path_list);
}


int main(int argc, char **argv)
{
  DIR *dir;
  char *path;
  struct dirent *dirent;
  char *file_name;
  char *file;
  struct stat info;
  char *ext;

  get_arguments(argc, argv);

  TRY
  {
    COMPILE_init();

    path_init(FILE_get_dir(COMP_project));

    for(;;)
    {
      if (path_current >= path_count())
        break;

      path = path_list[path_current++];
      dir = opendir(path);

      if (dir == NULL)
      {
        fprintf(stderr, "gbc: Warning: Cannot open dir: %s\n", path);
        goto _NEXT_PATH;
      }

      if (chdir(path) != 0)
      {
        fprintf(stderr, "gbc: Warning: Cannot change dir: %s\n", path);
        goto _NEXT_PATH;
      }

      while ((dirent = readdir(dir)) != NULL)
      {
        file_name = dirent->d_name;
        if (*file_name == '.')
          continue;

        file = FILE_cat(path, file_name, NULL);

        if (stat(file_name, &info))
        {
          fprintf(stderr, "gbc: Warning: Cannot stat file: %s\n", file);
          continue;
        }

        if (S_ISDIR(info.st_mode))
          path_add(file);
        else
        {
          ext = FILE_get_ext(file);

          if ((strcasecmp(FILE_get_ext(file), ".module") == 0)
              || (strcasecmp(FILE_get_ext(file), ".class") == 0))
            compile_file(file);
        }
      }

_NEXT_PATH:
      if (dir != NULL) closedir(dir);
      MEM_free(&path);
    }

    path_exit();

    COMPILE_exit();
    MEM_check();
    printf("OK\n");
  }
  CATCH
  {
    fflush(NULL);
    if (JOB->line)
      fprintf(stderr, "%s:%ld: ", JOB->name, JOB->line); /*, (long)(JOB->current - JOB->pattern));*/
    ERROR_print();
  }

  return 0;
}

