/***************************************************************************

  header.c

  Analyzing class description

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define _TRANS_HEADER_C

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "common.h"
#include "error.h"
#include "str.h"
#include "file.h"

#include "compile.h"
#include "header.h"

/*#define DEBUG*/


PRIVATE char *get_num_desc(int num)
{
  static char *num_desc[3] = { "first", "second", "third" };
  static char desc[6];

  if (num < 1)
    return NULL;

  if (num < 4)
    return num_desc[num - 1];

  sprintf(desc, "%dth", num);
  return desc;
}


PRIVATE void analyze_function_desc(TRANS_FUNC *func, int flag)
{
  PATTERN *look = JOB->current;
  TRANS_PARAM *param;
  boolean is_output;
  boolean is_optional = FALSE;
  TRANS_DECL ttyp;

  if (!PATTERN_is_identifier(*look))
    THROW("Syntax error. Invalid identifier in function name");

  func->index = PATTERN_INDEX(*look);
  look++;

  if (flag & HF_EVENT)
    TABLE_copy_symbol_with_prefix(JOB->class->table, func->index, ':', NULL, &func->index);

  if (!PATTERN_is(*look, RS_LBRA))
    THROW("Syntax error. Missing '('");
  look++;

  func->nparam = 0;

  for(;;)
  {
    param = &func->param[func->nparam];

    if (PATTERN_is(*look, RS_RBRA))
    {
      look++;
      break;
    }

    if (func->nparam > 0)
    {
      if (!PATTERN_is(*look, RS_COMMA))
        THROW("Syntax error. Missing ','");
      look++;
    }

    if (PATTERN_is(*look, RS_3PTS))
    {
      param->type = TYPE_make(T_NULL, 0, 0);
      look++;
      if (!PATTERN_is(*look, RS_RBRA))
        THROW("Syntax error. '...' must be the last parameter", get_num_desc(func->nparam + 1));
      look++;
      break;
    }

    is_output = FALSE;

    /* Pas de paramtre en sortie pour l'instant !
    if (PATTERN_is(*look, RS_AT))
    {
      is_output = TRUE;
      look++;
    }
    */

    if (!(flag & HF_NO_OPT))
    {
      if (PATTERN_is(*look, RS_OPTIONAL))
      {
        look++;
        is_optional = TRUE;
      }
    }

    if (!PATTERN_IS_IDENTIFIER(*look))
      THROW("Syntax error. The %s parameter is not a valid identifier", get_num_desc(func->nparam + 1));

    param->index = PATTERN_INDEX(*look);
    look++;
    JOB->current = look;

    if (!TRANS_type(TT_NOTHING, &ttyp))
      THROW("Syntax error. Invalid type description of %s parameter", get_num_desc(func->nparam + 1));

    param->type = ttyp.type;

    /*
    if (is_output)
      TYPE_set_flag(&param->type, TF_OUTPUT);
    */

    look = JOB->current;

    if (is_optional)
    {
      param->optional = look;
      for(;;)
      {
        if (PATTERN_is(*look, RS_COMMA) || PATTERN_is(*look, RS_RBRA) || PATTERN_is_newline(*look))
          break;
        look++;
      }
      JOB->current = look;
    }

    func->nparam++;
  }

  JOB->current = look;
}


PRIVATE void header_module_type(void)
{
  char *ext;

  /*JOB->class->name = STR_copy(FILE_get_name(JOB->name));*/

  ext = FILE_get_ext(JOB->name);

  if (strcasecmp(ext, ".module") == 0)
  {
    JOB->is_module = TRUE;
    JOB->is_form = FALSE;
  }
  else if (strcasecmp(ext, ".class") == 0)
  {
    JOB->is_module = FALSE;
    JOB->is_form = FALSE;
  }
  else if (strcasecmp(ext, ".form") == 0)
  {
    JOB->is_module = FALSE;
    JOB->is_form = TRUE;
  }
  else
    THROW("Unknown file extension");

  JOB->declared = TRUE;
}


PRIVATE boolean header_event(TRANS_EVENT *event)
{
  PATTERN *look = JOB->current;
  TRANS_DECL ttyp;

  if (!PATTERN_is(*look, RS_EVENT))
    return FALSE;

  Clear(event);

  if (JOB->is_module)
    THROW("A module cannot raise events");

  JOB->current++;
  analyze_function_desc((TRANS_FUNC *)event, HF_NORMAL);

  if (PATTERN_is(*JOB->current, RS_AS))
  {
    if (!TRANS_type(TT_CAN_SQUARE, &ttyp))
      THROW("Syntax error in return type");
    event->type = ttyp.type;
  }

  TYPE_set_kind(&event->type, TK_EVENT);
  TYPE_set_flag(&event->type, TF_PUBLIC);

  return TRUE;
}



PRIVATE boolean header_extern(TRANS_EXTERN *trans)
{
  PATTERN *look = JOB->current;
  TRANS_DECL ttyp;

  Clear(trans);

  if (!PATTERN_is(*look, RS_EXTERN))
    return FALSE;

  JOB->current++;
  analyze_function_desc((TRANS_FUNC *)trans, HF_NO_3PTS);

  if (PATTERN_is(*JOB->current, RS_AS))
  {
    if (!TRANS_type(TT_CAN_SQUARE, &ttyp))
      THROW("Syntax error in return type");
    trans->type = ttyp.type;
  }

  if (!PATTERN_is(*JOB->current, RS_IN))
    THROW("IN missing");
  JOB->current++;

  if (!PATTERN_is_string(*JOB->current))
    THROW("Syntax error in library name");

  trans->library = PATTERN_index(*JOB->current);
  JOB->current++;

  TYPE_set_kind(&trans->type, TK_EXTERN);

  return TRUE;
}


PRIVATE boolean header_class(TRANS_DECL *decl)
{
  if (!PATTERN_is(*JOB->current, RS_CLASS))
    return FALSE;

  JOB->current++;

  if (!PATTERN_is_identifier(*JOB->current))
    THROW("Syntax error. CLASS needs an identifier");

  decl->index = PATTERN_index(*JOB->current);
  JOB->current++;

  return TRUE;
}


PRIVATE boolean header_declaration(TRANS_DECL *decl)
{
  PATTERN *look = JOB->current;
  boolean is_static = FALSE;
  boolean is_public = FALSE;
  boolean is_const = FALSE;

  Clear(decl);

  /* static ! */

  if (JOB->is_module)
  {
    is_static = TRUE;
  }
  else if (PATTERN_is(*look, RS_STATIC))
  {
    is_static = TRUE;
    look++;
  }

  /* public ou private ? */

  is_public = JOB->is_module;

  if (PATTERN_is(*look, RS_PUBLIC))
  {
    is_public = TRUE;
    look++;
  }
  else if (PATTERN_is(*look, RS_PRIVATE))
  {
    is_public = FALSE;
    look++;
  }

  /* const ? */

  is_const = FALSE;

  if (PATTERN_is(*look, RS_CONST))
  {
    /* static const <=> const */

    /*
    if (is_static)
      return FALSE;
    */

    is_const = TRUE;
    look++;
  }

  if (!PATTERN_IS_IDENTIFIER(*look))
    return FALSE;

  decl->index = PATTERN_INDEX(*look);
  look++;

  JOB->current = look;

  if (!TRANS_type(((!is_const && !is_public) ? (TT_CAN_SQUARE  | TT_CAN_ARRAY) : 0) | TT_CAN_NEW, decl))
    return FALSE;

  if (is_static) TYPE_set_flag(&decl->type, TF_STATIC);
  if (is_public) TYPE_set_flag(&decl->type, TF_PUBLIC);
  if (is_const)
    TYPE_set_kind(&decl->type, TK_CONST);
  else
    TYPE_set_kind(&decl->type, TK_VARIABLE);

  look = JOB->current;

  if (is_const)
  {
    if (!PATTERN_is(*look, RS_EQUAL))
      THROW("Syntax error. Missing '='");
    look++;

    /*
    if ((!PATTERN_IS_STRING(*look)) && (!PATTERN_IS_NUMBER(*look)))
      THROW("Syntax error in constant declaration");
    */

    TRANS_get_constant_value(decl, *look);
    look++;
  }

  JOB->current = look;

  return TRUE;
}


PRIVATE boolean header_function(TRANS_FUNC *func)
{
  PATTERN *look = JOB->current;
  PATTERN pat;
  TRANS_DECL ttyp;

  boolean is_proc = FALSE;
  boolean is_static = FALSE;
  boolean is_public = FALSE;

  Clear(func);

  /* static ? */

  if (JOB->is_module)
  {
    is_static = TRUE;
  }
  else if (PATTERN_is(*look, RS_STATIC))
  {
    is_static = TRUE;
    look++;
  }

  /* public ou private ? */

  is_public = JOB->is_module;

  if (PATTERN_is(*look, RS_PUBLIC))
  {
    is_public = TRUE;
    look++;
  }
  else if (PATTERN_is(*look, RS_PRIVATE))
  {
    is_public = FALSE;
    look++;
  }

  if (PATTERN_is(*look, RS_PROCEDURE) || PATTERN_IS(*look, RS_SUB))
    is_proc = TRUE;
  else if (!PATTERN_is(*look, RS_FUNCTION))
    return FALSE;
  look++;

  JOB->current = look;
  analyze_function_desc(func, HF_NORMAL);

  if (!is_proc)
  {
    if (!TRANS_type(TT_CAN_SQUARE, &ttyp))
      THROW("Syntax error. Invalid return type");
    func->type = ttyp.type;
  }
  TYPE_set_kind(&func->type, TK_FUNCTION);
  if (is_static) TYPE_set_flag(&func->type, TF_STATIC);
  if (is_public) TYPE_set_flag(&func->type, TF_PUBLIC);

  /* on saute le corps de la fonction */

  if (!PATTERN_IS_NEWLINE(*(JOB->current)))
    THROW("Syntax error after function declaration");

  func->line = PATTERN_INDEX(*(JOB->current)) + 1; /* a commence  la ligne suivante ! */
  func->start = JOB->current + 1;

  for(;;)
  {
    pat = *JOB->current;
    if (PATTERN_IS_END(pat))
    {
      THROW("Missing END");
    }

    if (PATTERN_IS_NEWLINE(pat))
      if (PATTERN_is(JOB->current[1], RS_END))
      {
        if (PATTERN_is(JOB->current[2], (is_proc ? RS_PROCEDURE : RS_FUNCTION)))
        {
          JOB->current += 3;
          break;
        }
        else if (PATTERN_is_newline(JOB->current[2]))
        {
          JOB->current += 2;
          break;
        }
      }

    JOB->current++;
  }

  return TRUE;
}


PRIVATE boolean header_inherits(void)
{
  long index;

  if (!PATTERN_is(*JOB->current, RS_INHERITS))
    return FALSE;

  JOB->current++;

  if (!PATTERN_is_identifier(*JOB->current))
    THROW("Syntax error. INHERITS needs an identifier");

  index = PATTERN_index(*JOB->current);

  JOB->class->parent = CLASS_add_class(JOB->class, index);
  /*printf("JOB->class->parent = %d\n", JOB->class->parent);*/

  JOB->current++;
  return TRUE;
}


PUBLIC void HEADER_do(void)
{
  TRANS_DECL decl;
  TRANS_FUNC func;
  TRANS_EVENT event;
  TRANS_EXTERN ext;

  TRANS_reset();

  /*
  while (JOB->current < JOB->end)
  {
    if PATTERN_IS_END(*JOB->current)
      break;

    if (TRANS_newline())
      continue;

    if (header_module_type())
      break;

    THROW("Module declaration expected");
  }
  */

  header_module_type();

  while (JOB->current < JOB->end)
  {
    if PATTERN_IS_END(*JOB->current)
      break;

    if (TRANS_newline())
      continue;

    if (header_function(&func))
    {
      CLASS_add_function(JOB->class, &func);
      continue;
    }

    if (header_event(&event))
    {
      CLASS_add_event(JOB->class, &event);
      continue;
    }

    if (header_extern(&ext))
    {
      CLASS_add_extern(JOB->class, &ext);
      continue;
    }

    if (header_declaration(&decl))
    {
      CLASS_add_declaration(JOB->class, &decl);
      continue;
    }

    if (header_class(&decl))
    {
      CLASS_add_class(JOB->class, decl.index);
      continue;
    }

    if (header_inherits())
      continue;

    THROW(E_SYNTAX);
  }

  if (JOB->verbose)
    CLASS_dump();
}
