/*

  memory.c

  Allocations de mmoire

*/

#define _MEMORY_C

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "common.h"
#include "error.h"
#include "memory.h"


#define MAGIC 0x44554B45L

/*#define DEBUG*/


PRIVATE long count = 0;


PUBLIC void MEM_alloc(void *p_ptr, size_t size)
{
  char **ptr = (char **)p_ptr;
  char *alloc;

  alloc = malloc(size + 4);
  if (alloc)
  {
    *((long *)alloc) = MAGIC;
    *ptr  = &alloc[4];
    memset(*ptr, 0, size);
    count++;
  }
  else
  {
    *ptr = NULL;
   THROW(E_MEMORY);
  }
  
  #ifdef DEBUG
  printf("---- malloc(%d) -> 0x%08lX\n", size, (long)*ptr);
  fflush(stdout);
  #endif
}

PUBLIC void MEM_realloc(void *p_ptr, size_t size)
{
  char **ptr = (char **)p_ptr;
  char *alloc;
#ifdef DEBUG
  char *old = *ptr;
#endif

  if (!*ptr) goto BAD;

  alloc = *ptr - 4;
  if (*((long *)alloc) != MAGIC) goto BAD;

  alloc = realloc(alloc, size + 4);
  if (!*ptr)
    THROW(E_MEMORY);
  else
    *ptr = &alloc[4];

#ifdef DEBUG
  printf("---- realloc(0x%08lX, %d) -> 0x%08lX\n", (long)old, size, (long)*ptr);
  fflush(stdout);
#endif

  return;

BAD:

  THROW("MEM_realloc: Bad pointer 0x%08X", (long)(*ptr));

}

PUBLIC void MEM_free(void *p_ptr)
{
  char **ptr = (char **)p_ptr;
  char *alloc;
  
  if (!*ptr) goto BAD;
  
  alloc = *ptr - 4;
  if (*((long *)alloc) != MAGIC) goto BAD;

  *((long *)alloc) = 0x23232323;

  #ifdef DEBUG
  printf("---- free(0x%08lX)\n", (long)*ptr);
  fflush(stdout);
  #endif

  free(alloc);

  *ptr = NULL;
  count--;

  return;

BAD:

  THROW("MEM_free: Bad pointer 0x%08X", (long)(*ptr));
}

PUBLIC void MEM_check()
{
  if (count)
    THROW("%d allocations non libres.", count);
}
