/***************************************************************************

  str.c

  Common string management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __STR_C

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdarg.h>

#include "common.h"
#include "error.h"
#include "limit.h"
#include "memory.h"

#include "str.h"

PUBLIC char *STR_add(char *d, char *s)
{
  for(;;)
  {
    if ((*d = *s) == 0)
      break;

    d++;
    s++;
  }

  return d;
}

PUBLIC char *STR_copy(char *str)
{
  char *cpy;

  MEM_alloc(&cpy, strlen(str) + 1);
  strcpy(cpy, str);
  return cpy;
}

PUBLIC void STR_free(char *str)
{
  if (str)
    MEM_free(&str);
}


PUBLIC char *STR_cat(char *str, ...)
{
  va_list args;
  char *cpy, *p;
  int len = 0;

  va_start(args, str);

  p = str;
  while (p)
  {
    len += strlen(p);
    p = va_arg(args, char *);
  }

  MEM_alloc(&cpy, len + 1);
  p = cpy;

  va_start(args, str);

  while (str)
  {
    p = STR_add(p, str);
    str = va_arg(args, char *);
  }

  return cpy;
}


