/*

  symbol.c
  
  Gestion de table des symboles
  
*/


#define _TABLE_C

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "common.h"
#include "error.h"
#include "memory.h"
#include "limit.h"

#include "table.h"
#include "class.h"


#define SYM(table, ind) (TABLE_get_symbol(table, ind))

PRIVATE int compare(char *s1, long len1, char *s2, long len2)
{
  long i;
  long len = (len1 < len2) ? len1 : len2;
  
  for (i = 0; i < len; i++)
  {
    if (*s1 > *s2) return 1;
    if (*s1 < *s2) return -1;
    s1++; s2++;
  }

  if (len1 < len2)
    return -1;
  else if (len1 > len2)
    return 1;
  else
    return 0;  
}


PRIVATE int compare_ignore_case(char *s1, long len1, char *s2, long len2)
{
  long i;
  long len = (len1 < len2) ? len1 : len2;
  char c1;
  char c2;

  for (i = 0; i < len; i++)
  {
    c1 = toupper(*s1);
    c2 = toupper(*s2);
    
    if (c1 > c2) return 1;
    if (c1 < c2) return -1;
    s1++; s2++;
  }
  
  if (len1 < len2)
    return -1;
  else if (len1 > len2)
    return 1;
  else
    return 0;  
}


/* ATTENTION: index ne correspond pas  l'index dans la table, mais au numro d'ordre ! */

PRIVATE boolean search(TABLE *table, char *name, int len, long *index)
{
  long pos, new_pos,deb, fin;
  int cmp;
  int (*cmp_func)(char *, long, char *, long);
  SYMBOL *sym;
  
  cmp_func = ((table->flag == TF_IGNORE_CASE) ? compare_ignore_case : compare);
  
  pos = 0;
  deb = 0;
  fin = ARRAY_count(table->symbol);
  
  for(;;)
  {
    if (deb >= fin)
    {
      *index = deb;
      return FALSE;
    }
  
    new_pos = deb + (fin - deb) / 2;
    
    pos = new_pos;
    sym = SYM(table, SYM(table, pos)->sort); /*&table->symbol[table->symbol[pos].sort];*/
    cmp = (*cmp_func)(name, len, sym->name, sym->len);
    
    if (cmp == 0) 
    {
      *index = pos;
      return TRUE;
    }

    if (cmp < 0)
      fin = pos;
    else
      deb = pos + 1;
  }
}


PUBLIC char *TABLE_get_symbol_name(TABLE *table, long index)
{
  static char buffer[MAX_SYMBOL_LEN + 1];
  SYMBOL *sym;

  if ((index < 0) || (index >= ARRAY_count(table->symbol)))
    return "(?)";

  sym = SYM(table, index);
  memcpy(buffer, sym->name, sym->len);
  buffer[sym->len] = 0;

  return buffer;
}


PUBLIC void TABLE_create(TABLE **result, size_t size, TABLE_FLAG flag)
{
  TABLE *table;
  
  MEM_alloc(&table, sizeof(TABLE));
  
  ARRAY_create_with_size(&table->symbol, Max(size, sizeof(SYMBOL)), 64);
  
  table->flag = flag;

  *result = table;
}


PUBLIC void TABLE_create_from(TABLE **result, size_t size, char *sym_list[], TABLE_FLAG flag)
{
  TABLE *table;
  
  TABLE_create(&table, size, flag);
  
  while (*sym_list)
  {
    TABLE_add_symbol(table, *sym_list, strlen(*sym_list), NULL, NULL);
    sym_list++;
  }
  
  *result = table;
}


PUBLIC void TABLE_delete(TABLE **table)
{
  if (*table)
  {
    ARRAY_delete(&((*table)->symbol));
    MEM_free(table);
  }
}


PUBLIC long TABLE_count(TABLE *table)
{
  return ARRAY_count(table->symbol);
}


PUBLIC boolean TABLE_find_symbol(TABLE *table, char *name, int len, SYMBOL **symbol, long *index)
{
  long ind;
  boolean result;
  SYMBOL *sym;

  result = search(table, name, len, &ind);

  if (result)
  {
    sym = SYM(table, ind);
    ind = sym->sort;
    if (symbol) *symbol = SYM(table, ind);
    if (index) *index = ind;
  }

  return result;
}


PUBLIC boolean TABLE_add_symbol(TABLE *table, char *name, int len, SYMBOL **symbol, long *index)
{
  long ind;
  long i;
  boolean result;
  SYMBOL *sym;
  long count;

  /*
  pos_name = BUFFER_add_string(table->buffer, name, len);
  if (pos_name < 0) 
    THROW("TABLE_add_symbol", E_MEMORY);
  */

  len = Min(len, MAX_SYMBOL_LEN);
  
  result = search(table, name, len, &ind);
  
  if (!result)
  {
    count = ARRAY_count(table->symbol);
  
    sym = (SYMBOL *)ARRAY_add_void(&table->symbol);
    
    sym->name = name;
    sym->len = len;
    
    /*
    printf("TABLE_add_symbol: %.*s %d %d\n", len, name, ((CLASS_SYMBOL *)sym)->global.type, 
      ((CLASS_SYMBOL *)sym)->local.type);
    */
      
    for (i = count; i > ind; i--)
    {
      SYM(table, i)->sort = SYM(table, i - 1)->sort;
      /*table->symbol[i].sort = table->symbol[i - 1].sort;*/
    }

    /*table->symbol[ind].sort = count;*/
    SYM(table, ind)->sort = count;
    ind = count;
  }
  else
    ind = SYM(table, ind)->sort; /*table->symbol[ind].sort;*/

  if (symbol) *symbol = SYM(table, ind); /*&table->symbol[ind];*/
  if (index) *index = ind;
  
  return result;
}


PUBLIC void TABLE_copy_symbol_with_prefix(TABLE *table, long ind_src, char prefix, SYMBOL **symbol, long *index)
{
  SYMBOL *sym;
  char *ptr;
  
  sym = TABLE_get_symbol(table, ind_src);
  
  ptr = sym->name - 1;
  
  if (!isspace(*ptr))
    ERROR_panic("Cannot add prefix to symbol");
  
  *ptr = prefix;
  
  TABLE_add_symbol(table, ptr, sym->len + 1, symbol, index);
}



/*
PUBLIC boolean TABLE_copy_symbol(TABLE *dst, TABLE *src, long index_src, SYMBOL **symbol, long *index)
{
  boolean result;
  SYMBOL *sym = &src->symbol[index_src];
  
  result = TABLE_add_symbol(dst, sym->name, sym->len, symbol, index);
  
  (*symbol)->value = sym->value;
  
  return result;
}
*/

PUBLIC void TABLE_sort(TABLE *table)
{
}


PUBLIC void TABLE_print(TABLE *table, boolean sort)
{
  int i;
  SYMBOL *sym;

  printf("capacit: %li\n", ARRAY_count(table->symbol));

  /*  
  for (i = 0; i < ARRAY_count(table->symbol); i++)
  {
    sym = SYM(table, i);
    printf("%*s (%li) ", (int)sym->len, sym->name, sym->sort);
  }

  printf("\n");
  */
  
  for (i = 0; i < ARRAY_count(table->symbol); i++)
  {
    if (sort)
      sym = SYM(table, SYM(table, i)->sort);
    else
      sym = SYM(table, i);
      
    if ((i > 0) && (!(i & 0xF))) 
      printf("\n");
    printf("%.*s ", (int)sym->len, sym->name);
  }

  printf("\n\n");
  
}


PUBLIC SYMBOL *TABLE_get_symbol_sort(TABLE *table, long index)
{
  SYMBOL *sym = TABLE_get_symbol(table, index);

  return TABLE_get_symbol(table, sym->sort);
}

