/***************************************************************************

  trans_subr.c

  Subroutine synonymous compiler

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define _TRANS_SUBR_C

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "common.h"
#include "error.h"
#include "compile.h"
#include "trans.h"
#include "code.h"
#include "limit.h"

/*#define DEBUG*/


PRIVATE void trans_subr(int subr, int nparam)
{
  static char *name[10] =
    { "Print", "Input", "Write", "Read", "Open",
      "Close", "Seek", "Type", "LineInput", "Flush" };

  static SUBR_INFO *info[10] = { NULL };

  SUBR_INFO *si;

  if (info[subr] == NULL)
    info[subr] = SUBR_get(name[subr]);

  si = info[subr];

  CODE_subr(si->opcode, nparam, si->optype, FALSE, FALSE);
}


PRIVATE boolean trans_stream(int default_stream)
{
  if (PATTERN_is(*JOB->current, RS_SHARP))
  {
    JOB->current++;
    TRANS_expression(FALSE);

    if (PATTERN_is(*JOB->current, RS_COMMA))
    {
      JOB->current++;
      if (PATTERN_is_newline(*JOB->current))
        THROW(E_SYNTAX);
    }

    return FALSE;
  }
  else
  {
    CODE_push_number(default_stream);
    return TRUE;
  }
}


/*
PRIVATE void trans_calc_type(void)
{
  trans_subr(TS_SUBR_TYPE, 1);
}
*/

PUBLIC void TRANS_print(void)
{
  int nparam = 1;
  boolean semicolon = FALSE;

  trans_stream(TS_STDOUT);

  for(;;)
  {
    if (PATTERN_is_newline(*JOB->current))
      break;

    TRANS_expression(FALSE);
    nparam++;
    semicolon = FALSE;

    if (PATTERN_is_newline(*JOB->current))
      break;

    if (PATTERN_is(*JOB->current, RS_SCOLON))
    {
      semicolon = TRUE;
    }
    else if (PATTERN_is(*JOB->current, RS_COMMA))
    {
      CODE_push_char('\t');
      nparam++;
      semicolon = FALSE;
    }
    else
      THROW(E_SYNTAX);

    JOB->current++;
  }

  if (!semicolon)
  {
    CODE_push_char('\n');
    nparam++;
  }

  trans_subr(TS_SUBR_PRINT, nparam);
  CODE_drop();
}


PUBLIC void TRANS_write(void)
{
  int nparam = 1;

  trans_stream(TS_STDOUT);

  TRANS_expression(FALSE);
  nparam++;

  if (!PATTERN_is_newline(*JOB->current))
  {
    TRANS_want(RS_COMMA);

    TRANS_expression(FALSE);
    nparam++;
  }

  trans_subr(TS_SUBR_WRITE, nparam);
  CODE_drop();
}


PUBLIC void TRANS_input(void)
{
  boolean stream = TRUE;

  trans_stream(TS_STDIN);

  for(;;)
  {
    trans_subr(TS_SUBR_INPUT, (stream ? 1 : 0));
    stream = FALSE;

    TRANS_reference();

    if (PATTERN_is_newline(*JOB->current))
      break;

    if (!PATTERN_is(*JOB->current, RS_COMMA)
        && !PATTERN_is(*JOB->current, RS_SCOLON))
      THROW(E_SYNTAX);

    JOB->current++;
  }
}


PUBLIC void TRANS_read(void)
{
  int nparam = 2;
  PATTERN *save_return;
  PATTERN *save_current;

  trans_stream(TS_STDIN);

  save_return = JOB->current;

  TRANS_expression(FALSE);
  /*trans_calc_type();*/

  if (!PATTERN_is_newline(*JOB->current))
  {
    TRANS_want(RS_COMMA);

    TRANS_expression(FALSE);
    nparam++;
  }

  trans_subr(TS_SUBR_READ, nparam);

  save_current = JOB->current;
  JOB->current = save_return;
  TRANS_reference();

  JOB->current = save_current;
}


PUBLIC void TRANS_open(void)
{
  int mode = TS_MODE_READ;

  /* Nom du fichier */

  TRANS_expression(FALSE);

  /* mode d'ouverture */

  if (PATTERN_is(*JOB->current, RS_FOR))
  {
    JOB->current++;

    if (PATTERN_is(*JOB->current, RS_READ))
      mode = TS_MODE_READ;
    else if (PATTERN_is(*JOB->current, RS_WRITE))
      mode = TS_MODE_WRITE;
    else if (PATTERN_is(*JOB->current, RS_CREATE))
      mode = TS_MODE_CREATE;
    else if (PATTERN_is(*JOB->current, RS_APPEND))
      mode = TS_MODE_APPEND;
    else
      JOB->current--;

    JOB->current++;

    if (PATTERN_is(*JOB->current, RS_DIRECT))
    {
      mode |= TS_MODE_DIRECT;
      JOB->current++;
    }

    if (PATTERN_is(*JOB->current, RS_BINARY))
    {
      mode |= TS_MODE_BINARY;
      JOB->current++;
    }

    JOB->current--;
    if (PATTERN_is(*JOB->current, RS_FOR))
      THROW("Syntax error in file open mode");
    JOB->current++;
  }

  CODE_push_number(mode);

  trans_subr(TS_SUBR_OPEN, 2);

  /* handle du fichier */

  TRANS_want(RS_AS);
  TRANS_ignore(RS_SHARP);

  TRANS_reference();

}


PUBLIC void TRANS_close(void)
{
  if (PATTERN_is_newline(*JOB->current))
    THROW(E_SYNTAX);

  TRANS_ignore(RS_SHARP);
  TRANS_expression(FALSE);

  trans_subr(TS_SUBR_CLOSE, 1);
  CODE_drop();
}


PUBLIC void TRANS_seek(void)
{
  int nparam;

  if (trans_stream(TS_STDIN))
    THROW("Syntax error. '#' expected");

  TRANS_expression(FALSE);
  nparam = 2;

  if (PATTERN_is(*JOB->current, RS_COMMA))
  {
    JOB->current++;
    TRANS_expression(FALSE);
    nparam++;
  }

  trans_subr(TS_SUBR_SEEK, nparam);
}


PUBLIC void TRANS_line_input(void)
{
  trans_stream(TS_STDIN);
  trans_subr(TS_SUBR_LINE_INPUT, 1);
  TRANS_reference();
}


PUBLIC void TRANS_flush(void)
{
  trans_stream(TS_STDOUT);
  trans_subr(TS_SUBR_FLUSH, 1);
  CODE_drop();
}
