/***************************************************************************

  type.h

  Datatypes definitions and management

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __TYPE_H
#define __TYPE_H

enum {
  T_VOID         = 0,
  T_BOOLEAN      = 1,
  T_BYTE         = 2,
  T_SHORT        = 3,
  T_INTEGER      = 4,
  T_LONG         = 5,
  T_FLOAT        = 6,
  T_DATE         = 7,
  T_STRING       = 8,
  T_CSTRING      = 9,
  T_VARIANT      = 10,
  T_ARRAY        = 11,
  T_STRUCT       = 12,
  T_FUNCTION     = 13,
  T_CLASS        = 14,
  T_NULL         = 15,
  T_OBJECT       = 16
  };

/*
enum {
  T_VOID         = 0,
  T_BOOLEAN      = 1,
  T_BYTE         = 2,
  T_SHORT        = 3,
  T_INTEGER      = 4,
  T_LONG         = 5,
  T_FLOAT        = 6,
  T_DATE         = 7,
  T_STRING       = 8,
  T_FUNCTION     = 10,
  T_VARIANT      = 11,
  T_ARRAY        = 12,
  T_OBJECT       = 15,
  T_OUTPUT       = 128
  };
*/

enum {
  TK_UNKNOWN     = 0,   /* symbole externe */
  TK_VARIABLE    = 1,   /* variable */
  TK_FUNCTION    = 2,   /* fonction */
  TK_PROPERTY    = 3,   /* proprit */
  TK_LABEL       = 4,   /* tiquette pour un GOTO */
  TK_EVENT       = 5,   /* vnement */
  TK_EXTERN      = 6,   /* fonction d'une librairie dynamique */
  TK_CONST       = 7,   /* constante */

  /*TF_OUTPUT      = 8,*/   /* flags pour un paramtre de fonction */
  TF_OPTIONAL    = 16,

  TF_STATIC      = 32,
  TF_PUBLIC      = 64,
  };

/***************************************************

  Format d'un Type

  F K T T X X X X

  F : Flags (TF_*)
  K : Kind (TK_*)
  T : Type (T_*)
  X : index, pour T_OBJECT, T_ARRAY, T_STRUCT

***************************************************/

/*
typedef
  ulong TYPE;
*/

typedef
  unsigned char TYPE_ID;

typedef
  struct {
    unsigned char flag;
    TYPE_ID id;
    short value;
    }
  PACKED
  TYPE;

typedef
  struct {
    long value;
    TYPE type;
    }
  VALUE;

/*#define TYPE_is_const(type)      (((type) >> 24) & TF_CONST)*/
#define TYPE_is_static(type)     ((type).flag & TF_STATIC)
#define TYPE_is_public(type)     ((type).flag & TF_PUBLIC)

#define TYPE_is_optional(type)   ((type).flag & TF_OPTIONAL)
/*#define TYPE_is_output(type)     (((type) >> 24) & TF_OUTPUT)*/

#define TYPE_is_array(type)      (TYPE_get_id(type) == T_ARRAY)
#define TYPE_is_object(type)     ((TYPE_get_id(type) == T_OBJECT) && (TYPE_get_value(type) >= 0))

#define TYPE_get_value(type)     ((type).value)
#define TYPE_get_kind(type)      ((type).flag & 0x7)
#define TYPE_get_id(type)        ((type).id)
#define TYPE_is_null(type)       (*((long *)&(type)) == 0)

#define TYPE_set_value(type, _value)     ((type)->value = (_value))
#define TYPE_set_id(type, _id)           ((type)->id = (_id))
#define TYPE_set_kind(type, _kind)       ((type)->flag |= (_kind))
#define TYPE_set_flag(type, _flag)       ((type)->flag |= (_flag))
#define TYPE_clear(type)                 (*((long *)(type)) = 0)

#define TYPE_long(type)         (*((long *)&(type)))

#define TYPE_can_be_long(type)  (TYPE_get_id(type) <= T_LONG)

/*PUBLIC long TYPE_get_class(TYPE type);*/
PUBLIC TYPE TYPE_make(TYPE_ID id, short value, int flag);
PUBLIC char *TYPE_get_desc(TYPE type);
PUBLIC long TYPE_sizeof(TYPE type);

#endif


