/***************************************************************************

  CApp.c

  The native class App

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CAPP_C

#include "common.h"
#include "gambas_api.h"
#include "gambas.h"
#include "class.h"
#include "project.h"
#include "local.h"

#include "Object.h"

#include "CApp.h"

BEGIN_PROPERTY(app_path)

  GB_ReturnZeroString(PROJECT_path);

END_PROPERTY


BEGIN_PROPERTY(app_home)

  GB_ReturnCopyString(getenv("HOME"), 0);

END_PROPERTY


BEGIN_PROPERTY(app_language)

  if (READ_PROPERTY)
    GB_ReturnString(LOCAL_get_lang(), 0);
  else
    LOCAL_set_lang(GB_ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY


BEGIN_PROPERTY(app_args)

  static OBJECT args;

  args.class = CLASS_AppArgs;

  GB_ReturnObject(&args);

END_PROPERTY


BEGIN_PROPERTY(app_args_count)

  GB_ReturnInt(PROJECT_argc);

END_PROPERTY


BEGIN_METHOD(app_args_get, long index)

  long index = PARAM(index);

  if (index < 0 || index >= PROJECT_argc)
    THROW(E_ARG);

  GB_ReturnString(PROJECT_argv[index], 0);

END_METHOD


PUBLIC GB_DESC NATIVE_AppArgs[] =
{
  GB_DECLARE(".App.Args", 0), GB_VIRTUAL_CLASS(),

  GB_PROPERTY_READ("Count", "i", app_args_count),
  GB_METHOD("_get", "s", app_args_get, "'Index'i"),

  GB_END_DECLARE
};


PUBLIC GB_DESC NATIVE_App[] =
{
  GB_DECLARE("App", 0), GB_VIRTUAL_CLASS(),

  GB_STATIC_PROPERTY_READ("Args", ".App.Args", app_args),
  GB_STATIC_PROPERTY("Language", "s", app_language),
  GB_STATIC_PROPERTY_READ("Path", "s", app_path),
  GB_STATIC_PROPERTY_READ("Home", "s", app_home),

  GB_END_DECLARE
};

/*!---------

\HELP
c-app
class

\IN
native

\TITLE
App

\TYPE
Static

\DESC:fr
Cette classe statique permet de rcuprer des informations sur l'application
en cours d'excution : emplacement du programme, ligne de commande, langue, etc.

----------*/




