/***************************************************************************

  CClass.c

  The native class Class needed for introspection

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CCLASS_C

#include <ctype.h>

#include "common.h"
#include "gambas_api.h"
#include "gambas.h"
#include "class.h"
#include "class_desc.h"
#include "exec.h"

#include "Object.h"
#include "CClass.h"

PRIVATE CLASS_DESC *get_desc(CLASS *class, char *name)
{
  long index;

  index = CLASS_find_symbol(class, name);
  if (index == NO_SYMBOL)
    THROW(E_NSYMBOL, name, class->name);

  return class->table[index].desc;
}


/**************************************************************************************************/

BEGIN_METHOD(class_get, GB_STRING name)

  char *name = GB_ToZeroString(PARAM(name));
  CLASS *class = NULL;

  if (name != NULL)
    class = CLASS_look(name);

  if (class == NULL)
  {
    GB_Error("Unknown class");
    return;
  }

  if (!class->ready)
  {
    GB_Error("class is not loaded");
    return;
  }

  GB_ReturnObject(class);

END_METHOD


BEGIN_METHOD(class_load, GB_STRING name)

  GB_Return(GB_GetReturnType(), CLASS_get(GB_ToZeroString(PARAM(name))));

END_METHOD


BEGIN_PROPERTY(class_name)

  GB_ReturnZeroString(OBJECT(CLASS)->name);

END_PROPERTY


BEGIN_METHOD_VOID(class_next)

  TABLE *table = CLASS_get_table();
  long *index = (long *)GB_GetEnum();
  CLASS *class;

  for(;;)
  {
    if (*index >= TABLE_count(table))
    {
      GB_StopEnum();
      break;
    }

    class = ((CLASS_SYMBOL *)TABLE_get_symbol(table, *index))->class;
    (*index)++;

    if (class->ready)
    {
      GB_ReturnObject(class);
      break;
    }
  }

END_METHOD


BEGIN_PROPERTY(class_symbols)

  GB_ReturnObject(_object);

END_METHOD


BEGIN_PROPERTY(class_count)

  GB_ReturnInt(CLASS_count());

END_PROPERTY


BEGIN_PROPERTY(class_hidden)

  GB_ReturnBoolean(*(OBJECT(CLASS)->name) == '.');

END_PROPERTY


BEGIN_PROPERTY(class_native)

  GB_ReturnBoolean(CLASS_is_native(OBJECT(CLASS)));

END_PROPERTY





/**************************************************************************************************/

BEGIN_PROPERTY(class_symbols_count)

  GB_ReturnInt(OBJECT(CLASS)->n_desc);

END_PROPERTY


BEGIN_METHOD_VOID(class_symbols_next)

  CLASS *class = OBJECT(CLASS);
  long *index = (long *)GB_GetEnum();

/*_HIDDEN:*/

  if (*index >= class->n_desc)
  {
    GB_StopEnum();
    return;
  }

  /*cd = class->table[*index].desc;*/

  /*if (cd->symbol.name[0] == '$' || cd->symbol.name[0] == '@')
    goto _HIDDEN;*/

  GB_ReturnObject(&class->table[class->table[*index].sort].ref);
  (*index)++;

END_METHOD


BEGIN_METHOD(class_symbols_get, GB_STRING name)

  CLASS *class = OBJECT(CLASS);
  CLASS_DESC_SYMBOL *cd = NULL;
  char *name = GB_ToZeroString(PARAM(name));

  if (name != NULL)
    cd = CLASS_get_symbol(class, name);

  if (cd == NULL)
  {
    GB_Error("Unknown symbol");
    return;
  }

  GB_ReturnObject(&cd->ref);

END_METHOD



/**************************************************************************************************/

BEGIN_PROPERTY(symbol_name)

  CLASS_DESC_SYMBOL *cds = SYMBOL_TO_CDS(OBJECT(void));

  GB_ReturnString(cds->name, cds->len);

END_PROPERTY


BEGIN_PROPERTY(symbol_kind)

  CLASS_DESC_SYMBOL *cds = SYMBOL_TO_CDS(OBJECT(void));

  switch(CLASS_DESC_get_type(cds->desc))
  {
    case CD_PROPERTY:
    case CD_PROPERTY_READ:
    case CD_STATIC_PROPERTY:
    case CD_STATIC_PROPERTY_READ:
      GB_ReturnInt(1);
      return;

    case CD_METHOD:
    case CD_STATIC_METHOD:
      GB_ReturnInt(2);
      return;

    case CD_EVENT:
      GB_ReturnInt(3);
      return;

    case CD_CONSTANT:
      GB_ReturnInt(4);
      return;

    default:
      GB_ReturnInt(0);
      return;
  }

END_PROPERTY


BEGIN_PROPERTY(symbol_static)

  CLASS_DESC_SYMBOL *cds = SYMBOL_TO_CDS(OBJECT(void));

  GB_ReturnBoolean(index(CD_STATIC_LIST, CLASS_DESC_get_type(cds->desc)) != NULL);

END_PROPERTY


BEGIN_PROPERTY(symbol_hidden)

  CLASS_DESC_SYMBOL *cds = SYMBOL_TO_CDS(OBJECT(void));

  GB_ReturnBoolean(*cds->name == '_');

END_PROPERTY


BEGIN_PROPERTY(symbol_read_only)

  CLASS_DESC_SYMBOL *cd = SYMBOL_TO_CDS(OBJECT(void));

  switch (CLASS_DESC_get_type(cd->desc))
  {
    case CD_PROPERTY:
    case CD_STATIC_PROPERTY:
      GB_ReturnBoolean(FALSE);
      break;

    default:
      GB_ReturnBoolean(TRUE);
      break;
  }

END_PROPERTY


BEGIN_PROPERTY(symbol_type)

  CLASS_DESC_SYMBOL *cd = SYMBOL_TO_CDS(OBJECT(void));

  GB_ReturnString(TYPE_to_string(cd->desc->property.type), 0); /* Valable pour tout symbole */

END_PROPERTY


BEGIN_PROPERTY(symbol_signature)

  CLASS_DESC_SYMBOL *cd = SYMBOL_TO_CDS(OBJECT(void));

  GB_ReturnZeroString(CLASS_DESC_get_signature(cd->desc));

END_METHOD


BEGIN_PROPERTY(symbol_value)

  CLASS_DESC *desc = SYMBOL_TO_CDS(OBJECT(void))->desc;

  if (CLASS_DESC_get_type(desc) != CD_CONSTANT)
  {
    GB_ReturnNull();
    return;
  }

  GB_ReturnPtr(desc->constant.type, (void *)&desc->constant.value);

END_PROPERTY




/**************************************************************************************************/


BEGIN_METHOD(object_get_property, void *object; GB_STRING property)

  char *name;
  CLASS_DESC *desc;
  CLASS *class;
  char type;
  void *object = PARAM(object);

  if (object == NULL)
    THROW(E_NULL);

  class = OBJECT_class(object);
  name = GB_ToZeroString(PARAM(property));
  desc = get_desc(class, name);
  type = CLASS_DESC_get_type(desc);

  if (type == CD_PROPERTY || type == CD_PROPERTY_READ)
  {
    if (desc->property.exec == NULL)
    {
      GB_ReturnPtr(desc->property.type, OBJECT_get_prop_addr(object, desc));
    }
    else
    {
      if (EXEC_call_native(desc->property.exec, object, NULL, desc->property.type))
        THROW_LIBRARY();

      /* Le rsultat est dj dans TEMP ! C'est magique !!
         Il faut juste le convertir en Variant au cas o */

      VALUE_conv(&TEMP, T_VARIANT);
    }
  }
  else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_PROPERTY_READ)
    THROW(E_STATIC, class->name, name);
  else
    THROW(E_NPROPERTY, class->name, name);

END_METHOD


BEGIN_METHOD(object_set_property, void *object; GB_STRING property; GB_VARIANT value)

  char *name;
  CLASS_DESC *desc;
  CLASS *class;
  char type;
  void *object = PARAM(object);
  VALUE value, temp;

  if (object == NULL)
    THROW(E_NULL);

  VALUE_read(&value, &(PARAM(value)), T_VARIANT);

  class = OBJECT_class(object);
  name = GB_ToZeroString(PARAM(property));
  desc = get_desc(class, name);
  type = CLASS_DESC_get_type(desc);

  if (type == CD_PROPERTY)
  {
    if (desc->property.exec == NULL)
    {
      VALUE_write(&value, OBJECT_get_prop_addr(object, desc), desc->property.type);
    }
    else
    {
      VALUE_put(&value, (void *)&temp, desc->property.type);
      if (EXEC_call_native(desc->property.exec, object, (char *)&temp, 0))
        THROW_LIBRARY();
    }
  }
  else if (type == CD_PROPERTY_READ)
    THROW(E_NWRITE);
  else if (type == CD_STATIC_PROPERTY || type == CD_STATIC_PROPERTY_READ)
    THROW(E_STATIC, class->name, name);
  else
    THROW(E_NPROPERTY, class->name, name);

END_METHOD


BEGIN_METHOD(object_attach, void *object; void *parent; GB_STRING name)

  void *object = PARAM(object);

  if (object == NULL)
    THROW(E_NULL);

  OBJECT_attach(object, PARAM(parent), GB_ToZeroString(PARAM(name)));

END_METHOD



BEGIN_METHOD(object_detach, void *object)

  void *object = PARAM(object);

  if (object == NULL)
    THROW(E_NULL);

  OBJECT_detach(object);

END_METHOD


BEGIN_METHOD(object_get_parent, void *object)

  void *object = PARAM(object);

  if (object == NULL)
    THROW(E_NULL);

  GB_ReturnObject(OBJECT_parent(object));

END_METHOD


BEGIN_METHOD(object_class, void *object)

  void *object = PARAM(object);

  if (object == NULL)
    THROW(E_NULL);

  GB_ReturnZeroString(OBJECT_class(object)->name);

END_METHOD



/**************************************************************************************************/


PUBLIC GB_DESC NATIVE_Symbol[] =
{
  GB_DECLARE("Symbol", 0), GB_VIRTUAL_CLASS(),

  GB_PROPERTY_READ("Name", "s", symbol_name),
  GB_PROPERTY_READ("Kind", "s", symbol_kind),
  GB_PROPERTY_READ("Type", "s", symbol_type),
  GB_PROPERTY_READ("ReadOnly", "b", symbol_read_only),
  GB_PROPERTY_READ("Hidden", "b", symbol_hidden),
  GB_PROPERTY_READ("Signature", "s", symbol_signature),
  GB_PROPERTY_READ("Static", "b", symbol_static),
  GB_PROPERTY_READ("Value", "v", symbol_value),

  GB_CONSTANT("Property", "i", 1),
  GB_CONSTANT("Method", "i", 2),
  GB_CONSTANT("Event", "i", 3),
  GB_CONSTANT("Constant", "i", 4),

  GB_END_DECLARE
};


PUBLIC GB_DESC NATIVE_ClassSymbols[] =
{
  GB_DECLARE(".Class.Symbols", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("_next", "Symbol", class_symbols_next, NULL),
  GB_METHOD("_get", "Symbol", class_symbols_get, "(Name)s"),
  GB_PROPERTY_READ("Count", "i", class_symbols_count),

  GB_END_DECLARE
};


PUBLIC GB_DESC NATIVE_Class[] =
{
  GB_DECLARE("Class", 0),  GB_VIRTUAL_CLASS(),

  GB_STATIC_METHOD("_next", "Class", class_next, NULL),
  GB_STATIC_METHOD("_get", "Class", class_get, "(Name)s"),
  GB_STATIC_PROPERTY_READ("Count", "i", class_count),

  GB_STATIC_METHOD("Load", "Class", class_load, "(Name)s"),

  GB_PROPERTY_READ("Symbols", ".Class.Symbols", class_symbols),
  GB_PROPERTY_READ("Name", "s", class_name),
  GB_PROPERTY_READ("Hidden", "b", class_hidden),
  GB_PROPERTY_READ("Native", "b", class_native),

  /*GB_METHOD("New", "Object", class_new, "."),*/

  GB_END_DECLARE
};


PUBLIC GB_DESC NATIVE_Object[] =
{
  GB_DECLARE("Object", 0),  GB_VIRTUAL_CLASS(),

  GB_STATIC_METHOD("GetProperty", "v", object_get_property, "(Object)o(Property)s"),
  GB_STATIC_METHOD("SetProperty", NULL, object_set_property, "(Object)o(Property)s(Value)v"),
  GB_STATIC_METHOD("Attach", NULL, object_attach, "(Object)o(Parent)o(Name)s"),
  GB_STATIC_METHOD("Detach", NULL, object_detach, "(Object)o"),
  GB_STATIC_METHOD("Class", "s", object_class, "(Object)o"),
  GB_STATIC_METHOD("Type", "s", object_class, "(Object)o"),
  GB_STATIC_METHOD("Parent", "o", object_get_parent, "(Object)o"),

  GB_END_DECLARE
};


/*!---------

\HELP
c-class
class

\IN
native

\TITLE
Class

\DESC:fr
Cette classe offre  l'interprteur des possibilits d'introspection. Elle permet :

<UL>
  <LI>D'examiner l'ensemble des classes charges en mmoire.
  <LI>De lister l'ensemble des symboles publics d'une classe.
  <LI>D'instancier dynamiquement une classe.
  <LI>D'accder dynamiquement aux proprits d'un objet.
  <LI>D'excuter dynamiquement toute mthode d'un objet.
</UL>

\REMARK:fr
Les objets de cette classe ne peuvent tre instancis.

\SEE
_c-symbol_

----------*/

/*!---------

\HELP
c-symbol
class

\IN
native

\TITLE
Symbol

\DESC:fr
Cette classe sert  dcrire les symboles publics exports par une classe.

\REMARK:fr
Les objets de cette classe ne peuvent tre instancis.

\SEE
_c-class_

----------*/

