/***************************************************************************

  CCollection.c

  The Collection native class.

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CCOLLECTION_C

#include "common.h"
#include "error.h"

#include "Variant.h"
#include "gambas_api.h"

#include "CCollection.h"


PRIVATE void collection_free(CCOLLECTION *col)
{
  VARIANT *value;
  CCOL_ENUM enum_state;

  enum_state.i = -1;
  enum_state.node = NULL;

  for(;;)
  {
    value = HASH_TABLE_next(col->hash_table, &enum_state.i, &enum_state.node);
    if (value == NULL)
      break;

    VARIANT_free(value);
  }

  HASH_TABLE_delete(&col->hash_table);
}


PRIVATE void *collection_get_key(CCOLLECTION *col, char *key, int len)
{
  return HASH_TABLE_lookup(col->hash_table, key, len);
}


PRIVATE void *collection_add_key(CCOLLECTION *col, char *key, int len)
{
  if (len < 0)
    len = strlen(key);

  return HASH_TABLE_insert(col->hash_table, key, len);
}


PRIVATE void collection_remove_key(CCOLLECTION *col, char *key, int len)
{
  void *value;

  if (len < 0)
    len = strlen(key);

  value = HASH_TABLE_lookup(col->hash_table, key, len);
  if (value == NULL)
    return;

  VARIANT_free(value);
  HASH_TABLE_remove(col->hash_table, key, len);
}


/*
PRIVATE void collection_new(void **ptr, CLASS *class, char *name, OBJECT *parent)
{
  *ptr = CCOLLECTION_new(T_VARIANT);
}


*/


BEGIN_METHOD_VOID(CCOLLECTION_new)

  CCOLLECTION *ob = OBJECT(CCOLLECTION);

  ob->type = T_VARIANT;
  HASH_TABLE_create(&ob->hash_table, TYPE_sizeof(T_VARIANT));

END_METHOD


BEGIN_METHOD_VOID(CCOLLECTION_free)

  collection_free(OBJECT(CCOLLECTION));

END_METHOD


BEGIN_PROPERTY(collection_count)

  GB_ReturnInt(HASH_TABLE_size(OBJECT(CCOLLECTION)->hash_table));

END_PROPERTY


BEGIN_METHOD(collection_add, GB_VARIANT value; GB_STRING key)

  VARIANT *data;

  data = (VARIANT *)collection_add_key(OBJECT(CCOLLECTION), PARAM(key).addr, PARAM(key).len);

  GB_Store(T_VARIANT, &PARAM(value), data);

END_METHOD


BEGIN_METHOD(collection_exist, GB_STRING key)

  GB_ReturnBoolean(
    collection_get_key(OBJECT(CCOLLECTION), PARAM(key).addr, PARAM(key).len) != NULL
    );

END_METHOD


BEGIN_METHOD(collection_remove, GB_STRING key)

  collection_remove_key(OBJECT(CCOLLECTION), PARAM(key).addr, PARAM(key).len);

END_METHOD


BEGIN_METHOD_VOID(collection_clear)

  CCOLLECTION *col = OBJECT(CCOLLECTION);

  collection_free(col);
  HASH_TABLE_create(&col->hash_table, TYPE_sizeof(col->type));

END_METHOD


BEGIN_METHOD_VOID(collection_first)

  CCOL_ENUM *enum_state = (CCOL_ENUM *)GB_GetEnum();

  enum_state->i = (-1);
  enum_state->node = NULL;

END_METHOD


BEGIN_METHOD_VOID(collection_next)

  void *value;
  HASH_TABLE *hash_table = OBJECT(CCOLLECTION)->hash_table;
  CCOL_ENUM *enum_state = (CCOL_ENUM *)GB_GetEnum();

  value = HASH_TABLE_next(hash_table, &enum_state->i, &enum_state->node);

  if (value == NULL)
    GB_StopEnum();
  else
    GB_ReturnPtr(OBJECT(CCOLLECTION)->type, value);

END_METHOD


/*
BEGIN_METHOD(collection_copy, CCOLLECTION *copy)

  CCOLLECTION *copy = PARAM(copy);
  HASH_TABLE *hash_table = THIS->hash_table;
  CCOL_ENUM enum_state;
  void *value;

  collection_clear(THIS, NULL);

  enum_state.i = -1;
  enum_state.node = NULL;

  for(;;)
  {
    value = HASH_TABLE_next(hash_table, &enum_state->i, &enum_state->node);
    if (value == NULL)
      break;

    GB_ReturnPtr(OBJECT(CCOLLECTION)->type, value);
    data = collection_add_key(THIS, PARAM(key).addr, PARAM(key).len);
    GB_Store(T_VARIANT, &PARAM(value), data);

END_METHOD
*/

BEGIN_METHOD(collection_get, GB_STRING key)

  CCOLLECTION *ob = OBJECT(CCOLLECTION);
  void *data;

  data = collection_get_key(ob, PARAM(key).addr, PARAM(key).len);

  if (data == NULL)
    GB_ReturnNull();
  else
    GB_ReturnPtr(T_VARIANT, data);

END_METHOD


BEGIN_METHOD(collection_put, GB_VARIANT value; GB_STRING key)

  CCOLLECTION *ob = OBJECT(CCOLLECTION);
  void *data;
  VARIANT *value = (VARIANT *)&PARAM(value);

  if (VARIANT_is_null(value))
    collection_remove_key(ob, PARAM(key).addr, PARAM(key).len);
  else
  {
    data = collection_add_key(ob, PARAM(key).addr, PARAM(key).len);
    GB_Store(T_VARIANT, &PARAM(value), data);
  }

END_METHOD




PUBLIC GB_DESC NATIVE_Collection[] =
{
  GB_DECLARE("Collection", sizeof(CCOLLECTION)),

  //GB_HOOK_NEW(collection_new),
  //GB_HOOK_FREE(collection_free),

  GB_METHOD("_new", NULL, CCOLLECTION_new, NULL),
  GB_METHOD("_free", NULL, CCOLLECTION_free, NULL),

  GB_PROPERTY_READ("Count", "i", collection_count),
  GB_PROPERTY_READ("Length", "i", collection_count),

  GB_METHOD("Add", NULL, collection_add, "(Value)v(Key)s"),
  GB_METHOD("Exist", "b", collection_exist, "(Key)s"),
  GB_METHOD("Remove", NULL, collection_remove, "(Key)s"),
  GB_METHOD("Clear", NULL, collection_clear, NULL),
  //GB_METHOD("Copy", NULL, collection_copy, "(Collection)Collection"),
  GB_METHOD("_first", NULL, collection_first, NULL),
  GB_METHOD("_next", "v", collection_next, NULL),
  GB_METHOD("_get", "v", collection_get, "(Key)s"),
  GB_METHOD("_put", NULL, collection_put, "(Value)v(Key)s"),

  GB_END_DECLARE
};


/*!---------

\HELP
c-collection
class

\IN
native

\TITLE
Collection

\DESC:fr
Cette classe implmente un dictionnaire pouvant contenir des
lments quelconques. Ces lments sont indexs par des chane de caractres.

\REMARK:fr
Les objets de cette classe peuvent tre numrs.

----------*/

/*!---------

\HELP
c-collection-count
function

\IN
c-collection-p

\TITLE
Count

\TYPE
Read-Only Property Integer

\DESC:fr
Retourne le nombre d'lments contenus dans une collection.

\SYNTAX

<I>nombre</I> = <I>objet</I>.<B>Count</B>

\ARG

<TR><TD><I>objet</I></TD><TD>Un objet de type <I>Collection</I>.</TD><TR>
<TR><TD><I>nombre</I></TD><TD>Le nombre .</TD><TR>

----------*/
