/***************************************************************************

  CFile.c

  The native File class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CFILE_C

#include <stdio.h>

#include "common.h"
#include "gambas_api.h"
#include "gambas.h"
#include "class.h"
#include "file.h"
#include "stream.h"

#include "String.h"
#include "CFile.h"


PUBLIC CFILE CFILE_in, CFILE_out, CFILE_err;

PRIVATE void init(CFILE *ob, FILE *file)
{
  ob->object.class = CLASS_File;
  ob->object.ref = 1;

  ob->stream.type = &STREAM_buffer;
  ob->stream.buffer.file = file;
  ob->stream.buffer.size = 0;
}

PUBLIC void CFILE_init(void)
{
  init(&CFILE_in, stdin);
  init(&CFILE_out, stdout);
  init(&CFILE_err, stderr);
}



PUBLIC CFILE *CFILE_new(void)
{
  void *object;

  OBJECT_new(&object, CLASS_File, NULL, NULL);
  OBJECT_UNREF_KEEP(&object, "CFILE_new");
  return (CFILE *)object;
}

BEGIN_METHOD_VOID(CFILE_free)

  STREAM_close(&(OBJECT(CFILE)->stream));

END_METHOD

BEGIN_PROPERTY(CFILE_get_in)

  GB_ReturnObject(&CFILE_in);

END_PROPERTY

BEGIN_PROPERTY(CFILE_get_out)

  GB_ReturnObject(&CFILE_out);

END_PROPERTY

BEGIN_PROPERTY(CFILE_get_err)

  GB_ReturnObject(&CFILE_err);

END_PROPERTY

BEGIN_PROPERTY(CFILE_type)

  GB_ReturnInt(FILE_stat_info.type);

END_PROPERTY

BEGIN_PROPERTY(CFILE_mode)

  GB_ReturnInt(FILE_stat_info.mode);

END_PROPERTY

BEGIN_PROPERTY(CFILE_hidden)

  GB_ReturnBoolean(FILE_stat_info.hidden);

END_PROPERTY

BEGIN_PROPERTY(CFILE_size)

  GB_ReturnInt(FILE_stat_info.size);

END_PROPERTY

BEGIN_PROPERTY(CFILE_time)

  GB_ReturnDate(FILE_stat_info.time);

END_PROPERTY

BEGIN_PROPERTY(CFILE_separator)

  GB_ReturnString("/", 1);

END_PROPERTY


BEGIN_METHOD(CFILE_dir, GB_STRING path)

  GB_ReturnCopyString(FILE_get_dir(GB_ToZeroString(PARAM(path))), -1);

/*
  char *path = PARAM(path).addr;
  int len = PARAM(path).len;

  if (len == 0)
  {
    GB_ReturnNull();
    return;
  }

  if (len == 1)
  {
    if (*path == '/')
      GB_ReturnCopyString(path, len);
    else
      GB_ReturnNull();
    return;
  }

  for(;;)
  {
    len--;
    if ((len == 0) || (path[len] == '/'))
    {
      GB_ReturnCopyString(path, len);
      return;
    }
  }
*/

END_METHOD


BEGIN_METHOD(CFILE_name, GB_STRING path)

  char *path = GB_ToZeroString(PARAM(path));

  if (path)
    GB_ReturnCopyString(FILE_get_name(path), -1);
  else
    GB_ReturnNull();

END_METHOD


BEGIN_METHOD(CFILE_ext, GB_STRING path)

  char *path = GB_ToZeroString(PARAM(path));

  if (path)
    GB_ReturnCopyString(FILE_get_ext(path), -1);
  else
    GB_ReturnNull();

END_METHOD


BEGIN_METHOD(CFILE_basename, GB_STRING path)

  char *path = GB_ToZeroString(PARAM(path));

  if (path)
    GB_ReturnCopyString(FILE_get_basename(path), -1);
  else
    GB_ReturnNull();

END_METHOD


BEGIN_METHOD(CFILE_load, GB_STRING path)

  char *path = GB_ToZeroString(PARAM(path));
  STREAM stream;
  long len;
  char *str;
  boolean opened = FALSE;

  TRY
  {
    STREAM_open(&stream, path, ST_READ);
    opened = TRUE;

    STREAM_lof(&stream, &len);

    STRING_new(&str, NULL, len);

    STREAM_read(&stream, str, len);
    STREAM_close(&stream);

    GB_ReturnNewString(str);
  }
  CATCH
  {
    if (opened)
      STREAM_close(&stream);
    PROPAGATE();
  }
  END_TRY

END_METHOD


BEGIN_METHOD(CFILE_save, GB_STRING path; GB_STRING data)

  char *path = GB_ToZeroString(PARAM(path));
  STREAM stream;
  boolean opened = FALSE;

  TRY
  {
    STREAM_open(&stream, path, ST_CREATE);
    opened = TRUE;
    STREAM_write(&stream, PARAM(data).addr, PARAM(data).len);
    STREAM_close(&stream);
  }
  CATCH
  {
    if (opened)
      STREAM_close(&stream);
    PROPAGATE();
  }
  END_TRY

END_METHOD



PUBLIC GB_DESC NATIVE_File[] =
{
  GB_DECLARE("File", sizeof(CFILE)),

  GB_HOOK_NEW(NULL),

  GB_METHOD("_free", NULL, CFILE_free, NULL),

  GB_STATIC_PROPERTY_READ("Separator", "s", CFILE_separator),

  GB_STATIC_PROPERTY_READ("In", "File", CFILE_get_in),
  GB_STATIC_PROPERTY_READ("Out", "File", CFILE_get_out),
  GB_STATIC_PROPERTY_READ("Err", "File", CFILE_get_err),

  GB_STATIC_PROPERTY_READ("Type", "i", CFILE_type),
  GB_STATIC_PROPERTY_READ("Mode", "i", CFILE_mode),
  GB_STATIC_PROPERTY_READ("Hidden", "b", CFILE_hidden),
  GB_STATIC_PROPERTY_READ("Size", "i", CFILE_size),
  GB_STATIC_PROPERTY_READ("Time", "d", CFILE_time),

  GB_STATIC_METHOD("Dir", "s", CFILE_dir, "(FileName)s"),
  GB_STATIC_METHOD("Name", "s", CFILE_name, "(FileName)s"),
  GB_STATIC_METHOD("Ext", "s", CFILE_ext, "(FileName)s"),
  GB_STATIC_METHOD("BaseName", "s", CFILE_basename, "(FileName)s"),

  GB_STATIC_METHOD("Load", "s", CFILE_load, "(FileName)s"),
  GB_STATIC_METHOD("Save", NULL, CFILE_save, "(FileName)s(Data)s"),

  GB_END_DECLARE
};


/*!---------

\HELP
c-file
class

\IN
native

\TITLE
File

\DESC:fr
<P>Cette classe reprsente un fichier ouvert ou tout autre flux
d'entres-sorties.</P>

<P>Trois proprits permettent d'accder aux flux d'entre standard,
de sortie standard et d'erreur standard.</P>

<P>Elle possde en outre des proprits statiques qui servent 
stocker le rsultat de la fonction _stat_.</P>

\REMARK:fr
Les objets de cette classe ne peuvent tre directement instancis.
Pour obtenir un objet de type <I>File</I>, il faut utiliser
l'instruction _open_.

\SEE
_open_ _flush_ _input_ _line-input_ _print_ _stat_

----------*/

