/***************************************************************************

  CGambas.c

  The interpreter constants

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CGAMBAS_C

#include "common.h"
#include "gambas_api.h"
#include "gambas.h"
#include "class.h"

#include "CGambas.h"


PUBLIC GB_DESC NATIVE_Gambas[] =
{
  GB_DECLARE("gb", 0), GB_VIRTUAL_CLASS(),

  GB_CONSTANT("Binary", "i", 0),
  GB_CONSTANT("Case", "i", 1),
  GB_CONSTANT("Lang", "i", 2),

  /* BE CAREFUL ! These constants are used in the compiler */
  GB_CONSTANT("Read", "i", 0),
  GB_CONSTANT("Write", "i", 1),
  GB_CONSTANT("Append", "i", 2),
  GB_CONSTANT("Create", "i", 3),
  GB_CONSTANT("Direct", "i", 4),

  /* BE CAREFUL ! These constants are used in the compiler */
  GB_CONSTANT("Null", "i", T_NULL),
  GB_CONSTANT("Boolean", "i", T_BOOLEAN),
  GB_CONSTANT("Byte", "i", T_BYTE),
  GB_CONSTANT("Short", "i", T_SHORT),
  GB_CONSTANT("Integer", "i", T_INTEGER),
  GB_CONSTANT("Long", "i", T_LONG),
  GB_CONSTANT("Float", "i", T_FLOAT),
  GB_CONSTANT("Date", "i", T_DATE),
  GB_CONSTANT("String", "i" , T_STRING),
  GB_CONSTANT("Function", "i" , T_FUNCTION),
  GB_CONSTANT("Variant", "i", T_VARIANT),
  GB_CONSTANT("Class", "i" , T_CLASS),
  GB_CONSTANT("Void", "i", T_VOID),
  GB_CONSTANT("Object", "i", T_OBJECT),

  GB_CONSTANT("File", "i", GB_STAT_FILE),
  GB_CONSTANT("Directory", "i", GB_STAT_DIRECTORY),
  GB_CONSTANT("Device", "i", GB_STAT_DEVICE),
  GB_CONSTANT("Pipe", "i", GB_STAT_PIPE),
  GB_CONSTANT("Socket", "i", GB_STAT_SOCKET),
  GB_CONSTANT("Link", "i", GB_STAT_LINK),

  GB_CONSTANT("Ok", "i", GB_MSG_OK),
  GB_CONSTANT("Cancel", "i", GB_MSG_CANCEL),
  GB_CONSTANT("Yes", "i", GB_MSG_YES),
  GB_CONSTANT("No", "i", GB_MSG_NO),
  GB_CONSTANT("Abort", "i", GB_MSG_ABORT),
  GB_CONSTANT("Retry", "i", GB_MSG_RETRY),
  GB_CONSTANT("Ignore", "i", GB_MSG_IGNORE),

  GB_CONSTANT("OkOnly", "i", GB_MSG_OK_ONLY),
  GB_CONSTANT("OkCancel", "i", GB_MSG_OK_CANCEL),
  GB_CONSTANT("YesNo", "i", GB_MSG_YES_NO),
  GB_CONSTANT("YesNoCancel", "i", GB_MSG_YES_NO_CANCEL),

  GB_CONSTANT("NewLine", "s", "\n"),
  GB_CONSTANT("Tab", "s", "\t"),

  GB_CONSTANT("Standard", "i", 1),
  GB_CONSTANT("GeneralNumber", "i", 2),
  GB_CONSTANT("Fixed", "i", 3),
  GB_CONSTANT("Percent", "i", 4),
  GB_CONSTANT("Scientific", "i", 5),
  GB_CONSTANT("Currency", "i", 6),
  GB_CONSTANT("GeneralDate", "i", 7),
  GB_CONSTANT("LongDate", "i", 8),
  GB_CONSTANT("MediumDate", "i", 9),
  GB_CONSTANT("ShortDate", "i", 10),
  GB_CONSTANT("LongTime", "i", 11),
  GB_CONSTANT("MediumTime", "i", 12),
  GB_CONSTANT("ShortTime", "i", 13),

  GB_END_DECLARE
};


/*!---------

\HELP
c-gambas
class

\IN
native

\TITLE
gb

\TYPE
Static

\DESC:fr
<P>Cette classe statique contient les contantes couramment utilises
par l'interprteur.</P><BR>

<TABLE BORDER=1 CELLPADDING=2 WIDTH=400>
  <CAPTION><B>Constantes de types de donnes</B></CAPTION>
  <TR><TD WIDTH=160><B>gb.Null</B></TD><TD>Type de donnes <I>Null</I>.</TD></TR>
  <TR><TD><B>gb.Boolean</B></TD><TD>Type de donnes <I>Boolean</I>.</TD></TR>
  <TR><TD><B>gb.Byte</B></TD><TD>Type de donnes <I>Byte</I>.</TD></TR>
  <TR><TD><B>gb.Short</B></TD><TD>Type de donnes <I>Short</I>.</TD></TR>
  <TR><TD><B>gb.Integer</B></TD><TD>Type de donnes <I>Integer</I>.</TD></TR>
  <TR><TD><B>gb.Long</B></TD><TD>Type de donnes <I>Long</I>.</TD></TR>
  <TR><TD><B>gb.Float</B></TD><TD>Type de donnes <I>Float</I>.</TD></TR>
  <TR><TD><B>gb.Date</B></TD><TD>Type de donnes <I>Date</I>.</TD></TR>
  <TR><TD><B>gb.String</B></TD><TD>Type de donnes <I>String</I>.</TD></TR>
  <TR><TD><B>gb.Function</B></TD><TD>Type de donnes <I>Function</I>.</TD></TR>
  <TR><TD><B>gb.Variant</B></TD><TD>Type de donnes <I>Variant</I>.</TD></TR>
  <TR><TD><B>gb.Class</B></TD><TD>Type de donnes <I>Class</I>.</TD></TR>
  <TR><TD><B>gb.Void</B></TD><TD>Type de donnes <I>Void</I>.</TD></TR>
  <TR><TD><B>gb.Object</B></TD><TD>Type de donnes <I>Object</I>.</TD></TR>
</TABLE><BR>

<TABLE BORDER=1 CELLPADDING=2 WIDTH=400>
  <CAPTION><B>Constantes de types de fichier</B></CAPTION>
  <TR><TD WIDTH=160><B>gb.File</B></TD><TD>Fichier normal.</TD></TR>
  <TR><TD><B>gb.Directory</B></TD><TD>Rpertoire.</TD></TR>
  <TR><TD><B>gb.Device</B></TD><TD>Priphrique.</TD></TR>
  <TR><TD><B>gb.Pipe</B></TD><TD>Tube.</TD></TR>
  <TR><TD><B>gb.Socket</B></TD><TD>Socket.</TD></TR>
  <TR><TD><B>gb.Link</B></TD><TD>Lien.</TD></TR>
</TABLE><BR>

<TABLE BORDER=1 CELLPADDING=2 WIDTH=400>
  <CAPTION><B>Constantes de format</B></CAPTION>
  <TR><TD WIDTH=160><B>gb.Standard</B></TD><TD>Format standard par dfaut.</TD></TR>
  <TR><TD><B>gb.GeneralNumber</B></TD><TD></TD></TR>
  <TR><TD><B>gb.Fixed</B></TD><TD></TD></TR>
  <TR><TD><B>gb.Percent</B></TD><TD></TD></TR>
  <TR><TD><B>gb.Scientific</B></TD><TD></TD></TR>
  <TR><TD><B>gb.GeneralDate</B></TD><TD></TD></TR>
  <TR><TD><B>gb.LongDate</B></TD><TD></TD></TR>
  <TR><TD><B>gb.MediumDate</B></TD><TD></TD></TR>
  <TR><TD><B>gb.ShortDate</B></TD><TD></TD></TR>
  <TR><TD><B>gb.LongTime</B></TD><TD></TD></TR>
  <TR><TD><B>gb.MediumTime</B></TD><TD></TD></TR>
  <TR><TD><B>gb.ShortTime</B></TD><TD></TD></TR>
</TABLE><BR>

<TABLE BORDER=1 CELLPADDING=2 WIDTH=400>
  <CAPTION><B>Constantes diverses</B></CAPTION>
  <TR><TD WIDTH=160><B>gb.NewLine</B></TD><TD>Le caractre de fin de ligne '\\n'.</TD></TR>
  <TR><TD><B>gb.Tab</B></TD><TD>Le caractre de tabulation '\\t'.</TD></TR>
</TABLE>

----------*/

