/***************************************************************************

  Object.h

  Object management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __OBJECT_H
#define __OBJECT_H

#include "debug.h"
#include "value.h"
#include "class.h"

typedef
  struct {
    CLASS *class;
    long ref;
    }
  OBJECT;

typedef
  struct {
    OBJECT *parent;
    OBJECT *next;
    OBJECT *prev;
    ushort event[0];
    }
  OBJECT_EVENT;

#define OBJECT_event(object) ((OBJECT_EVENT *)((char *)object + ((OBJECT *)(object))->class->off_event))

#define OBJECT_is_class(_object) (OBJECT_class(_object) == CLASS_Class)
#define OBJECT_class(_object) ((_object) ? ((OBJECT *)_object)->class : NULL)

/*
#define OBJECT_data(object) (((OBJECT *)object)->data)
*/

PUBLIC void OBJECT_new(void **ptr, CLASS *class, char *name, OBJECT *parent);
PUBLIC void OBJECT_attach(OBJECT *ob, OBJECT *parent, char *name);
PUBLIC void OBJECT_detach(OBJECT *ob);
PUBLIC void OBJECT_release(CLASS *class, OBJECT *ob);
PUBLIC void OBJECT_free(CLASS *class, OBJECT *ob);

PUBLIC void OBJECT_alloc(void **ptr, CLASS *class, size_t size);
PUBLIC boolean OBJECT_comp_value(VALUE *ob1, VALUE *ob2);

PUBLIC void OBJECT_exit(void);

/*
PRIVATE INLINE CLASS *OBJECT_class(void *object)
{
  if (object)
    return ((OBJECT *)object)->class;
  else
    return object;
}
*/

PRIVATE INLINE void OBJECT_ref(void *object)
{
  if (object)
  {
#if DEBUG_REF
    if (OBJECT_class(object) == (CLASS *)0x23232323)
    {
      printf("Objet dj libr ! %p\n", object);
      fflush(NULL);
    }
#endif
    (*(OBJECT_class(object)->ref))(object);
  }
}


PRIVATE INLINE void OBJECT_unref(void **object)
{
  if (*object)
  {
#if DEBUG_REF
    if (OBJECT_class(*object) == (CLASS *)0x23232323)
    {
      printf("Objet dj libr ! %p\n", *object);
      fflush(NULL);
    }
#endif
    (*(OBJECT_class(*object)->unref))(object, TRUE);
  }
}


PRIVATE INLINE void OBJECT_unref_keep(void **object)
{
  if (*object)
  {
#if DEBUG_REF
    if (OBJECT_class(*object) == (CLASS *)0x23232323)
    {
      printf("Objet dj libr ! %p\n", *object);
      fflush(NULL);
    }
#endif
    (*(OBJECT_class(*object)->unref))(object, FALSE);
  }
}

PRIVATE INLINE void *OBJECT_get_prop_addr(void *object, CLASS_DESC *desc)
{
  return (void *)((char *)object + desc->property.offset);
}

#if DEBUG_REF

#define OBJECT_REF(_ob, _where) printf("REF <" _where "> \n"), OBJECT_ref(_ob)
#define OBJECT_UNREF(_ob, _where) printf("UNREF <" _where "> \n"), OBJECT_unref(_ob)
#define OBJECT_UNREF_KEEP(_ob, _where) printf("UNREF_KEEP <" _where "> \n"), OBJECT_unref_keep(_ob)

#else

#define OBJECT_REF(_ob, _where) OBJECT_ref(_ob)
#define OBJECT_UNREF(_ob, _where) OBJECT_unref(_ob)
#define OBJECT_UNREF_KEEP(_ob, _where) OBJECT_unref_keep(_ob)

#endif


PRIVATE INLINE void OBJECT_null(VALUE *value, CLASS *class)
{
  value->_object.class = class;
  value->_object.object = NULL;
}


PRIVATE INLINE void OBJECT_put(VALUE *value, void *object)
{
  value->_object.class = OBJECT_class(object);
  value->_object.object = object;
}


PRIVATE INLINE void *OBJECT_parent(void *object)
{
  if (OBJECT_class(object)->n_event == 0)
    return NULL;
  else
    return (OBJECT_event(object)->parent);
}



#endif /* */
