/***************************************************************************

  String.c

  The String management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __STRING_C

#include "common.h"
#include "error.h"
#include "value.h"

#include "String.h"


/*PRIVATE char VoidString[5] = { 0 };*/
PRIVATE char CharString[256] = { 0 };


void STRING_new(char **string, char *src, int len)
{
  long *ptr;

  if (len < 0 && src != NULL)
    len = strlen(src);

  if (len <= 0)
  {
    *string = NULL;
    return;
  }

  ALLOC(&ptr, len + 1 + sizeof(long), "STRING_new");
  *ptr = len;
  *string = ((char *)ptr) + sizeof(long);

  if (src != NULL)
  {
    memcpy(*string, src, len);
    (*string)[len] = 0;
  }
}


void STRING_extend(char **string, int new_len)
{
  long *ptr;
  long len = STRING_length(*string);

  if (len >= new_len)
    return;

  if (len == 0)
    ALLOC(&ptr, new_len + 1 + sizeof(long), "STRING_extend");
  else
  {
    ptr = (long *)(*string) - 1;
    REALLOC(&ptr, new_len + 1 + sizeof(long), "STRING_extend");
  }

  *ptr = new_len;
  *string = ((char *)ptr) + sizeof(long);
}



void STRING_copy_from_value(char **string, VALUE *value)
{
  STRING_new(string, &value->_string.addr[value->_string.start], value->_string.len);
}


void STRING_new_value(VALUE *value, char *src, int len)
{
  STRING_new(&(value->_string.addr), src, len);

  value->_string.len = STRING_length(value->_string.addr);
  value->_string.start = 0;
  value->type = T_STRING;
}


void STRING_new_constant_value(VALUE *value, char *src, int len)
{
  value->_string.addr = src;
  value->_string.len = ((len < 0) ? strlen(src) : len);
  value->_string.start = 0;
  value->type = T_CSTRING;
}


void STRING_void_value(VALUE *value)
{
  value->type = T_CSTRING;
  value->_string.addr = NULL;
  value->_string.start = 0;
  value->_string.len = 0;
}


void STRING_char_value(VALUE *value, char car)
{
  char *addr = &CharString[(int)car]; /* * 2 */

  value->type = T_CSTRING;
  value->_string.addr = addr;
  value->_string.start = 0;
  value->_string.len = 1;

  *addr = car;
}


void STRING_free(char **string)
{
  long *ptr;

  if (*string == NULL)
    return;

  ptr = (long *)(*string - sizeof(long));

  if (*ptr == 0)
    return;

  FREE(&ptr, "STRING_free");
  *string = NULL;
}



int STRING_comp_value(VALUE *str1, VALUE *str2)
{
  long i;
  long len = Min(str1->_string.len, str2->_string.len);
  long diff = str1->_string.len - str2->_string.len;
  char *s1, *s2;

  s1 = str1->_string.addr + str1->_string.start;
  s2 = str2->_string.addr + str2->_string.start;

  for (i = 0; i < len; i++)
  {
    if (*s1 > *s2) return 1;
    if (*s1 < *s2) return -1;
    s1++; s2++;
  }

  return (diff < 0) ? (-1) : (diff > 0) ? 1 : 0;
}
