/***************************************************************************

  archive.c

  The archive management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __ARCHIVE_C

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "common.h"
#include "error.h"
#include "value.h"
#include "limit.h"
#include "memory.h"

#include "archive.h"


PRIVATE int arch_fd;
PRIVATE ARCH_HEADER arch_header;
PRIVATE char *arch_string;

PRIVATE ARCH_SYMBOL *arch_symbol;

PRIVATE void arch_error(char *msg)
{
  if (msg == NULL)
    THROW(E_ARCH, strerror(errno));
  else
    THROW(E_ARCH, msg);
}

PRIVATE void set_pos(long pos)
{
  if (lseek(arch_fd, pos, SEEK_SET) < 0)
    arch_error(NULL);
}

PRIVATE void read_at(long pos, void *buf, long len)
{
  set_pos(pos);

  if (read(arch_fd, buf, len) != len)
    arch_error(NULL);
}


PUBLIC void ARCH_init(char *path)
{
  long len;
  int i;
  long pos;

  arch_fd = open(path, O_RDONLY);
  if (arch_fd < 0)
    THROW(E_OPEN, path, strerror(errno));

  /* Lecture de l'entte */

  read_at(32, &arch_header, sizeof(ARCH_HEADER));

  /* Lecture des chanes */

  len = arch_header.pos_table - arch_header.pos_string;
  if (len <= 0)
    arch_error("corrupted header");

  ALLOC(&arch_string, len, "ARCH_init");
  read_at(arch_header.pos_string, arch_string, len);

  /* Lecture de la table */

  len = arch_header.n_symbol * sizeof(ARCH_SYMBOL);
  if (len <= 0)
    arch_error("corrupted header");

  ALLOC(&arch_symbol, len, "ARCH_init");
  read_at(arch_header.pos_table, arch_symbol, len);

  /* Relocation des chanes */

  pos = 0;
  for (i = 0; i < arch_header.n_symbol; i++)
  {
    arch_symbol[i].sym.name = &arch_string[pos];
    pos += arch_symbol[i].sym.len;
  }
}


PUBLIC void ARCH_exit(void)
{
  FREE(&arch_string, "ARCH_exit");
  FREE(&arch_symbol, "ARCH_exit");
  close(arch_fd);
}


PUBLIC boolean ARCH_get(char *path, int len_path, long *pos, long *len)
{
  long index;
  ARCH_SYMBOL *sym;

  if (len_path <= 0)
    len_path = strlen(path);

  SYMBOL_find(arch_symbol, arch_header.n_symbol, sizeof(ARCH_SYMBOL), TF_NORMAL, path, len_path, 0, &index);

  if (index == NO_SYMBOL)
    return TRUE;

  sym = &arch_symbol[index];
  *pos = sym->pos;
  *len = sym->len;

  return FALSE;
}


PUBLIC boolean ARCH_read(long pos, void *buffer, long len)
{
  if (lseek(arch_fd, pos, SEEK_SET) < 0)
    return TRUE;

  if (read(arch_fd, buffer, len) != len)
    return TRUE;

  return FALSE;
}
