/***************************************************************************

  array.c

  C array management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define _ARRAY_C

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "common.h"
#include "error.h"
#include "memory.h"
#include "array.h"


#define DATA_TO_ARRAY(data) ((ARRAY *)((char *)data - sizeof(ARRAY)))
#define ARRAY_TO_DATA(array) ((char *)array + sizeof(ARRAY))


PUBLIC void ARRAY_create_with_size(void *p_data, size_t size, long inc)
{
  ARRAY *array;

  ALLOC(&array, sizeof(ARRAY), "ARRAY_create");

  array->count = 0;
  array->max = 0;
  array->size = size;
  array->inc = inc;

  *((void **)p_data) = ARRAY_TO_DATA(array);
}


PUBLIC void ARRAY_delete(void *p_data)
{
  void **data = (void **)p_data;
  ARRAY *alloc = DATA_TO_ARRAY(*data);

  if (!*data)
    return;

  FREE(&alloc, "ARRAY_delete");

  *data = NULL;
}


PUBLIC long ARRAY_count(void *data)
{
  if (data == NULL)
    return 0;
  else
    return DATA_TO_ARRAY(data)->count;
}


PRIVATE void *add(void *p_data, int num, boolean zero)
{
  void **data = (void **)p_data;
  ARRAY *array = DATA_TO_ARRAY(*data);
  char *ptr;

  array->count += num;

  if (array->count > array->max)
  {
    array->max = array->inc + ((array->count + array->inc) / array->inc) * array->inc ;
    REALLOC(&array, sizeof(ARRAY) + array->max * array->size, "array.add");
    *data = ARRAY_TO_DATA(array);
  }

  ptr = (char *)array + sizeof(ARRAY) + array->size * (array->count - num);

  if (zero) memset(ptr, 0, array->size * num);

  return ptr;
}


PUBLIC void *ARRAY_add(void *p_data)
{
  return add(p_data, 1, FALSE);
}


PUBLIC void *ARRAY_add_many(void *p_data, int num)
{
  return add(p_data, num, FALSE);
}


PUBLIC void *ARRAY_add_void(void *p_data)
{
  return add(p_data, 1, TRUE);
}


/*
PUBLIC void ARRAY_remove_last(void *p_data)
{
  void **data = (void **)p_data;
  ARRAY *array = DATA_TO_ARRAY(*data);

  if (!array->count)
    return;

  array->count--;
}
*/

PUBLIC void *ARRAY_get(void *data, int pos)
{
  ARRAY *array = DATA_TO_ARRAY(data);

  return (char *)data + array->size * pos;
}


PUBLIC void *ARRAY_insert(void *p_data, long pos)
{
  void **data;
  ARRAY *array;
  char *addr;
  long len;

  ARRAY_add(p_data);

  data = (void **)p_data;
  array = DATA_TO_ARRAY(*data);

  if ((pos < 0) || (pos >= array->count))
    pos = array->count - 1;

  addr = ((char *)(*data)) + array->size * pos;
  len = (array->count - pos - 1) * array->size;

  if (len > 0)
    memmove(addr + array->size, addr, len);

  memset(addr, 0, array->size);

  return addr;
}


PUBLIC void ARRAY_remove(void *p_data, long pos)
{
  void **data = (void **)p_data;
  ARRAY *array = DATA_TO_ARRAY(*data);
  char *addr;
  long len;

  if ((pos < 0) || (pos >= array->count))
    return;

  addr = ((char *)(*data)) + array->size * pos;
  len = (array->count - pos - 1) * array->size;

  if (len > 0)
    memmove(addr, addr + array->size, len);

  array->count--;

  if (array->max <= array->inc)
    return;

  if (array->count > (array->max / 2))
    return;

  array->max = ((array->count + array->inc) / array->inc) * array->inc;
  REALLOC(&array, sizeof(ARRAY) + array->max * array->size, "ARRAY_remove");
  *data = ARRAY_TO_DATA(array);
}
