/***************************************************************************

  class.h

  Class loader

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __CLASS_H
#define __CLASS_H

#include "error.h"
#include "memory.h"
#include "type.h"
#include "table.h"
#include "class_desc.h"


typedef
  long CLASS_ID;

/*
typedef
  struct {
    unsigned char flag;
    unsigned char id;
    short value;
    }
  PACKED
  CLASS_TYPE;
*/

typedef
  struct {
    CTYPE type;
    long pos;
    }
  PACKED
  CLASS_VAR;

typedef
  ushort CLASS_REF;

typedef
  union {
    TYPE type;
    struct { TYPE type; double value; } _float;
    struct { TYPE type; long value; } _integer;
    struct { TYPE type; char *addr; long len; } _string;
    }
  PACKED
  CLASS_CONST;

typedef
  struct {
    CTYPE type;
    }
  PACKED
  CLASS_LOCAL;

typedef
  struct {
    TYPE type;
    }
  PACKED
  CLASS_PARAM;

typedef
  struct {
    SYMBOL sym;
    long value;
    }
  LOCAL_SYMBOL;

typedef
  struct {
    ushort line;
    ushort nline;
    ushort *pos;
    char *name;
    LOCAL_SYMBOL *local;
    ushort n_local;
  }
  PACKED
  FUNC_DEBUG;

typedef
  struct {
    TYPE type;
    short n_param;
    short npmin;
    short n_local;
    short n_ctrl;
    short stack_usage;
    short error;
    ushort *code;
    CLASS_PARAM *param;
    CLASS_LOCAL *local;
    FUNC_DEBUG *debug;
    }
  PACKED
  FUNCTION;

typedef
  struct {
    TYPE type;
    short n_param;
    CLASS_PARAM *param;
    char *name;
    }
  PACKED
  CLASS_EVENT;

typedef
  struct {
    ulong magic;
    ulong version;
    ulong checksum;
    ulong flag;
    }
  PACKED
  CLASS_HEADER;

typedef
  struct {
    short parent;
    long s_static;
    long s_dynamic;
    }
  PACKED
  CLASS_INFO;

typedef
  struct {
    TYPE type;
    long dim[0];
    }
  CLASS_ARRAY;

struct _CLASS;

typedef
  struct {
    SYMBOL sym;
    CTYPE ctype;
    long value;
    }
  GLOBAL_SYMBOL;

typedef
  struct {
    short n_cst;
    short n_stat;
    short n_dyn;
    short n_func;

    CLASS_CONST *cst;
    CLASS_VAR *stat;
    CLASS_VAR *dyn;
    FUNCTION *func;
    CLASS_EVENT *event;
    CLASS_ARRAY **array;

    struct _CLASS **class_ref;
    char **unknown;

    GLOBAL_SYMBOL *global;    /* symboles globaux */
    short n_global;
    }
  PACKED
  CLASS_LOAD;

typedef
  struct _CLASS {
    struct _CLASS *class;     /* pointe sur la classe Class ! */
    long count;               /* nombres d'objets */
    struct _CLASS *parent;    /* classe hrite */
    char *name;               /* nom de la classe */
    char *path;               /* emplacement du fichier source */

    unsigned ready : 1;       /* si la classe a t initialise */
    unsigned debug : 1;       /* s'il y a des informations de dbogage */
    unsigned free_name : 1;   /* s'il faut librer le nom */
    unsigned free_event : 1;  /* s'il faut librer class->event */
    unsigned in_load : 1;     /* si la classe est en cours de chargement */
    unsigned _flag : 11;

    short n_desc;             /* nombre de descriptions */
    CLASS_DESC_SYMBOL *table; /* description de la classe */

    short n_event;            /* Nombre d'vnements dclenchables */
    ushort off_event;         /* offset de la structure d'vnement dans l'objet */

    CLASS_EVENT *event;       /* description des vnements */

    void (*new)();            /* instanciation d'un objet de la classe */
    void (*free)();           /* libration d'un objet de cette classe */
    void (*ref)();            /* rfrencement d'un objet de cette classe */
    void (*unref)();          /* drfrencement d'un objet de cette classe */
    int (*check)();           /* vrification de la validit d'un objet */

    CLASS_DESC *special[8];   /* indice des fonctions spciales [], $FIRST, $NEXT, etc. */

    long size_stat;           /* occupation mmoire statique de la classe */
    long size;                /* occupation mmoire dynamique de la classe */

    /*short n_stat;*/             /* nombre de symboles publics statiques de la classe */
    /*short n_dyn;*/              /* nombre de symboles publics dynamiques de la classe */

    /*short n_stat_parent;*/  /* Idem pour TOUTES les classes parentes */
    /*short n_dyn_parent;*/   /* Idem pour TOUTES les classes parentes */

    char *src;                /* emplacement du fichier source */
    char *data;               /* adresse du fichier classe en mmoire */
    CLASS_LOAD *load;         /* informations sur la classe charge */

    char *stat;               /* allocation des donnes statiques */
    TYPE *signature;          /* adresse des signatures */
    char *string;             /* adresse des chanes de caractres */
    }
  CLASS;

typedef
  struct {
    SYMBOL sym;
    CLASS *class;
    }
  PACKED
  CLASS_SYMBOL;

typedef
  enum {
    SPEC_NEW,
    SPEC_FREE,
    SPEC_GET,
    SPEC_PUT,
    SPEC_FIRST,
    SPEC_NEXT,
    SPEC_CALL,
    }
  CLASS_SPECIAL;

typedef
  enum
  {
    CF_DEBUG = 1
  }
  CLASS_FLAG;


#define FUNCTION_is_static(func) ((func)->type & TF_STATIC)
#define CLASS_is_extern(class) ((class)->load == NULL)
#define CLASS_is_native CLASS_is_extern

#define FUNC_INIT_STATIC   0
#define FUNC_INIT_DYNAMIC  1

#ifndef __CLASS_INIT_C
EXTERN CLASS *CLASS_Array;
EXTERN CLASS *CLASS_Class;
EXTERN CLASS *CLASS_Collection;
EXTERN CLASS *CLASS_File;
EXTERN CLASS *CLASS_Symbol;
EXTERN CLASS *CLASS_AppArgs;
#endif

/* class.c */

PUBLIC void CLASS_init(void);
PUBLIC void CLASS_exit(void);

PUBLIC CLASS *CLASS_get(char *name);
PUBLIC long CLASS_count(void);

PUBLIC CLASS_DESC_SYMBOL *CLASS_get_symbol(CLASS *class, char *name);
PUBLIC CLASS_DESC *CLASS_get_symbol_desc(CLASS *class, char *name);
PUBLIC CLASS_DESC *CLASS_get_symbol_desc_kind(CLASS *class, char *name, int kind, int kind2);

PUBLIC long CLASS_find_symbol(CLASS *class, char *name);
PUBLIC long CLASS_find_symbol_with_prefix(CLASS *class, char *name, char *prefix);

PUBLIC CLASS *CLASS_look(char *name);
PUBLIC CLASS *CLASS_find(char *name);

PUBLIC TABLE *CLASS_get_table(void);

PUBLIC boolean CLASS_inherits(CLASS *class, CLASS *parent);

PUBLIC void CLASS_ref(void *object);
PUBLIC void CLASS_unref(void **object, boolean can_free);
PUBLIC void CLASS_release(CLASS *class, char *data);

PUBLIC CLASS **CLASS_get_inheritance(CLASS *class);

PUBLIC void CLASS_do_nothing();
PUBLIC int CLASS_return_zero();

PUBLIC void CLASS_sort(CLASS *class);

PUBLIC void CLASS_inheritance(CLASS *class, char *parent);
PUBLIC void CLASS_make_description(CLASS *class, CLASS_DESC *desc, long n_desc, long *first);
PUBLIC void CLASS_make_event(CLASS *class, int *first);
PUBLIC void CLASS_calc_info(CLASS *class, int n_event, int size_dynamic, boolean all, int size_static);

/* class_init.c */

PUBLIC void CLASS_init_native(void);

/* class_load.c */

PUBLIC void CLASS_load(CLASS *class);

/* class_native.c */

PUBLIC CLASS *CLASS_register(void *pdesc);

#define SET_IF_NULL(ptr, val)  if ((ptr) == NULL) (ptr) = (val)
/*
#define DO_NOTHING(ptr)  if ((ptr) == NULL) (ptr) = CLASS_do_nothing
#define RETURN_ZERO(ptr)  if ((ptr) == NULL) (ptr) = CLASS_return_zero
*/

#define CLASS_is_virtual(class)  (*(class)->name == '.')

#endif /* _CLASS_H */
