/***************************************************************************

  error.h

  Error management

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __ERROR_H
#define __ERROR_H

#include <errno.h>
#include <setjmp.h>

enum {
  E_UNKNOWN,
  E_MEMORY,
  E_CLASS,
  E_STACK,
  E_NEPARAM,
  E_TMPARAM,
  E_TYPE,
  E_OVERFLOW,
  E_ILLEGAL,
  E_NFUNC,
  E_CSTATIC,
  E_NSYMBOL,
  E_NOBJECT,
  E_NULL,
  E_STATIC,
  E_NREAD,
  E_NWRITE,
  E_NPROPERTY,
  E_NRETURN,
  E_MATH,
  E_ARG,
  E_BOUND,
  E_NDIM,
  E_NARRAY,
  E_MAIN,
  E_NNEW,
  E_ZERO,
  E_LIBRARY,
  E_EVENT,
  E_IOBJECT,
  E_FROMLIB,
  E_ENUM,
  E_BADKEY,
  E_DATE,
  E_BADPATH,
  E_OPEN,
  E_PROJECT,
  E_FULL,
  E_EXIST,
  E_EOF,
  E_FORMAT,
  E_DYNAMIC,
  E_SYSTEM,
  E_ACCESS,
  E_TOOLONG,
  E_NEXIST,
  E_DIR,
  E_READ,
  E_WRITE,
  E_NDIR,
  E_REGEXP,
  E_ARCH,
  E_REGISTER,
  E_CLOSED,
  E_VIRTUAL,
  };

#ifndef _ERROR_C
EXTERN long ERROR_code;
#endif


#define TRY                                 \
  {                                         \
     ERROR *err = alloca(sizeof(ERROR));    \
     ERROR_enter(err);                      \
     err->code = setjmp(err->env);          \
     if (err->code == 0)

#define FINALLY

#define CATCH \
     if (err->code != 0 && !(err->code = 0))

#define CATCH_GET(get_it) \
     if (err->code != 0 && (get_it = err->code) && !(err->code = 0))

#define END_TRY          \
    if (err->code == 0)  \
      ERROR_leave(err);  \
    else                 \
      THROW(err->code);  \
  }

/*
#define TRY(err)                      \
  ERROR_enter(&err);                  \
  (err).code = setjmp((err).env);     \
  if (!((err).code))

#define CATCH(err)  else { ERROR_leave(&err);

#define END_TRY(err)   } if (!((err).code)) ERROR_leave(&err);
*/

#define MESSAGE printf

typedef
  struct _ERROR {
    struct _ERROR *prev;
    int code;
    jmp_buf env;
    char *msg;
    }
  ERROR;

#define ERROR_LEAVE_DONE ((ERROR *)-1)

PUBLIC char *ERROR_get(void);

PUBLIC void ERROR_enter(ERROR *err);
PUBLIC void ERROR_leave(ERROR *err);

PUBLIC void PROPAGATE() NORETURN;

PUBLIC void THROW(long code, ...) NORETURN;
PUBLIC void THROW_LIBRARY(void) NORETURN;
PUBLIC void THROW_SYSTEM(int err, char *path);

PUBLIC void ERROR_panic(char *error, ...) NORETURN;

PUBLIC void ERROR_print(void);
PUBLIC void ERROR_print_at(FILE *where);

#endif
