/***************************************************************************

  event.c

  Event management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __EVENT_C

#include "common.h"
#include "common_buffer.h"

#include "event.h"

#define DEBUG


PUBLIC void *EVENT_Last = NULL;



PRIVATE void check_event_method(CLASS *class, char *name, CLASS_DESC_METHOD *desc, CLASS_EVENT *event)
{
  if (TYPE_compare_signature(desc->signature, desc->npmax, (TYPE *)event->param, event->n_param))
    THROW(E_EVENT, class->name, name);
}


PUBLIC void EVENT_search(CLASS *class, ushort *event, char *name, OBJECT *parent)
{
  CLASS *class_parent;
  int i, index;
  char *event_name;
  CLASS_DESC *desc;
  int kind;
  char *func_name = COMMON_buffer;

  kind = parent->class == CLASS_Class ? CD_STATIC_METHOD : CD_METHOD;

  if (kind == CD_STATIC_METHOD)
    class_parent = (CLASS *)parent;
  else
    class_parent = parent->class;

  /*
  if (name == NULL)
    name = class->name;
  */

  for (i = 0; i < class->n_event; i++)
  {
    event_name = class->event[i].name;
    if (*event_name == ':')
      event_name++;

    if (name)
    {
      snprintf(func_name, COMMON_BUF_MAX, "%s_%s", name, event_name);

      index = CLASS_find_symbol(class_parent, func_name);

      if (index != NO_SYMBOL)
      {
        desc = class_parent->table[index].desc;
        if (CLASS_DESC_get_type(desc) == kind)
          goto FOUND;
      }
    }

    snprintf(func_name, COMMON_BUF_MAX, "%s_%s", class->name, event_name);

    index = CLASS_find_symbol(class_parent, func_name);

    if (index != NO_SYMBOL)
    {
      desc = class_parent->table[index].desc;
      if (CLASS_DESC_get_type(desc) == kind)
        goto FOUND;
    }

    event[i] = 0;
    continue;

FOUND:

    #if DEBUG_EVENT
    printf("Event %s.%s  index = %d  parent = %s.%p\n", class_parent->name, func_name, index, parent->class->name, parent);
    #endif
    check_event_method(class_parent, func_name, &desc->method, &class->event[i]);
    event[i] = index + 1;

  }
}



