/***************************************************************************

  exec.h

  Subroutines for the interpreter : executing methods, native methods,
  the NEW operator, the casting operator, etc.

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __EXEC_H
#define __EXEC_H

#include "memory.h"
#include "error.h"
#include "class.h"
#include "value.h"
#include "pcode.h"
#include "stack.h"

#include "String.h"
#include "Object.h"
#include "Variant.h"
#include "Array.h"

typedef
  void (*EXEC_FUNC)();

typedef
  struct {
    CLASS *class;
    void *object;
    short function;
    short nparam;
    }
  EXEC_FUNCTION;

typedef
  struct {
    void (*main)();
    int (*loop)();
    void (*wait)();
    int (*message)();
    void (*beep)();
    void (*watch)();
    }
  EXEC_HOOK;

#ifndef __EXEC_C

EXTERN STACK_CONTEXT EXEC_current;
EXTERN PCODE EXEC_code;
EXTERN VALUE *SP;
EXTERN VALUE *RP;
EXTERN VALUE TEMP;

/*
EXTERN boolean EXEC_stop_next;
EXTERN boolean EXEC_stop_next_line;
EXTERN FUNCTION *EXEC_stop_function;
EXTERN PCODE *EXEC_begin_line;
EXTERN PCODE *EXEC_end_line;
*/

EXTERN boolean EXEC_debug;
EXTERN boolean EXEC_arch;
EXTERN boolean EXEC_signal;

EXTERN double EXEC_start_time;

EXTERN EXEC_HOOK EXEC_Hook;

EXTERN boolean EXEC_enum_stop;
EXTERN void *EXEC_enum_data;

/*EXTERN long EXEC_const[];*/

#endif

/* Pointeur de rfrence des variables locales */
#define BP EXEC_current.bp
/* Classe en cours */
#define CP EXEC_current.cp
/* Objet en cours */
#define OP EXEC_current.op
/* Pointeur de rfrence des paramtres */
/*#define PP EXEC_current.pp*/
/* Sauvegarde du pointeur de pile pour un TRY */
#define EP EXEC_current.ep
/* fonction en cours */
#define FP EXEC_current.fp
/* Pointeur de programme */
#define PC EXEC_current.pc
/* Emplacement o aller en cas d'erreur */
#define EC EXEC_current.ec


#define HOOK(func) (!EXEC_Hook.func) ? 0 : (*EXEC_Hook.func)
#define HOOK_DEFAULT(func, def) (*((!EXEC_Hook.func) ? def : EXEC_Hook.func))

#define GET_NPARAM(var)         short var = *PC & 0x3F
#define GET_PARAM(var, nparam)  VALUE *var = &SP[-nparam]

#define GET_XXX()   (((signed short)(*PC << 4)) >> 4)
#define GET_7XX()   (*PC & 0x7FF)
#define GET_XX()    ((signed char)(*PC & 0xFF))
#define GET_3X()    (*PC & 0x3F)

/*
#define EXEC_output_on_file(file) (EXEC_Output = (file), EXEC_OutputString = NULL)
#define EXEC_output_on_string(str) (EXEC_Output = NULL, EXEC_OutputString = (str))
*/

PRIVATE __inline__ void BORROW(VALUE *value)
{
  if (TYPE_is_object(value->type))
    OBJECT_REF(value->_object.object, "BORROW");
  else if ((value->type == T_VARIANT) && (TYPE_is_object(value->_variant.vtype)))
    OBJECT_REF(*((void **)value->_variant.value), "BORROW");
}

PRIVATE __inline__ void UNBORROW(VALUE *value)
{
  if (TYPE_is_object(value->type))
    OBJECT_UNREF_KEEP(&value->_object.object, "UNBORROW");
  else if ((value->type == T_VARIANT) && (TYPE_is_object(value->_variant.vtype)))
    OBJECT_UNREF_KEEP((void **)value->_variant.value, "UNBORROW");
  /* Avant pas il n'y avait pas de KEEP !? */
}

PRIVATE __inline__ void RELEASE(VALUE *value)
{
  if (value->type == T_STRING)
    STRING_free(&value->_string.addr);
  else if (TYPE_is_object(value->type))
    OBJECT_UNREF(&value->_object.object, "RELEASE");
  else if (value->type == T_VARIANT)
    VARIANT_free((VARIANT *)&value->_variant.vtype);
  else if (value->type == T_ARRAY)
    ARRAY_free(&value->_array.addr, value->_array.desc);
}

PRIVATE __inline__ void RELEASE_STRING(VALUE *value)
{
  if (value->type == T_STRING)
    STRING_free(&value->_string.addr);
  else if ((value->type == T_VARIANT) && (value->_variant.vtype == T_STRING))
    STRING_free((char **)value->_variant.value);
}


PUBLIC void EXEC_init(void);

PUBLIC void EXEC_enter(VALUE_FUNCTION *func, int nparam);
PUBLIC void EXEC_leave(VALUE *ret);
PUBLIC void EXEC_loop(void);

PUBLIC void EXEC_object(VALUE *SP, CLASS **class, OBJECT **object, boolean *defined);

PUBLIC boolean EXEC_call_native(void (*exec)(), void *object, char *param, TYPE type);
PUBLIC void EXEC_native(VALUE_FUNCTION *func, int nparam, boolean drop);
PUBLIC void EXEC_function(VALUE_FUNCTION *func, int nparam);

PUBLIC void EXEC_public(CLASS *class, void *object, char *name, int nparam);

PUBLIC boolean EXEC_spec(int spec, CLASS *class, void *object, int nparam, boolean drop, boolean use_stack);
#define EXEC_special(_spec, _class, _object, _nparam, _drop) EXEC_spec(_spec, _class, _object, _nparam, _drop, FALSE)
#define EXEC_special_use_stack(_spec, _class, _object, _nparam, _drop) EXEC_spec(_spec, _class, _object, _nparam, _drop, TRUE)

PUBLIC void EXEC_special_inheritance(int special, CLASS *class, OBJECT *object, int nparam, boolean drop);

PUBLIC void EXEC_nop(void);
PUBLIC void EXEC_ILLEGAL(void);

PUBLIC void EXEC_push_unknown(void);
PUBLIC void EXEC_push_array(void);
PUBLIC void EXEC_push_special(void);

PUBLIC void EXEC_pop_unknown(void);
PUBLIC void EXEC_pop_array(void);

PUBLIC void EXEC_enum_first(void);
PUBLIC boolean EXEC_enum_next(void);

PUBLIC void EXEC_new(void);
PUBLIC void EXEC_class(CLASS *class, int nparam, boolean drop);

#endif /* */
