/***************************************************************************

  exec_pop.c

  Read values from stack and put them in memory

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "common.h"
#include "limit.h"
#include "exec.h"

#include "String.h"
#include "Object.h"
#include "Array.h"
#include "CCollection.h"


PUBLIC void EXEC_pop_unknown(void)
{
  static void *jump[6] = {
    &&_POP_GENERIC,
    &&_FIN,
    &&_POP_PROPERTY,
    &&_POP_STATIC_PROPERTY,
    &&_POP_NATIVE_PROPERTY,
    &&_FIN
    };

  char *name;
  long index;
  CLASS_DESC *desc;
  CLASS *class;
  OBJECT *object;
  char *addr;
  boolean defined;


  SP--;
  EXEC_object(SP, &class, &object, &defined);
  SP--;

  /*printf("> exec_pop_unknown: SP = %p -> %p\n", SP, SP->_string.addr);*/

  goto *jump[*PC & 0xFF];


_POP_GENERIC:

  name = CP->load->unknown[PC[1]];

  index = CLASS_find_symbol(class, name);
  if (index == NO_SYMBOL)
    THROW(E_NSYMBOL, name, class->name);

  desc = class->table[index].desc;

  if (defined)
    PC[1] = index;

  switch (CLASS_DESC_get_type(desc))
  {
    case CD_CONSTANT:

      THROW(E_NPROPERTY, class->name, name);

    case CD_PROPERTY:

      if (object == NULL)
        THROW(E_DYNAMIC, class->name, name);

      if (desc->property.exec == NULL)
      {
        if (defined) *PC |= 2;
        goto _POP_PROPERTY_2;
      }
      else
      {
        if (defined) *PC |= 4;
        goto _POP_NATIVE_PROPERTY_2;
      }

    case CD_STATIC_PROPERTY:

      if (object != NULL)
        THROW(E_STATIC, class->name, name);

      if (desc->property.exec == NULL)
      {
        if (defined) *PC |= 3;
        goto _POP_STATIC_PROPERTY_2;
      }
      else
      {
        if (defined) *PC |= 4;
        goto _POP_NATIVE_PROPERTY_2;
      }

    case CD_PROPERTY_READ:
    case CD_STATIC_PROPERTY_READ:

      THROW(E_NWRITE);

    case CD_METHOD:
    case CD_STATIC_METHOD:

      THROW(E_NPROPERTY, class->name, name);

    default:

      THROW(E_NSYMBOL, name, class->name);
  }



_POP_PROPERTY:

  desc = class->table[PC[1]].desc;

_POP_PROPERTY_2:

  addr = (char *)object + desc->property.offset;
  VALUE_write(SP, (void *)addr, desc->property.type);
  goto _FIN;



_POP_STATIC_PROPERTY:

  desc = class->table[PC[1]].desc;

_POP_STATIC_PROPERTY_2:

  addr = (char *)class->stat + desc->property.offset;
  VALUE_write(SP, (void *)addr, desc->property.type);
  goto _FIN;



_POP_NATIVE_PROPERTY:

  desc = class->table[PC[1]].desc;

_POP_NATIVE_PROPERTY_2:

  VALUE_put(SP, (void *)&TEMP, desc->property.type);

  SP++;

  if (EXEC_call_native(desc->property.exec, object, (char *)&TEMP, 0))
    THROW_LIBRARY();

  SP--;

_FIN:

  /*printf("< exec_pop_unknown: SP = %p -> %p\n", SP, SP->_string.addr);*/
  RELEASE_STRING(SP);
  PC++;
}


/*
PUBLIC void EXEC_pop_unknown(void)
{
  char *name;
  long index;
  CLASS_DESC *desc;
  CLASS *class;
  OBJECT *object;
  char *addr;
  boolean free;
  boolean defined;

  SP--;

  EXEC_object(SP, &class, &object, &defined);

  PC++;
  name = CP->info.unknown[*PC];

  index = CLASS_find_symbol(class, name);
  if (index == NO_SYMBOL)
    THROW(E_NSYMBOL, name, class->name);
   
  desc = &class->desc[index];
  
  SP--;
   
  switch (CLASS_DESC_get_type(desc))
  {
    case CD_CONSTANT:

      VALUE_write(SP, (void *)&desc->constant.value, desc->constant.type);
      break;
      
    case CD_PROPERTY:
    
      if (CLASS_is_extern(class))
      {
        if (desc->property.read == NULL)
          THROW(E_NWRITE);
        
        free = VALUE_put(SP, (void *)&TEMP, desc->property.type);
        (*desc->property.write)(class, object, (void *)&TEMP);
        if (free) STRING_free((char **)&TEMP);
      }
      else
      {
        if (object == NULL)
          addr = (char *)class->stat;
        else
          addr = (char *)object;

        addr += desc->property.param;
        VALUE_write(SP, (void *)addr, desc->property.type);
      }

      break;

    case CD_METHOD:

      THROW(E_NPROPERTY, class->name, name);

    default:
      THROW(E_NSYMBOL, name, class->name);

  }
}
*/


PUBLIC void EXEC_pop_array(void)
{
  CLASS *class;
  OBJECT *object;
  GET_NPARAM(np);
  long dim[MAX_ARRAY_DIM];
  int i;
  void *data;
  boolean defined;
  /*VALUE swap;*/

  SP -= np;

  if (SP->type == T_ARRAY)
  {
    SP++; np--;

    for (i = 0; i < np; i++)
    {
      VALUE_conv(&SP[i], T_INTEGER);
      dim[i] = SP[i]._integer.value;
    }

    SP--;

    data = ARRAY_get_address((ARRAY_DESC *)SP->_array.desc, SP->_array.addr, np, dim);

    VALUE_write(SP - 1, data, ((ARRAY_DESC *)SP->_array.desc)->type);
    SP--;

    return;
  }

  EXEC_object(SP, &class, &object, &defined);

  /*
  if (object != NULL)
  {
    if (class == CLASS_Collection)
    {
      SP++; np--;

      if (np < 1)
        THROW(E_NEPARAM);
      else if (np > 1)
        THROW(E_TMPARAM);

      VALUE_conv(SP, T_STRING);

      if (TYPE_is_null(SP[-2].type))
      {
        CCOLLECTION_remove_key((CCOLLECTION *)object, SP->_string.addr + SP->_string.start, SP->_string.len);
        RELEASE_STRING(SP);
      }
      else
      {
        data = CCOLLECTION_add_key((CCOLLECTION *)object, SP->_string.addr + SP->_string.start, SP->_string.len);
        RELEASE_STRING(SP);
        SP -= 2;
        VALUE_write(SP, data, ((CCOLLECTION *)object)->type);
      }

      return;
    }
  }
  */

  /* Excution de la mthode spciale _put */

  /*SP += np;

  if (EXEC_special(SPEC_PUT, class, object, np + 1, FALSE))
    THROW(E_NARRAY, class->name);*/

  /*swap = SP[0];*/
  SP[0] = SP[-1];
  /*SP[-1] = swap;*/

  SP += np;

  if (EXEC_special_use_stack(SPEC_PUT, class, object, np, FALSE))
    THROW(E_NARRAY, class->name);

  SP--;
}
