/***************************************************************************

  exec_push.c

  Instructions that push symbols, special functions and arrays on the
  stack.

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/


#include "common.h"
#include "limit.h"
#include "exec.h"
#include "pcode.h"
#include "gambas_api.h"

#include "String.h"
#include "Array.h"
#include "CCollection.h"


PUBLIC void EXEC_push_unknown(void)
{
  static void *jump[6] = {
    &&_PUSH_GENERIC,
    &&_PUSH_CONSTANT,
    &&_PUSH_PROPERTY,
    &&_PUSH_STATIC_PROPERTY,
    &&_PUSH_NATIVE_PROPERTY,
    &&_PUSH_METHOD
    };

  char *name;
  long index;
  CLASS_DESC *desc;
  CLASS *class;
  OBJECT *object;
  char *addr;
  boolean defined;


  SP--;
  EXEC_object(SP, &class, &object, &defined);

  goto *jump[*PC & 0xFF];


_PUSH_GENERIC:

  name = CP->load->unknown[PC[1]];

  index = CLASS_find_symbol(class, name);
  if (index == NO_SYMBOL)
    THROW(E_NSYMBOL, name, class->name);

  desc = class->table[index].desc;

  if (defined)
    PC[1] = index;

  switch (CLASS_DESC_get_type(desc))
  {
    case CD_CONSTANT:

      if (defined)
      {
        if (TYPE_is_integer(desc->constant.type)
            && ((PC[-1] & 0xF800) == C_PUSH_CLASS))
        {
          PC[-1] = C_PUSH_LONG;
          *((long *)PC) = desc->constant.value._integer;
        }
        else
          *PC |= 1;
      }

      goto _PUSH_CONSTANT_2;

    case CD_PROPERTY:
    case CD_PROPERTY_READ:

      if (object == NULL)
        THROW(E_DYNAMIC, class->name, name);

      if (desc->property.exec == NULL)
      {
        if (defined) *PC |= 2;
        goto _PUSH_PROPERTY_2;

      }
      else
      {
        if (defined) *PC |= 4;
        goto _PUSH_NATIVE_PROPERTY_2;
      }

    case CD_STATIC_PROPERTY:
    case CD_STATIC_PROPERTY_READ:

      if (object != NULL)
        THROW(E_STATIC, class->name, name);

      if (desc->property.exec == NULL)
      {
        if (defined) *PC |= 3;
        goto _PUSH_STATIC_PROPERTY_2;

      }
      else
      {
        if (defined) *PC |= 4;
        goto _PUSH_NATIVE_PROPERTY_2;
      }

    case CD_METHOD:

      if (object == NULL)
        THROW(E_DYNAMIC, class->name, name);

      if (defined) *PC |= 5;

      goto _PUSH_METHOD_2;

    case CD_STATIC_METHOD:

      //if (object != NULL)
      //  THROW(E_STATIC, class->name, name);
      object = NULL;

      if (defined) *PC |= 5;

      goto _PUSH_METHOD_2;

    default:

      THROW(E_NSYMBOL, name, class->name);
  }


_PUSH_CONSTANT:

  desc = class->table[PC[1]].desc;

_PUSH_CONSTANT_2:

  VALUE_read(SP, (void *)&desc->constant.value, desc->constant.type);
  goto _FIN_DEFINED;


_PUSH_PROPERTY:

  desc = class->table[PC[1]].desc;

_PUSH_PROPERTY_2:

  addr = (char *)object + desc->property.offset;
  goto _READ_PROPERTY;


_PUSH_STATIC_PROPERTY:

  desc = class->table[PC[1]].desc;

_PUSH_STATIC_PROPERTY_2:

  addr = (char *)class->stat + desc->property.offset;
  goto _READ_PROPERTY;


_READ_PROPERTY:

  VALUE_read(SP, (void *)addr, desc->property.type);
  goto _FIN_DEFINED;


_PUSH_NATIVE_PROPERTY:

  desc = class->table[PC[1]].desc;

_PUSH_NATIVE_PROPERTY_2:

  if (EXEC_call_native(desc->property.exec, object, NULL, desc->property.type))
    THROW_LIBRARY();

  *SP = TEMP;
  goto _FIN_DEFINED;


_PUSH_METHOD:

  desc = class->table[PC[1]].desc;

_PUSH_METHOD_2:

  SP->type = T_FUNCTION;
  SP->_function.class = class;
  SP->_function.object = object;
  SP->_function.function = (long)&desc->method;
  goto _FIN;


_FIN_DEFINED:

  if (!defined)
    VALUE_conv(SP, T_VARIANT);

_FIN:

  SP++;
  PC++;
}



PUBLIC void EXEC_push_special(void)
{
  CLASS_DESC *desc;
  CLASS *class;
  OBJECT *object;
  boolean defined;

  SP--;

  EXEC_object(SP, &class, &object, &defined);

  desc = class->special[*PC & 0xFF];
  if (desc == NULL)
    THROW(E_NNEW);

  if (CLASS_DESC_get_type(desc) != CD_METHOD)
    THROW(E_NFUNC);

  SP->type = T_FUNCTION;
  SP->_function.class = class;
  SP->_function.object = object;
  SP->_function.function = (long)&desc->method;
  SP++;
}


PUBLIC void EXEC_push_array(void)
{
  CLASS *class;
  OBJECT *object;
  GET_NPARAM(np);
  long dim[MAX_ARRAY_DIM];
  int i;
  void *data;
  boolean defined;

  SP -= np;

  if (SP->type == T_ARRAY)
  {
    SP++; np--;

    for (i = 0; i < np; i++)
    {
      VALUE_conv(&SP[i], T_INTEGER);
      dim[i] = SP[i]._integer.value;
    }

    SP--;

    data = ARRAY_get_address((ARRAY_DESC *)SP->_array.desc, SP->_array.addr, np, dim);

    VALUE_read(SP, data, ((ARRAY_DESC *)SP->_array.desc)->type);

    SP++;

  }
  else
  {
    EXEC_object(SP, &class, &object, &defined);

    /* Excution de la mthode spciale _get */

    SP += np;

    if (EXEC_special_use_stack(SPEC_GET, class, object, np - 1, FALSE))
      THROW(E_NARRAY, class->name);
  }
}
