/***************************************************************************

  local.c

  The intenationalization management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __LOCAL_C

#include "common.h"
#include "common_buffer.h"

#include "error.h"
#include "value.h"
#include "limit.h"

#include <locale.h>
#include <time.h>
#include <ctype.h>
#include <math.h>
#include <float.h>
#include <string.h>

#include "mathext.h"
#include "Date.h"

#include "local.h"


/* Localisation interne par dfaut */

PUBLIC LOCAL_INFO LOCAL_default = {
  '.', ',', '/', ':',
  { LO_MONTH, LO_DAY, LO_YEAR },
  { LO_HOUR, LO_MINUTE, LO_SECOND },
  "dddd mmmm d yyyy",
  "mmm d yy",
  "mm/dd/yy",
  "hh:nn:ss",
  "hh:nn AM/PM",
  "hh:nn"
  };

/* Localisation... locale */

PUBLIC LOCAL_INFO LOCAL_local = { 0 };

PRIVATE LOCAL_INFO *local_current;


PRIVATE void begin(void)
{
  buffer_init(COMMON_buffer, COMMON_BUF_MAX - 4);
}

PRIVATE void end(char **str, long *len)
{
  *(get_current()) = 0;
  *str = COMMON_buffer;
  *len = buffer_pos;
}


PRIVATE void stradd_sep(char *dst, char *src, char *sep)
{
  if (*dst)
    strcat(dst, sep);
  strcat(dst, src);
}


PRIVATE void add_thousand_sep(int *before)
{
  if (before == NULL)
    return;

  if ((*before > 1) && ((*before - 1) == (((*before - 1) / 3) * 3)))
  {
    if (local_current->thousand_sep != 0)
      put_char(local_current->thousand_sep);
  }

  (*before)--;
}


/*PRIVATE char *strnadd(char *dst, char *src, int len, int *before)*/
PRIVATE void add_string(char *src, long len, int *before)
{
  while (len > 0)
  {
    put_char(*src++);
    len--;

    add_thousand_sep(before);
  }
}


PRIVATE void add_zero(int zero, int *before)
{
  while (zero > 0)
  {
    put_char('0');
    zero--;

    add_thousand_sep(before);
  }
}


PRIVATE long search(char *src, long len, char *list, long start, boolean not)
{
  long i;
  char c;

  for (i = start; i < len; i++)
  {
    c = src[i];

    if (c == '\\')
    {
      i++;
      continue;
    }

    if ((index(list, c) != NULL) ^ not)
      return i;
  }

  return len;
}


PRIVATE void add_sign(char mode, int sign)
{
  if (sign < 0)
    put_char('-');
  else if (mode != 0)
  {
    if (sign > 0)
      put_char(mode);
    else
      put_char(' ');
  }
}


PUBLIC void LOCAL_init(void)
{
  static struct tm tm = { 0 };
  struct lconv *info;
  char buf[32];
  char *p;
  char c;
  char *dp;
  char *tp;

  /* Localisation courante */

  setlocale(LC_ALL, "");
  info = localeconv();

  LOCAL_local.decimal_point = *(info->decimal_point);
  LOCAL_local.thousand_sep = *(info->thousands_sep);
  if (LOCAL_local.thousand_sep == 0)
    LOCAL_local.thousand_sep = ' ';

  /* Date */

  tm.tm_year = 4; /* 02/03/1904 05:06:07 */
  tm.tm_mday = 2;
  tm.tm_mon = 2;
  tm.tm_hour = 5;
  tm.tm_min = 6;
  tm.tm_sec = 7;

  strftime(buf, sizeof(buf), "%x %X", &tm);

  dp = LOCAL_local.date_order;
  tp = LOCAL_local.time_order;

  for (p = buf;;)
  {
    c = *p++;
    if (!c)
      break;

    switch(c)
    {
      case '4':
        *dp++ = LO_YEAR;
        stradd_sep(LOCAL_local.long_date, "yyyy", " ");
        stradd_sep(LOCAL_local.medium_date, "yy", " ");
        stradd_sep(LOCAL_local.short_date, "yy", "/");
        stradd_sep(LOCAL_local.general_date, "yyyy", "/");
        break;

      case '3':
        *dp++ = LO_MONTH;
        stradd_sep(LOCAL_local.long_date, "mmmm", " ");
        stradd_sep(LOCAL_local.medium_date, "mmm", " ");
        stradd_sep(LOCAL_local.short_date, "mm", "/");
        stradd_sep(LOCAL_local.general_date, "mm", "/");
        break;

      case '2':
        *dp++ = LO_DAY;
        stradd_sep(LOCAL_local.long_date, "dddd d", " ");
        stradd_sep(LOCAL_local.medium_date, "dd", " ");
        stradd_sep(LOCAL_local.short_date, "dd", "/");
        stradd_sep(LOCAL_local.general_date, "dd", "/");
        break;

      case '5':
        *tp++ = LO_HOUR;
        stradd_sep(LOCAL_local.long_time, "hh", ":");
        stradd_sep(LOCAL_local.medium_time, "hh", ":");
        stradd_sep(LOCAL_local.short_time, "hh", ":");
        break;

      case '6':
        *tp++ = LO_MINUTE;
        stradd_sep(LOCAL_local.long_time, "nn", ":");
        stradd_sep(LOCAL_local.medium_time, "nn", ":");
        stradd_sep(LOCAL_local.short_time, "nn", ":");
        break;

      case '7':
        *tp++ = LO_SECOND;
        stradd_sep(LOCAL_local.long_time, "ss", ":");
        break;

      default:
        if (!isdigit(c))
        {
          if (tp != LOCAL_local.time_order)
          {
            if (LOCAL_local.time_sep == 0)
              LOCAL_local.time_sep = c;
          }
          else
          {
            if (LOCAL_local.date_sep == 0)
              LOCAL_local.date_sep = c;
          }
        }

    }
  }

  stradd_sep(LOCAL_local.general_date, LOCAL_local.long_time, " ");
  stradd_sep(LOCAL_local.medium_time, "AM/PM", " ");

#if 0
  {
    char *str;
    long len;
    VALUE value;

    DATE_now(&value);

    LOCAL_format_date(DATE_split(&value), LF_USER, "ddd dd mmm yyyy hh:mm:ss AM/PM", 0, &str, &len);

    printf("FORMAT->%s\n", str);
  }
#endif
}


PUBLIC char *LOCAL_get_lang(void)
{
  return getenv("LANG");
}

PUBLIC void LOCAL_set_lang(char *lang)
{
  setenv("LANG", lang, TRUE);
  setlocale(LC_ALL, "");
}

PUBLIC boolean LOCAL_format_number(double number, int fmt_type, char *fmt, long len_fmt, char **str, long *len_str, boolean local)
{
  char c;
  long n;

  char buf[32];
  char *buf_start;

  long pos;
  long pos2;
  int thousand;
  int *thousand_ptr;

  char sign;
  boolean comma;
  boolean point;
  int before, before_zero;
  int after, after_zero;
  char exposant;
  char exp_sign;
  int exp_digit;
  int exp_zero;

  int number_sign;
  double number_mant;
  int number_exp;
  int number_real_exp;
  int ndigit;

  if (local)
    local_current = &LOCAL_local;
  else
    local_current = &LOCAL_default;

  switch(fmt_type)
  {
    case LF_USER:
      break;

    case LF_STANDARD:
      if ((number != 0.0) && ((fabs(number) < 1E-4) || (fabs(number) >= 1E7)))
        fmt = "-0.############E+#";
      else
        fmt = "-0.############";

      break;

    case LF_GENERAL_NUMBER:
      if ((fabs(number) < 1E-4) || (fabs(number) >= 1E7))
        fmt = "0.############E+#";
      else
        fmt = "0.############";

      break;

    case LF_FIXED:
      fmt = "0.00";
      break;

    case LF_PERCENT:
      fmt = "###%";
      break;

    case LF_SCIENTIFIC:
      fmt = "0.##################E+#";
      break;

    default:
      return TRUE;
  }

  if (len_fmt == 0)
    len_fmt = strlen(fmt);

  if (len_fmt >= COMMON_BUF_MAX)
    return TRUE;

  /* on recherche les formats de nombre ngatif et nul */

  pos = search(fmt, len_fmt, ";", 0, FALSE);

  if (number <= 0.0)
  {
    if (pos < len_fmt)
    {
      if (number < 0.0)
      {
        if ((pos < (len_fmt - 1)) && fmt[pos + 1] != ';')
        {
          fmt = &fmt[pos + 1];
          len_fmt -= pos + 1;
        }
        else
          len_fmt = pos;
      }
      else /* nombre gal  0 */
      {
        pos2 = search(fmt, len_fmt, ";", pos + 1, FALSE);
        if (pos2 < len_fmt)
        {
          if ((pos2 < (len_fmt - 1)) && fmt[pos2 + 1] != ';')
          {
            fmt = &fmt[pos2 + 1];
            len_fmt -= pos2 + 1;
          }
          else
            len_fmt = pos;
        }
      }
    }
  }
  else if (pos < len_fmt)
    len_fmt = pos;


  /* on transcrit le format du basic en format pour sprintf */

  sign = 0;
  comma = FALSE;
  before = 0;
  before_zero = 0;
  point = FALSE;
  after = 0;
  after_zero = 0;
  exposant = 0;
  exp_sign = 0;
  exp_zero = 0;

  begin();

  /* Recherche du '%' */

  pos = search(fmt, len_fmt, "%", 0, FALSE);
  if (pos < len_fmt)
    number *= 100;

  /* prfixe de formatage */

  pos = search(fmt, len_fmt, "-+#0.,", 0, FALSE);

  if (pos >= len_fmt)
    return TRUE;

  if (pos > 0)
    add_string(fmt, pos, NULL);

  /* on dtermine le signe */

  if (fmt[pos] == '-')
  {
    sign = ' ';
    pos++;
  }
  else if (fmt[pos] == '+')
  {
    sign = '+';
    pos++;
  }

  if (pos >= len_fmt)
    return TRUE;

  /* Les chiffres avant la virgule */

  for(; pos < len_fmt; pos++)
  {
    c = fmt[pos];

    if (c == ',')
    {
      comma = TRUE;
      continue;
    }

    if (c == '#' || c == '0')
    {
      before++;
      if (c == '0' || before_zero > 0)
        before_zero++;
      continue;
    }

    break;
  }

  if (pos >= len_fmt)
    goto _FORMAT;

  /* La virgule */

  if (fmt[pos] != '.')
    goto _FORMAT;

  pos++;
  point = TRUE;

  if (pos >= len_fmt)
    goto _FORMAT;

  /* Les chiffres aprs la virgule */

  for(; pos < len_fmt; pos++)
  {
    c = fmt[pos];

    if (c == '#' || c == '0')
    {
      after++;
      if (c == '0')
        after_zero = after;
      continue;
    }

    break;
  }

  if (pos >= len_fmt)
    goto _FORMAT;

  /* L'exposant */

  if (fmt[pos] == 'e' || fmt[pos] == 'E')
  {
    exposant = fmt[pos];
    exp_sign = ' ';

    pos++;
    if (pos >= len_fmt)
      return TRUE;

    if (fmt[pos] == '-')
    {
      pos++;
    }
    else if (fmt[pos] == '+')
    {
      exp_sign = '+';
      pos++;
    }

    if (pos >= len_fmt)
      return TRUE;

    for(; pos < len_fmt; pos++)
    {
      c = fmt[pos];

      if (c == '#' || c == '0')
      {
        exp_digit++;
        if (c == '0' || exp_zero > 0)
          exp_zero++;
        continue;
      }

      break;
    }
  }

_FORMAT:

  if (before == 0 && after == 0)
    return TRUE;

  number_sign = fsgn(number);
  number_mant = frexp10(fabs(number), &number_exp);
  number_exp++; /* simplifie les choses */

  number_real_exp = number_exp;

  if (exposant != 0)
    number_exp = 1;

  ndigit = sprintf(buf, "%.*f", MinMax(after + number_exp - 1, 1, DBL_DIG - 2), number_mant);
  buf[1] = buf[0];
  ndigit--;
  buf_start = &buf[1];

  if (*buf_start == '0')
  {
    buf_start++;
    ndigit--;
  }

  while (ndigit > 1 && buf_start[ndigit - 1] == '0')
    ndigit--;

  /* le signe */

  add_sign(sign, number_sign);

  /* les chiffres avant la virgule */

  thousand = Max(before_zero, number_exp);
  thousand_ptr = comma ? &thousand : NULL;

  if (number_exp > 0)
  {
    if (before_zero > number_exp)
      add_zero(before_zero - number_exp, thousand_ptr);

    add_string(buf_start, Min(number_exp, ndigit), thousand_ptr);

    if (number_exp > ndigit)
      add_zero(number_exp - ndigit, thousand_ptr);
  }
  else
  {
    if (before_zero > 0)
      add_zero(before_zero, thousand_ptr);
  }

  /* la virgule */

  if (point)
    put_char(local_current->decimal_point);

  /* les chiffres aprs la virgule */

  if ((ndigit - number_exp) > 0)
  {
    if (number_exp < 0)
    {
      n = Min(after, (- number_exp));
      if (n == after)
      {
        add_zero(after_zero, NULL);
        goto _EXPOSANT;
      }
      else
      {
        add_zero(n, NULL);
        after -= n;
        after_zero -= n;
      }
    }

    if (number_exp > 0)
    {
      buf_start += number_exp;
      ndigit -= number_exp;
    }

    n = Min(ndigit, after);
    add_string(buf_start, n, NULL);

    after -= n;
    after_zero -= n;

    if (after_zero > 0)
      add_zero(after_zero, NULL);
  }
  else
    add_zero(after_zero, NULL);

_EXPOSANT:

  /* exposant */

  if (exposant != 0)
  {
    put_char(exposant);
    n = sprintf(buf, "%+.*d", exp_zero, number_real_exp - 1);
    add_string(buf, n, NULL);
  }

  /* suffixe de formatage */

  if (pos < len_fmt)
    add_string(&fmt[pos], len_fmt - pos, NULL);

  /* On enlve la virgule si elle se trouve  la fin */

  buffer_pos--;
  if (look_char() != local_current->decimal_point)
    buffer_pos++;

  /* on retourne le rsultat */

  end(str, len_str);
  return FALSE;
}

PRIVATE void add_strftime(char *format, struct tm *tm)
{
  int n;

  n = strftime(get_current(), get_size_left(), format, tm);
  buffer_pos += n;
}


PRIVATE void add_date_token(DATE_SERIAL *date, char *token, int count)
{
  struct tm tm;
  char buf[8];
  int n;

  if (*token == 0)
    return;

  switch (*token)
  {
    case 'd':

      if (count <= 2)
      {
        n = sprintf(buf, (count == 1 ? "%d" : "%02d"), date->day);
        add_string(buf, n, NULL);
      }
      else if (count >= 3)
      {
        tm.tm_wday = date->weekday;
        add_strftime(count == 3 ? "%a" : "%A", &tm);
      }

      break;

    case 'm':

      if (count <= 2)
      {
        n = sprintf(buf, (count == 1 ? "%d" : "%02d"), date->month);
        add_string(buf, n, NULL);
      }
      else if (count >= 3)
      {
        tm.tm_mon = date->month - 1;
        add_strftime(count == 3 ? "%b" : "%B", &tm);
      }

      break;

    case 'y':

      if (count <= 2 && date->year >= 1939 && date->year <= 2038)
        n = sprintf(buf, "%02d", date->year - (date->year >= 2000 ? 2000 : 1900));
      else
        n = sprintf(buf, "%d", date->year);

      add_string(buf, n, NULL);

      break;

    case 'h':
    case 'n':
    case 's':

      n = sprintf(buf, (count == 1) ? "%d" : "%02d",
        (*token == 'h') ? date->hour : ((*token == 'n') ? date->min : date->sec));

      add_string(buf, n, NULL);

      break;
  }

  *token = 0;
}


PUBLIC boolean LOCAL_format_date(DATE_SERIAL *date, int fmt_type, char *fmt, long len_fmt, char **str, long *len_str)
{
  char c;
  long pos;
  long pos_ampm = -1;
  struct tm date_tm;
  char real_hour = 0;

  char token;
  int token_count;

  local_current = &LOCAL_local;

  switch(fmt_type)
  {
    case LF_USER:
      break;

    case LF_STANDARD:
    case LF_GENERAL_DATE:
      if (date->year == 0)
        fmt = local_current->long_time;
      else if (date->hour == 0 && date->min == 0 && date->sec == 0)
        fmt = local_current->short_date;
      else
        fmt = local_current->general_date;
      break;

    case LF_LONG_DATE:
      fmt = local_current->long_date;
      break;

    case LF_MEDIUM_DATE:
      fmt = local_current->medium_date;
      break;

    case LF_SHORT_DATE:
      fmt = local_current->short_date;
      break;

    case LF_LONG_TIME:
      fmt = local_current->long_time;
      break;

    case LF_MEDIUM_TIME:
      fmt = local_current->medium_time;
      break;

    case LF_SHORT_TIME:
      fmt = local_current->short_time;
      break;

    default:
      return TRUE;
  }

  if (len_fmt == 0)
    len_fmt = strlen(fmt);

  if (len_fmt >= COMMON_BUF_MAX)
    return TRUE;

  /* recherche de AM/PM */

  for (pos = 0; pos < len_fmt - 4; pos++)
  {
    if (fmt[pos] == '\\')
    {
      pos++;
      continue;
    }

    if (strncasecmp(&fmt[pos], "am/pm", 5) == 0)
    {
      pos_ampm = pos;
      if (date->hour >= 12)
      {
        real_hour = date->hour;
        date->hour -= 12;
      }
      else
        real_hour = date->hour;
      break;
    }
  }

  /* Formatage */

  begin();

  token = 0;
  token_count = 0;

  for (pos = 0; pos < len_fmt; pos++)
  {
    c = fmt[pos];

    if (pos == pos_ampm)
    {
      /* passage en struct tm */

      date_tm.tm_sec = date->sec;
      date_tm.tm_min = date->min;
      date_tm.tm_hour = real_hour;
      date_tm.tm_mday = 1;
      date_tm.tm_mon = 0;
      date_tm.tm_year = 0;

      add_strftime((c == 'a' ? "%P" : "%p"), &date_tm);

      pos += 4;
      continue;
    }

    if (index("dmyhns", c) != NULL)
    {
      if (c != token)
      {
        add_date_token(date, &token, token_count);
        if (token == 'h' && c == 'm')
          c = 'n';

        token = c;
        token_count = 0;
      }

      token_count++;
    }
    else
    {
      add_date_token(date, &token, token_count);
      if (c == '/')
        put_char(local_current->date_sep);
      else if (c == ':')
        put_char(local_current->time_sep);
      else
        put_char(c);
    }
  }

  add_date_token(date, &token, token_count);

  /* on retourne le rsultat */

  end(str, len_str);
  return FALSE;
}

