/***************************************************************************

  main.c

  interpreter startup

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "common.h"
#include "memory.h"
#include "error.h"

#include <dlfcn.h>

#define _GNU_SOURCE
#include <getopt.h>

#include "class.h"
#include "exec.h"
#include "stack.h"
#include "trace.h"
#include "file.h"
#include "library.h"
#include "project.h"
#include "local.h"
#include "watch.h"
#include "gambas_api.h"

#include "CFile.h"

extern void _exit(int) NORETURN;


PRIVATE struct option LongOptions[] =
{
  { "version", 0, NULL, 'V' },
  { "help", 0, NULL, 'h' },
  { "debug", 0, NULL, 'g' },
  { "signal", 0, NULL, 's' },
  { 0 }
};



PRIVATE void init(void)
{
  LOCAL_init();
  STACK_init();
  EXEC_init();
  CLASS_init();
  CFILE_init();
  WATCH_init();
  TRACE_init();
}


PRIVATE void my_exit(int ret)
{
  LIBRARY_exit();
  PROJECT_exit();
  exit(ret);
}


PRIVATE void main_exit()
{
  OBJECT_exit();
  TRACE_exit();
  WATCH_exit();
  FILE_exit();
  GAMBAS_exit();
  CLASS_exit();
  LIBRARY_exit();
  PROJECT_exit();
  STACK_exit();
}

int main(int argc, char **argv)
{
  CLASS *class;
  VALUE_FUNCTION exec;
  CLASS_DESC_METHOD *startup;

  int opt;
  int ind = 0;
  int ret = 0;

  /*
  int i;

  for (i = 0; i < argc; i++)
    printf("argv[%d] = %s\n", i, argv[i]);
  */

  /*opterr = 0;*/
  optind = 0;

  for(;;)
  {
    opt = getopt_long(argc, argv, "gvVxsh", LongOptions, &ind);
    if (opt < 0) break;

    switch (opt)
    {
      case 'V':
        printf("gbx-" VERSION "\n");
        my_exit(0);

      case 'g':
        EXEC_debug = TRUE;
        break;

      case 'x':
        EXEC_arch = TRUE;
        break;

      case 'h':
        printf(
          "\n"
          "GAMBAS Interpreter version " VERSION " " __DATE__ " " __TIME__ "\n"
          "(c) 2000-2002 Benot Minisini\n\n"
          "This program is free software; you can redistribute it and/or \n"
          "modify it under the terms of the GNU General Public License as \n"
          "published by the Free Software Foundation; either version 1, or \n"
          "(at your option) any later version.\n\n"
          "This program is distributed in the hope that it will be useful,\n"
          "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
          "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
          "GNU General Public License for more details.\n\n"
          "Usage: gbx [options] <project file> -- ...\n\n"
          "Options:\n"
          "  -x                         execute an archive\n"
          "  -g  --debug                enter debugging mode\n"
          "  -V  --version              display version\n"
          "  -h  --help                 display this help\n"
          "\n"
          );

        my_exit(0);

      case 's':
        EXEC_signal = TRUE;

      /* 'l:' dans getopt_long_only
      case 'l':
        LIBRARY_add(optarg);
        break;
      */

      /*
      default:
        printf("try gbx -help\n");
        my_exit(1);
      */

    }
  }

  if (optind == argc)
  {
    fprintf(stderr, "gbx: no input file.\n");
    my_exit(1);
  }

  /*
  if (optind < (argc - 1))
  {
    fprintf(stderr, "gbx: too many input files.\n");
    my_exit(1);
  }
  */

  TRY
  {
    init();

    PROJECT_init(argv[0], argv[optind]);

    memmove(&argv[1], &argv[optind + 1], (argc - optind) * sizeof(char *));
    argc -= optind;

    HOOK(main)(&argc, argv);

    PROJECT_argc = argc;
    PROJECT_argv = argv;

    class = CLASS_get(PROJECT_startup);
    startup = (CLASS_DESC_METHOD *)CLASS_get_symbol_desc_kind(class, "main", CD_STATIC_METHOD, 0);
    if (startup == NULL)
      THROW(E_MAIN);
  }
  CATCH
  {
    main_exit();
    ERROR_print();
    abort();
  }
  END_TRY

  TRY
  {
    if (EXEC_debug)
    {
      TRACE_welcome();
      /*TRACE_start_function = &class->load->func[(long)startup->exec];*/
      TRACE_main(FALSE);
    }

    exec.class = class;
    exec.object = NULL;
    exec.function = (long)startup->exec;

    EXEC_function(&exec, 0);

    ret = HOOK_DEFAULT(loop, WATCH_loop)();
  }
  FINALLY
  {
    main_exit();
  }
  CATCH
  {
    ERROR_panic("Unhandled exception !");
  }
  END_TRY

  fflush(NULL);

  /*return ret;*/
  exit(ret);
  /*_exit(ret);*/
}

