/***************************************************************************

  stack.c

  Stack management

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __STACK_C

#include "common.h"
#include "error.h"
#include "memory.h"
#include "exec.h"
#include "error.h"

#include "stack.h"

PUBLIC long STACK_size = 4096L;
PUBLIC char *STACK_base = NULL;
PUBLIC char *STACK_limit = NULL;
PUBLIC STACK_CONTEXT *STACK_frame;
PUBLIC int STACK_frame_count;

PUBLIC void STACK_init(void)
{
  ALLOC(&STACK_base, STACK_size, "STACK_init");
  STACK_limit = STACK_base + STACK_size;
  STACK_frame = (STACK_CONTEXT *)STACK_limit;
  STACK_frame_count = 0;

  SP = (VALUE *)STACK_base;
}


PUBLIC void STACK_exit(void)
{
  FREE(&STACK_base, "STACK_exit");
}

PUBLIC void STACK_check(int need)
{
#if DEBUG_STACK
  static VALUE *old = NULL;

  if (SP > old)
  {
    printf("STACK = %d bytes\n", ((char *)SP - STACK_base));
    old = SP;
  }
#endif

  if ((char *)(SP + need) >= STACK_limit)
    THROW(E_STACK);
}


PUBLIC void STACK_push_frame(STACK_CONTEXT *context)
{
  if (((char *)SP + sizeof(STACK_CONTEXT)) >= (char *)STACK_frame)
    THROW(E_STACK);

  STACK_frame--;
  *STACK_frame = *context;
  STACK_frame_count++;
  STACK_limit = (char *)STACK_frame;
}


PUBLIC void STACK_pop_frame(STACK_CONTEXT *context)
{
  if (STACK_frame_count <= 0)
    ERROR_panic("Stack frame is void");

  *context = *STACK_frame;
  STACK_frame++;
  STACK_frame_count--;
  STACK_limit = (char *)STACK_frame;
}

PUBLIC boolean STACK_has_error_handler(void)
{
  int i;

  for (i = 0; i < STACK_frame_count; i++)
    if (STACK_frame[i].ec != NULL)
      return TRUE;

  return FALSE;
}




/*
PRIVATE long *Chunk[STACK_MAX_CHUNK] = { NULL };


PUBLIC VALUE *STACK_check(VALUE *sp, short need)
{
  int chunk = -1;
  long addr = 0;

#ifdef DEBUG

  printf("STACK_check(%p, %d)\n", sp, need);
  fflush(NULL);

#endif

  if (sp)
  {
    addr = (long)sp & ~(STACK_CHUNK_SIZE - 1);
    chunk = *((long *)addr);

#ifdef DEBUG

  printf("chunk == %d\n", chunk);
  fflush(NULL);

#endif

    if ((sp + need) < (VALUE *)(addr + STACK_CHUNK_SIZE))
      return sp;
  }

  if (need >= (STACK_CHUNK_SIZE / sizeof(VALUE)))
    ERROR_panic("STACK_check: need too much !\n");

  chunk++;
  if (chunk >= STACK_MAX_CHUNK)
  {
#ifdef DEBUG
    printf("** Stack overflow\n");
    fflush(NULL);
#endif
    THROW(E_STACK);
  }

  if (Chunk[chunk] == NULL)
  {
#ifdef DEBUG
    printf("** Alloc chunk for stack %d\n", chunk);
    fflush(NULL);
#endif
    MEMORY_alloc_chunk(&Chunk[chunk], STACK_CHUNK_SIZE);
    *Chunk[chunk] = chunk;
  }

  sp = (VALUE *)Chunk[chunk] + 1;
#ifdef DEBUG
  printf("** new SP is %p\n", sp);
  fflush(NULL);
#endif

  return sp;
}


*/
