/***************************************************************************

  stream.h

  The stream management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __STREAM_H
#define __STREAM_H

#include "value.h"

union STREAM;

typedef
  struct STREAM_CLASS {
    int (*open)(union STREAM *stream, char *path, int mode);
    int (*close)(union STREAM *stream);
    int (*read)(union STREAM *stream, char *buffer, long len);
    int (*write)(union STREAM *stream, char *buffer, long len);
    int (*seek)(union STREAM *stream, long pos, int whence);
    int (*tell)(union STREAM *stream, long *pos);
    int (*flush)(union STREAM *stream);
    int (*eof)(union STREAM *stream);
    int (*lof)(union STREAM *stream, long *len);
    }
  STREAM_CLASS;

typedef
  struct {
    STREAM_CLASS *type;
    long size;
    long fd;
    char car;
    boolean has_car;
    }
  STREAM_DIRECT;

typedef
  struct {
    STREAM_CLASS *type;
    long size;
    FILE *file;
    }
  STREAM_BUFFER;

typedef
  struct {
    STREAM_CLASS *type;
    long size;
    void *addr;
    long pos;
    }
  STREAM_MEMORY;

typedef
  struct {
    STREAM_CLASS *type;
    long size;
    long start;
    long pos;
    }
  STREAM_ARCH;


typedef
  union STREAM
  {
    STREAM_CLASS *type;
    STREAM_DIRECT direct;
    STREAM_BUFFER buffer;
    STREAM_MEMORY memory;
    STREAM_ARCH arch;
  }
  STREAM;

enum {
  ST_READ = 0,
  ST_WRITE = 1,
  ST_APPEND = 2,
  ST_CREATE = 3,
  ST_DIRECT = 4,
  ST_BINARY = 8,
  ST_MODE = 3,
  ST_SWAP = 16
  };

#ifndef __STREAM_IMPL_C

EXTERN STREAM_CLASS STREAM_direct;
EXTERN STREAM_CLASS STREAM_buffer;
EXTERN STREAM_CLASS STREAM_memory;
EXTERN STREAM_CLASS STREAM_arch;

#else

#define DECLARE_STREAM(stream) \
PUBLIC STREAM_CLASS stream = \
{ \
  (void *)_open, \
  (void *)_close, \
  (void *)_read, \
  (void *)_write, \
  (void *)_seek, \
  (void *)_tell, \
  (void *)_flush, \
  (void *)_eof, \
  (void *)_lof \
}

#endif


PUBLIC void STREAM_exit(void);

PUBLIC void STREAM_open(STREAM *stream, char *path, int mode);
PUBLIC void STREAM_close(STREAM *stream);
PUBLIC void STREAM_write(STREAM *stream, void *addr, long len);
PUBLIC void STREAM_line_input(STREAM *stream, char **addr);
PUBLIC void STREAM_input(STREAM *stream, char **addr);
PUBLIC long STREAM_tell(STREAM *stream);
PUBLIC void STREAM_seek(STREAM *stream, long pos, int whence);
PUBLIC void STREAM_read(STREAM *stream, void *addr, long len);
PUBLIC void STREAM_read_type(STREAM *stream, TYPE type, VALUE *value, long len);
PUBLIC void STREAM_write(STREAM *stream, void *addr, long len);
PUBLIC void STREAM_write_type(STREAM *stream, TYPE type, VALUE *value, long len);
PUBLIC void STREAM_flush(STREAM *stream);

PUBLIC void STREAM_load(char *path, char **buffer, long *len);

#define STREAM_eof(_stream) ((*((_stream)->type->eof))(_stream))
#define STREAM_lof(_stream, _len) ((*((_stream)->type->lof))(_stream, _len))

#endif
