/***************************************************************************

  stream_arch.c

  The archive stream management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __STREAM_IMPL_C

#include "common.h"
#include "error.h"
#include "value.h"
#include "limit.h"

#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>

#include "archive.h"
#include "stream.h"


PRIVATE int _open(STREAM *stream, char *path, int mode)
{
  long pos;
  long len;

  if (ARCH_get(path, 0, &pos, &len))
  {
    errno = ENOENT;
    return TRUE;
  }

  if (mode != ST_READ)
  {
    errno = EACCES;
    return TRUE;
  }

  stream->arch.size = len;
  stream->arch.start = pos;
  stream->arch.pos = 0;

  return FALSE;
}


PRIVATE int _close(STREAM *stream)
{
  return FALSE;
}


PRIVATE int _read(STREAM *stream, char *buffer, long len)
{
  boolean strip = FALSE;
  long max;

  max = stream->arch.size - stream->arch.pos;

  if (len > max)
  {
    strip = TRUE;
    len = max;
  }

  if (ARCH_read(stream->arch.start + stream->arch.pos, buffer, len))
    strip = TRUE;

  stream->arch.pos += len;

  return strip;
}


PRIVATE int _write(STREAM *stream, char *buffer, long len)
{
  return TRUE;
}


PRIVATE int _seek(STREAM *stream, long pos, int whence)
{
  long new_pos;

  switch(whence)
  {
    case SEEK_SET:
      new_pos = pos;
      break;

    case SEEK_CUR:
      new_pos = stream->arch.pos + pos;
      break;

    case SEEK_END:
      new_pos = stream->arch.size - pos;
      break;

    default:
      return TRUE;
  }

  if (new_pos < 0 || new_pos > stream->memory.size)
    return TRUE;

  stream->arch.pos = new_pos;
  return FALSE;
}


PRIVATE int _tell(STREAM *stream, long *pos)
{
  *pos = stream->arch.pos;
  return FALSE;
}


PRIVATE int _flush(STREAM *stream)
{
  return FALSE;
}


PRIVATE int _eof(STREAM *stream)
{
  return (stream->arch.pos >= stream->arch.size);
}


PRIVATE int _lof(STREAM *stream, long *len)
{
  *len = stream->arch.size;
  return FALSE;
}


DECLARE_STREAM(STREAM_arch);

