/***************************************************************************

  stream_buffer.c

  The buffered stream management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __STREAM_IMPL_C

#include "common.h"
#include "error.h"
#include "value.h"
#include "limit.h"

#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>

#include "stream.h"


#define FD (stream->buffer.file)


PRIVATE int _open(STREAM *stream, char *path, int mode)
{
  FILE *file;
  char fmode[4];
  struct stat info;

  switch (mode & ST_MODE)
  {
    case ST_READ:
      strcpy(fmode, "r");
      break;

    case ST_WRITE:
      strcpy(fmode, "r+");
      break;

    case ST_APPEND:
      strcpy(fmode, "a+");
      break;

    case ST_CREATE:
      strcpy(fmode, "w");
      break;
  }

  if (mode & ST_BINARY)
    strcat(fmode, "b");

  file = fopen(path, fmode);
  if (file == NULL)
    return TRUE;

  if (fstat(fileno(file), &info) < 0)
    return TRUE;
  else
    stream->buffer.size = info.st_size;

  FD = file;
  return FALSE;
}


PRIVATE int _close(STREAM *stream)
{
  if (FD == NULL)
    return FALSE;

  if (fclose(FD) < 0)
    return TRUE;

  FD = NULL;
  return FALSE;
}


PRIVATE int _read(STREAM *stream, char *buffer, long len)
{
  size_t eff_read;
  size_t len_read;

  while (len > 0)
  {
    len_read = Min(len, MAX_IO);
    eff_read = fread(buffer, 1, len_read, FD);

    if (eff_read < len_read)
    {
      if (ferror(FD) || feof(FD))
        return TRUE;
    }

    len -= eff_read;
    buffer += eff_read;
  }

  return FALSE;
}


PRIVATE int _write(STREAM *stream, char *buffer, long len)
{
  size_t eff_write;
  size_t len_write;

  while (len > 0)
  {
    len_write = Min(len, MAX_IO);
    eff_write = fwrite(buffer, 1, len_write, FD);

    if (eff_write < len_write)
    {
      if (ferror(FD))
        return TRUE;
    }

    len -= eff_write;
    buffer += eff_write;
  }

  return FALSE;
}


PRIVATE int _seek(STREAM *stream, long pos, int whence)
{
  return (fseek(FD, pos, whence) != 0);
}


PRIVATE int _tell(STREAM *stream, long *pos)
{
  *pos = ftell(FD);
  return (*pos < 0);
}


PRIVATE int _flush(STREAM *stream)
{
  return (fflush(FD) != 0);
}


PRIVATE int _eof(STREAM *stream)
{
  unsigned char c;

  fread(&c, 1, 1, FD);
  if (feof(FD))
    return TRUE;

  ungetc(c, FD);
  return FALSE;
}


PRIVATE int _lof(STREAM *stream, long *len)
{
  *len = stream->buffer.size;
  return FALSE;
}


DECLARE_STREAM(STREAM_buffer);
