/***************************************************************************

  stream_direct.c

  The direct stream management routines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __STREAM_IMPL_C

#include "common.h"
#include "error.h"
#include "value.h"
#include "limit.h"

#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>

#include "stream.h"


#define FD (stream->direct.fd)


PRIVATE int _open(STREAM *stream, char *path, int mode)
{
  int fd;
  struct stat info;

  switch (mode & ST_MODE)
  {
    case ST_READ:
      mode = O_RDONLY;
      break;

    case ST_WRITE:
      mode = O_RDWR | O_CREAT;
      break;

    case ST_APPEND:
      mode = O_RDWR | O_CREAT;
      break;

    case ST_CREATE:
      mode = O_RDWR | O_CREAT | O_TRUNC;
      break;
  }

  fd = open(path, mode, 0666);
  if (fd < 0)
    return TRUE;

  if (fstat(fd, &info) < 0)
    return TRUE;
  else
    stream->direct.size = info.st_size;

  if ((mode & ST_MODE) == ST_APPEND)
  {
    if (lseek(fd, 0, SEEK_END) < 0)
      return TRUE;
  }

  FD = fd;
  return FALSE;
}


PRIVATE int _close(STREAM *stream)
{
  if (close(FD) < 0)
    return TRUE;

  FD = -1;
  return FALSE;
}


PRIVATE int _read(STREAM *stream, char *buffer, long len)
{
  size_t eff_read;
  size_t len_read;

  if (stream->direct.has_car)
  {
    *buffer++ = stream->direct.car;
    len--;
    stream->direct.has_car = FALSE;
  }

  while (len > 0)
  {
    len_read = Min(len, MAX_IO);
    eff_read = read(FD, buffer, len_read);

    if (eff_read < len_read)
    {
      if (eff_read == 0)
        errno = 0;
      if (eff_read <= 0 && errno != EINTR)
        return TRUE;
    }

    len -= eff_read;
    buffer += eff_read;
  }

  return FALSE;
}


PRIVATE int _write(STREAM *stream, char *buffer, long len)
{
  size_t eff_write;
  size_t len_write;

  while (len > 0)
  {
    len_write = Min(len, MAX_IO);
    eff_write = write(FD, buffer, len_write);

    if (eff_write < len_write)
    {
      if (eff_write <= 0 && errno != EINTR)
        return TRUE;
    }

    len -= eff_write;
    buffer += eff_write;
  }

  return FALSE;
}


PRIVATE int _seek(STREAM *stream, long pos, int whence)
{
  return (lseek(FD, pos, whence) < 0);
}


PRIVATE int _tell(STREAM *stream, long *pos)
{
  *pos = lseek(FD, 0, SEEK_CUR);
  return (*pos < 0);
}


PRIVATE int _flush(STREAM *stream)
{
  return FALSE;
}


PRIVATE int _eof(STREAM *stream)
{
  char c;

  if (_read(stream, &c, 1))
    return TRUE;

  stream->direct.car = c;
  stream->direct.has_car = TRUE;
  return FALSE;

  /*
  pos = lseek(FD, 0, SEEK_CUR);
  end = lseek(FD, 0, SEEK_END);
  lseek(FD, pos, SEEK_SET);

  return (pos == end || pos < 0 || end < 0);
  */
}


PRIVATE int _lof(STREAM *stream, long *len)
{
  *len = stream->direct.size;
  return FALSE;
}


DECLARE_STREAM(STREAM_direct);

