/***************************************************************************

  subr.c

  Useful routines for subroutines and miscellaneous subroutines

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "common.h"
#include "subr.h"
#include "gambas.h"
#include "gambas_api.h"


PUBLIC boolean SUBR_check_string(VALUE *param)
{
  if (param->type == T_VARIANT)
    VARIANT_undo(param);

  if (TYPE_is_null(param->type))
    return TRUE;

  if (TYPE_is_string(param->type))
  {
    if (param->_string.len == 0)
      return TRUE;
    else
      return FALSE;
    }

  THROW(E_TYPE, TYPE_get_name(T_STRING), TYPE_get_name((param)->type));
}


PUBLIC void SUBR_check_integer(VALUE *param)
{
  if (param->type == T_VARIANT)
    VARIANT_undo(param);

  if (TYPE_is_integer(param->type))
    return;

  THROW(E_TYPE, TYPE_get_name(T_INTEGER), TYPE_get_name((param)->type));
}


PUBLIC void SUBR_check_float(VALUE *param)
{
  if (param->type == T_VARIANT)
    VARIANT_undo(param);

  if (TYPE_is_number(param->type))
  {
    VALUE_conv(param, T_FLOAT);
    return;
  }

  THROW(E_TYPE, TYPE_get_name(T_INTEGER), TYPE_get_name((param)->type));
}


PUBLIC long SUBR_get_integer(VALUE *param)
{
  SUBR_check_integer(param);
  return param->_integer.value;
}


PUBLIC double SUBR_get_float(VALUE *param)
{
  SUBR_check_float(param);
  return param->_float.value;
}


PUBLIC char *SUBR_get_string(VALUE *param)
{
  GB_STRING str;
  char *ret;

  if (SUBR_check_string(param))
    return "";

  str.addr = param->_string.addr + param->_string.start;
  str.len = param->_string.len;

  ret = GB_ToZeroString(str);

  RELEASE_STRING(param);

  return ret;
}


PUBLIC char *SUBR_copy_string(VALUE *param)
{
  char *ret;

  SUBR_check_string(param);
  STRING_copy_from_value(&ret, param);
  RELEASE_STRING(param);
  return ret;
}


PUBLIC void SUBR_get_string_len(VALUE *param, char **str, long *len)
{
  SUBR_check_string(param);
  VALUE_get_string(param, str, len);
}

/***********************************************************************************************/

PUBLIC void SUBR_error(void)
{
  SUBR_ENTER_PARAM(0);

  RETURN->type = T_INTEGER;
  RETURN->_integer.value = ERROR_code;

  SUBR_LEAVE();
}


PUBLIC void SUBR_shell(void)
{
  char *exec;

  SUBR_ENTER_PARAM(1);

  exec = SUBR_get_string(PARAM);

  /* Il faudrait implmenter son propre system (cf. 'man system') */

  RETURN->type = T_INTEGER;
  RETURN->_integer.value = system(exec);

  SUBR_LEAVE();
}



PRIVATE void wait_nothing(long wait)
{
  ERROR_panic("WAIT: not yet implemented");
}


PUBLIC void SUBR_wait(void)
{
  double wait;

  SUBR_ENTER();

  if (NPARAM == 0)
  {
    HOOK_DEFAULT(wait, wait_nothing)(0);
  }
  else
  {
    wait = SUBR_get_float(PARAM);
    HOOK_DEFAULT(wait, wait_nothing)((long)(wait * 1000 + 0.5));
  }

  SUBR_LEAVE();
}
