/***************************************************************************

  subr_message.c

  The Message() subroutine

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include "common.h"
#include "subr.h"
#include "gambas.h"


PRIVATE int do_message(GAMBAS_MESSAGE *msg)
{
  switch (msg->icon)
  {
    case GB_MSG_WARNING:
      printf("WARNING: ");
      break;

    case GB_MSG_CRITICAL:
      printf("** ERROR: ");
      break;
  }

  if (msg->title)
    printf("%s: ", msg->title);

  if (msg->text)
    printf("%s", msg->text);

  printf("\n");
  return 0;
}


PRIVATE void get_user_defined(GAMBAS_MESSAGE *msg, char *str)
{
  char *p;
  int i;

  for (i = 0; i < 3; i++)
  {
    msg->button[i] = str;

    p = index(str, '|');
    if (p == NULL)
      break;

    *p++ = 0;
    str = p;
  }
}

PUBLIC void SUBR_message()
{
  GAMBAS_MESSAGE msg;
  long len_text;
  long len_title;
  char *user = NULL;

  SUBR_ENTER();

  Clear(&msg);

  msg.text = SUBR_get_string(PARAM);
  len_text = strlen(msg.text);

  /*
  VALUE_conv(PARAM, T_STRING);
  VALUE_get_string(PARAM, &msg.text, &len_text);
  */

  msg.icon = GB_MSG_INFORMATION;

  if (len_text > 0)
  {
    switch (*msg.text)
    {
      case '!':
        msg.icon = GB_MSG_WARNING;
        break;
      case '?':
        msg.icon = GB_MSG_QUESTION;
        break;
      case '*':
        msg.icon = GB_MSG_CRITICAL;
        break;
    }

    if (msg.icon != GB_MSG_INFORMATION)
    {
      msg.text++;
      len_text--;
    }
  }

  msg.type = GB_MSG_OK_ONLY;
  if (NPARAM == 1) goto _FIN;

  if (TYPE_is_string(PARAM[1].type))
  {
    STRING_copy_from_value(&user, &PARAM[1]);
    get_user_defined(&msg, user);
    msg.type = GB_MSG_USER_DEFINED;
  }
  else
  {
    VALUE_conv(&PARAM[1], T_INTEGER);
    msg.type = PARAM[1]._integer.value;
  }

  if (NPARAM == 2) goto _FIN;

  VALUE_conv(&PARAM[2], T_STRING);
  VALUE_get_string(&PARAM[2], &msg.title, &len_title);

_FIN:

  /*RELEASE_STRING(PARAM);*/
  if (NPARAM == 3) RELEASE_STRING(&PARAM[2]);

  RETURN->type = T_INTEGER;
  RETURN->_integer.value = HOOK_DEFAULT(message, do_message)(&msg);

  if (user)
    STRING_free(&user);

  SUBR_LEAVE();
}
