/*

  subr_time.c

  Routines de gestion du temps

*/



#include "common.h"

#include <unistd.h>
#include <sys/time.h>

#include "error.h"
#include "value.h"
#include "subr.h"

#include "Date.h"


void SUBR_timer(void)
{
  struct timeval tv;
  double result;

  SUBR_ENTER_PARAM(0);

  if (gettimeofday(&tv, NULL) == 0)
    result = (double)tv.tv_sec + (double)tv.tv_usec / 1E6 - EXEC_start_time;
  else
    result = 0.0;

  RETURN->type = T_FLOAT;
  RETURN->_float.value = result;

  SUBR_LEAVE();
}


void SUBR_now(void)
{
  SUBR_ENTER_PARAM(0);

  DATE_now(RETURN);

  SUBR_LEAVE();
}


void SUBR_year(void)
{
  DATE_SERIAL *date;

  SUBR_ENTER_PARAM(1);

  VALUE_conv(PARAM, T_DATE);

  date = DATE_split(PARAM);

  PARAM->type = T_INTEGER;
  PARAM->_integer.value = ((short *)date)[(EXEC_code & 0xF) - 1];

  SUBR_LEAVE();
}


void SUBR_date(void)
{
  DATE_SERIAL date;

  SUBR_ENTER();

  if (NPARAM <= 1)
  {
    if (NPARAM == 0)
      DATE_now(PARAM);

    date = *DATE_split(PARAM);
    date.hour = 0;
    date.min = 0;
    date.sec = 0;
    date.msec = 0;
  }
  else
  {
    VALUE_conv(PARAM, T_INTEGER);
    VALUE_conv(&PARAM[1], T_INTEGER);
    VALUE_conv(&PARAM[2], T_INTEGER);

    Clear(&date);
    date.year = PARAM->_integer.value;
    date.month = PARAM[1]._integer.value;
    date.day = PARAM[2]._integer.value;

    if (NPARAM >= 4)
    {
      VALUE_conv(&PARAM[3], T_INTEGER);
      date.hour = PARAM[3]._integer.value;
    }

    if (NPARAM >= 5)
    {
      VALUE_conv(&PARAM[4], T_INTEGER);
      date.min = PARAM[4]._integer.value;
    }

    if (NPARAM >= 6)
    {
      VALUE_conv(&PARAM[5], T_INTEGER);
      date.sec = PARAM[5]._integer.value;
    }
  }

  DATE_make(&date, RETURN);

  SUBR_LEAVE();
}

void SUBR_time(void)
{
  DATE_SERIAL date;

  SUBR_ENTER();

  if (NPARAM <= 1)
  {
    if (NPARAM == 0)
      DATE_now(PARAM);

    date = *DATE_split(PARAM);
    date.year = 0;
  }
  else if (NPARAM == 3)
  {
    VALUE_conv(PARAM, T_INTEGER);
    VALUE_conv(&PARAM[1], T_INTEGER);
    VALUE_conv(&PARAM[2], T_INTEGER);

    Clear(&date);
    date.hour = PARAM->_integer.value;
    date.min = PARAM[1]._integer.value;
    date.sec = PARAM[2]._integer.value;
  }
  else
    THROW(E_NEPARAM);

  DATE_make(&date, RETURN);

  SUBR_LEAVE();
}
