/***************************************************************************

  value.c

  Datatype management routines. Conversions between each Gambas datatype,
  and conversions between Gambas datatypes and native datatypes.

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/


#include "common.h"
#include <float.h>
#include <math.h>

#include "mathext.h"
#include "type.h"

#include "String.h"
#include "Number.h"
#include "Object.h"
#include "Variant.h"
#include "Date.h"

#include "exec.h"
#include "local.h"
#include "common_buffer.h"

#include "value.h"


PUBLIC void VALUE_default(VALUE *value, TYPE type)
{
  static void *jump[16] = {
    &&_VOID, &&_BOOLEAN, &&_BYTE, &&_SHORT, &&_INTEGER, &&_LONG, &&_FLOAT, &&_DATE,
    &&_STRING, &&_STRING, &&_VARIANT, &&_ARRAY, &&_STRUCT, &&_FUNCTION, &&_CLASS, &&_NULL
    };

  value->type = type;

  if (TYPE_is_object(type))
    goto _OBJECT;
  else
    goto *jump[type];

_BOOLEAN:
_BYTE:
_SHORT:
_INTEGER:

  value->_integer.value = 0;
  return;

_FLOAT:
  value->_float.value = 0;
  return;

_STRING:
  value->_string.addr = NULL;
  value->_string.start = 0;
  value->_string.len = 0;
  return;

_VARIANT:
  value->_variant.vtype = T_NULL;
  return;

_DATE:
  value->_date.date = 0;
  value->_date.time = 0;
  return;

_VOID:
  return;

_OBJECT:
  value->_object.class = (CLASS *)type;
  value->_object.object = NULL;
  return;

_LONG:
_ARRAY:
_STRUCT:
_FUNCTION:
_CLASS:
_NULL:
  ERROR_panic("Unknown default type");
}


PUBLIC void VALUE_convert(VALUE *value, TYPE type)
{
  static void *jump[16][16] =
  {
   /*             void      b         c         h         i         l         f         d         s         s         v         array     struct    func      class     n         */
   /* void   */ { &&_OK,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    &&_NR,    },
   /* b      */ { &&_N,     &&_OK,    &&_b2c,   &&_b2h,   &&_OK,    &&_N,     &&_b2f,   &&_N,     &&_b2s,   &&_b2s,   &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* c      */ { &&_N,     &&_c2b,   &&_OK,    &&_c2h,   &&_OK,    &&_N,     &&_c2f,   &&_c2d,   &&_c2s,   &&_c2s,   &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* h      */ { &&_N,     &&_h2b,   &&_h2c,   &&_OK,    &&_OK,    &&_N,     &&_h2f,   &&_h2d,   &&_h2s,   &&_h2s,   &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* i      */ { &&_N,     &&_i2b,   &&_i2c,   &&_i2h,   &&_OK,    &&_N,     &&_i2f,   &&_i2d,   &&_i2s,   &&_i2s,   &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* l      */ { &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_OK,    &&_N,     &&_N,     &&_N,     &&_N,     &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* f      */ { &&_N,     &&_f2b,   &&_f2c,   &&_f2h,   &&_f2i,   &&_N,     &&_OK,    &&_f2d,   &&_f2s,   &&_f2s,   &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* d      */ { &&_N,     &&_N,     &&_d2c,   &&_d2h,   &&_d2i,   &&_N,     &&_d2f,   &&_OK,    &&_d2s,   &&_d2s,   &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* s      */ { &&_N,     &&_s2b,   &&_s2c,   &&_s2h,   &&_s2i,   &&_N,     &&_s2f,   &&_s2d,   &&_OK,    &&_OK,    &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* s      */ { &&_N,     &&_s2b,   &&_s2c,   &&_s2h,   &&_s2i,   &&_N,     &&_s2f,   &&_s2d,   &&_OK,    &&_OK,    &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     },
   /* v      */ { &&_N,     &&_v2,    &&_v2,    &&_v2,    &&_v2,    &&_N,     &&_v2,    &&_v2,    &&_v2,    &&_v2,    &&_OK,    &&_N,     &&_N,     &&_N,     &&_N,     &&_v2,     },
   /* array  */ { &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_OK,    &&_N,     &&_N,     &&_N,     &&_N,     },
   /* struct */ { &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_OK,    &&_N,     &&_N,     &&_N,     },
   /* func   */ { &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_OK,    &&_N,     &&_N,     },
   /* class  */ { &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_OK,    &&_N,     },
   /* null   */ { &&_N,     &&_n2b,   &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_N,     &&_n2s,   &&_n2s,   &&_2v,    &&_N,     &&_N,     &&_N,     &&_N,     &&_OK,    },
  };

  long len;
  char *addr;
  long index;
  CLASS *class;

_conv:

  if ((type | value->type) >> 4)
    goto _OBJECT;
  else
    goto *jump[value->type][type];

_c2b:
_h2b:
_i2b:

  value->_integer.value = (value->_integer.value != 0) ? -1 : 0;
  goto _TYPE;

_f2b:

  value->_integer.value = (value->_float.value != 0) ? -1 : 0;
  goto _TYPE;

_b2c:
_h2c:
_i2c:

  value->_integer.value = (unsigned char)value->_integer.value;
  goto _TYPE;

_f2c:

  value->_integer.value = (unsigned char)value->_float.value;
  goto _TYPE;

_b2h:
_c2h:
_i2h:

  value->_integer.value = (short)value->_integer.value;
  goto _TYPE;

_f2h:

  value->_integer.value = (short)value->_float.value;
  goto _TYPE;

_f2i:

  value->_integer.value = (long)value->_float.value;
  goto _TYPE;

_b2f:
_c2f:
_h2f:
_i2f:

  value->_float.value = value->_integer.value;
  goto _TYPE;

_c2d:
_h2d:
_i2d:

  value->_date.date = Max(0, value->_integer.value);
  value->_date.time = 0;
  goto _TYPE;

_f2d:

  index = (long)floor(value->_float.value);
  value->_date.time = (long)(frac(value->_float.value) * 86400000.0 + 0.5);
  value->_date.date = index;
  goto _TYPE;

_d2c:
_d2h:
_d2i:

  value->_integer.value = value->_date.date;
  value->type = T_INTEGER;
  goto *jump[T_INTEGER][type];

_d2f:

  value->_float.value = (double)value->_date.date + (double)value->_date.time / 86400000.0;
  goto _TYPE;

_b2s:

  if (value->_boolean.value)
    STRING_char_value(value, 'T');
  else
    STRING_void_value(value);
  return;

_c2s:
_h2s:
_i2s:

  len = sprintf(COMMON_buffer, "%ld", value->_integer.value);
  STRING_new_value(value, COMMON_buffer, len);
  return;

_f2s:

  LOCAL_format_number(value->_float.value, LF_GENERAL_NUMBER, NULL, 0, &addr, &len, FALSE);
  STRING_new_value(value, addr, len);
  return;

_d2s:

  len = DATE_to_string(COMMON_buffer, value);
  STRING_new_value(value, COMMON_buffer, len);
  return;

_s2b:

  addr = value->_string.addr;
  value->_integer.value = ((addr != NULL) && (value->_string.len != 0)) ? -1 : 0;
  if (value->type == T_STRING)
    STRING_free(&addr);
  goto _TYPE;

_s2c:
_s2h:
_s2i:

  addr = value->type == T_STRING ? value->_string.addr : NULL;

  if (NUMBER_from_string(NB_READ_INTEGER, value->_string.addr + value->_string.start, value->_string.len, value))
    goto _N;

  STRING_free(&addr);
  goto *jump[T_INTEGER][type];

_s2f:

  addr = value->type == T_STRING ? value->_string.addr : NULL;

  if (NUMBER_from_string(NB_READ_FLOAT, value->_string.addr + value->_string.start, value->_string.len, value))
    goto _N;

  STRING_free(&addr);
  return;

_s2d:

  addr = value->type == T_STRING ? value->_string.addr : NULL;

  if (DATE_from_string(value->_string.addr + value->_string.start, value->_string.len, value, FALSE))
    goto _N;

  STRING_free(&addr);
  return;

_n2b:

  value->_integer.value = 0;
  goto _TYPE;

_n2s:

  STRING_void_value(value);
  return;

_v2:

  /* ATTENTION !
    VALUE_read sert normalement  lire un stockage en mmoire.
    On le dtourne ici pour convertir le variant.
    Mais si le variant contient un T_STRING, a doit rester un T_STRING ! */

  index = value->_variant.vtype;

  if (index != T_NULL)
    VALUE_read(value, &value->_variant.value, value->_variant.vtype);
  value->type = index;

  goto _conv;

_2v:

  if (TYPE_is_string(value->type))
  {
    STRING_copy_from_value(&addr, value);
    RELEASE_STRING(value);
    *((char **)value->_variant.value) = addr;
    value->type = T_STRING;
  }
  else if (value->type != T_NULL)
    VALUE_put(value, &value->_variant.value, value->type);


  /*
  if (value->type == T_CSTRING)
    value->_variant.vtype = T_STRING;
  else
  */

  value->_variant.vtype = value->type;
  goto _TYPE;

_OBJECT:

  if (!TYPE_is_object(type))
  {
    if (type == T_BOOLEAN)
    {
      value->_boolean.value = (value->_object.object != NULL) ? -1 : 0;
      goto _TYPE;
    }

    if (type == T_VARIANT)
      goto _2v;

    goto _N;
  }

  if (!TYPE_is_object(value->type))
  {
    if (value->type == T_NULL)
    {
      OBJECT_null(value, (CLASS *)type); /* marche aussi pour type = T_OBJECT */
      goto _TYPE;
    }

    if (value->type == T_VARIANT)
      goto _v2;

    goto _N;
  }

  if (type == T_OBJECT)
    goto _TYPE;

  if (value->type == T_OBJECT)
  {
    if (value->_object.object == NULL)
      goto _TYPE;

    class = OBJECT_class(value->_object.object);
    /* on continue */
  }
  else
    class = value->_object.class;

  if (CLASS_is_virtual(class))
    THROW(E_VIRTUAL);

  if ((class == (CLASS *)type) || CLASS_inherits(class, (CLASS *)type))
    goto _TYPE;

  THROW(E_TYPE, TYPE_get_name(type), TYPE_get_name((TYPE)class));

_TYPE:

  value->type = type;

_OK:

  return;

_N:

  THROW(E_TYPE, TYPE_get_name(type), TYPE_get_name(value->type));

_NR:

  THROW(E_NRETURN);
}



PUBLIC void VALUE_write(VALUE *value, void *addr, TYPE type)
{
  static void *jump[16] = {
    &&_VOID, &&_BOOLEAN, &&_BYTE, &&_SHORT, &&_INTEGER, &&_LONG, &&_FLOAT, &&_DATE,
    &&_STRING, &&_STRING, &&_VARIANT, &&_ARRAY, &&_STRUCT, &&_FUNCTION, &&_CLASS, &&_NULL
    };

  char *new_string;

_conv:

  if (TYPE_is_object(type))
    goto _OBJECT;
  else
    goto *jump[type];

_BOOLEAN:

  VALUE_conv(value, type);
  *((unsigned char *)addr) = (value->_boolean.value != 0 ? 1 : 0);
  return;

_BYTE:

  VALUE_conv(value, type);
  *((unsigned char *)addr) = (unsigned char)(value->_byte.value);
  return;

_SHORT:

  VALUE_conv(value, type);
  *((short *)addr) = (short)(value->_short.value);
  return;

_INTEGER:

  VALUE_conv(value, type);
  *((long *)addr) = value->_integer.value;
  return;

_FLOAT:

  VALUE_conv(value, type);
  *((double *)addr) = value->_float.value;
  return;

_DATE:

  VALUE_conv(value, type);
  ((long *)addr)[0] = value->_date.date;
  ((long *)addr)[1] = value->_date.time;
  return;

_STRING:

  /* Il faut faire l'affectation en deux temps au cas o
     value->_string.addr == *addr ! */

  VALUE_conv(value, type);

  /*
  if (value->type == T_STRING && value->_string.start == 0 && value->_string.len == STRING_length(value->_string.addr))
  {
    MEMORY_Eco += 2;
  }
  */

  /*
  if (value->type == T_STRING && value->_string.start == 0 && value->_string.len == STRING_length(value->_string.addr))
  {
    new_string = value->_string.addr;
    value->_string.addr = NULL;
  }
  else
  {
  */
    STRING_copy_from_value(&new_string, value);
    RELEASE(value);
  /*}*/
  STRING_free((char **)addr);
  *((char **)addr) = new_string;
  return;

_OBJECT:

  VALUE_conv(value, type);

  OBJECT_UNREF(((void **)addr), "VALUE_write");
  *((void **)addr) = value->_object.object;
  OBJECT_REF(*((void **)addr), "VALUE_write");
  return;

_VARIANT:

  VARIANT_undo(value);
  type = value->type;

  if (type == T_CSTRING) type = T_STRING;

  VARIANT_clear((VARIANT *)addr);

  ((VARIANT *)addr)->type = type;

  /* Et si type ne fait pas partie des types valides pour cette fonction ?? */
  if (type == T_NULL)
    return;

  addr = ((VARIANT *)addr)->value;
  /*goto *jump[Min(T_OBJECT, type)];*/
  goto _conv;

  /*
  {
    VARIANT *var = ((VARIANT *)addr);

    VARIANT_free(var);

    var->type = value->_variant.vtype;
    VARIANT_copy(value->_variant.value, var->value);

    if (TYPE_is_object(var->type))
      OBJECT_$ref(*((void **)var->value));

    return;
  }
  */

_VOID:
_LONG:
_ARRAY:
_STRUCT:
_FUNCTION:
_CLASS:
_NULL:

  ERROR_panic("Bad type (%d) for VALUE_write", type);
}



PUBLIC void VALUE_read(VALUE *value, void *addr, TYPE type)
{
  static void *jump[16] = {
    &&_VOID, &&_BOOLEAN, &&_BYTE, &&_SHORT, &&_INTEGER, &&_LONG, &&_FLOAT, &&_DATE,
    &&_STRING, &&_STRING, &&_VARIANT, &&_ARRAY, &&_STRUCT, &&_FUNCTION, &&_CLASS, &&_NULL
    };

  char *str;

  value->type = type;

  if (TYPE_is_object(type))
    goto _OBJECT;
  else
    goto *jump[type];

_BOOLEAN:

  value->_boolean.value = (*((unsigned char *)addr) != 0) ? (-1) : 0;
  return;

_BYTE:

  value->_byte.value = *((unsigned char *)addr);
  return;

_SHORT:

  value->_short.value = *((short *)addr);
  return;

_INTEGER:

  value->_integer.value = *((long *)addr);
  return;

_FLOAT:

  value->_float.value = *((double *)addr);
  return;

_DATE:

  value->_date.date = ((long *)addr)[0];
  value->_date.time = ((long *)addr)[1];
  return;

_STRING:

  str = *((char **)addr);

  value->type = T_CSTRING;
  value->_string.addr = str;
  value->_string.start = 0;
  value->_string.len = (str == NULL) ? 0 : strlen(str); /*STRING_length(*((char **)addr));*/

  return;

_OBJECT:

  value->_object.object = *((void **)addr);
  return;

_VARIANT:

  value->_variant.type = T_VARIANT;
  value->_variant.vtype = ((VARIANT *)addr)->type;

  if (value->_variant.vtype == T_STRING)
    value->_variant.vtype = T_CSTRING;
  else if (value->_variant.vtype == T_VOID)
    value->_variant.vtype = T_NULL;

  VARIANT_copy(((VARIANT *)addr)->value, value->_variant.value);

  return;

_VOID:
_LONG:
_ARRAY:
_STRUCT:
_CLASS:
_FUNCTION:
_NULL:

  ERROR_panic("Bad type (%d) for VALUE_read", type);
}


/* Retourne TRUE s'il y a eu besoin d'allouer qqch */

/*
#define MAX_FREE_PTR 64

PRIVATE int free_count = 0;
PRIVATE void *free_ptr[MAX_FREE_PTR];


PRIVATE void add_free_ptr(void *ptr)
{
  if (free_count >= MAX_FREE_PTR)
    ERROR_panic("VALUE_put: too many pointers to free");

  free_ptr[free_count] = ptr;
  free_count++;
}

PUBLIC void VALUE_put_free(void)
{
  int i;

  for (i = 0; i < free_count; i++)
    STRING_free((char **)&free_ptr[i]);

  free_count = 0;
}

PUBLIC void VALUE_put_forget(void)
{
  free_count = 0;
}
*/

PUBLIC void VALUE_put(VALUE *value, void *addr, TYPE type)
{
  static void *jump[16] = {
    &&_VOID, &&_BOOLEAN, &&_BYTE, &&_SHORT, &&_INTEGER, &&_LONG, &&_FLOAT, &&_DATE,
    &&_STRING, &&_STRING, &&_VARIANT, &&_ARRAY, &&_STRUCT, &&_FUNCTION, &&_CLASS, &&_NULL
    };

  VALUE_conv(value, type);

  if (TYPE_is_object(type))
    goto _OBJECT;
  else
    goto *jump[type];

_BOOLEAN:

  *((unsigned char *)addr) = (value->_boolean.value != 0 ? 1 : 0);
  return;

_BYTE:

  *((unsigned char *)addr) = (unsigned char)(value->_byte.value);
  return;

_SHORT:

  *((short *)addr) = (short)(value->_short.value);
  return;

_INTEGER:

  *((long *)addr) = value->_integer.value;
  return;

_FLOAT:

  *((double *)addr) = value->_float.value;
  return;

_DATE:

  /* Inverser au cas o value ~= addr */

  ((long *)addr)[1] = value->_date.time;
  ((long *)addr)[0] = value->_date.date;
  return;

_STRING:

  ((long *)addr)[0] = (long)(value->_string.addr + value->_string.start);
  ((long *)addr)[1] = value->_string.len;
  return;

_OBJECT:

  *((void **)addr) = value->_object.object;
  return;

_VARIANT:

  *((VARIANT *)addr) = *((VARIANT *)&value->_variant.vtype);
  return;

  /*
  if
  type = value->type;

  ((VARIANT *)addr)->type = type;
  addr = ((VARIANT *)addr)->value;
  goto _conv;
  */

_VOID:
_LONG:
_ARRAY:
_STRUCT:
_CLASS:
_FUNCTION:
_NULL:

  ERROR_panic("Bad type (%d) for VALUE_put", type);
}



PUBLIC void VALUE_free(void *addr, TYPE type)
{
  if (type == T_STRING)
    STRING_free((char **)addr);
  else if (TYPE_is_object(type))
    OBJECT_unref((void **)addr);
  else if (type == T_VARIANT)
    VARIANT_free((VARIANT *)addr);
}



PUBLIC void VALUE_to_string(VALUE *value, char **addr, long *len)
{
  static void *jump[16] = {
    &&_VOID, &&_BOOLEAN, &&_BYTE, &&_SHORT, &&_INTEGER, &&_LONG, &&_FLOAT, &&_DATE,
    &&_STRING, &&_STRING, &&_VARIANT, &&_ARRAY, &&_STRUCT, &&_FUNCTION, &&_CLASS, &&_NULL
    };

_conv:

  if (TYPE_is_object(value->type))
    goto _OBJECT;
  else
    goto *jump[value->type];

_NULL:

  *addr = ""; /* Pour tre cohrent avec print "" puisque Null == "" */
  *len = 0;
  return;

_BOOLEAN:

  if (value->_boolean.value)
  {
    *addr = "True";
    *len = 4;
  }
  else
  {
    *addr = "False";
    *len = 5;
  }
  return;

_BYTE:
_SHORT:
_INTEGER:

  *len = sprintf(COMMON_buffer, "% ld", value->_integer.value);
  *addr = COMMON_buffer;

  return;

_DATE:

  LOCAL_format_date(DATE_split(value), LF_STANDARD, NULL, 0, addr, len);
  return;

_FLOAT:

  LOCAL_format_number(value->_float.value, LF_STANDARD, NULL, 0, addr, len, TRUE);
  return;

_STRING:

  *len = value->_string.len;
  *addr = value->_string.addr + value->_string.start;
  return;

_OBJECT:

  if (VALUE_is_null(value))
    goto _NULL;

  *len = sprintf(COMMON_buffer, "(%s %p)", OBJECT_class(value->_object.object)->name, value->_object.object);
  *addr = COMMON_buffer;
  return;

_VARIANT:

  VARIANT_undo(value);
  goto _conv;

_VOID:
_LONG:
_ARRAY:
_STRUCT:
_FUNCTION:
_CLASS:

  THROW(E_TYPE, TYPE_get_name(T_STRING), TYPE_get_name(value->type));
}


PUBLIC void VALUE_from_string(VALUE *value, char *addr, long len)
{
  if (DATE_from_string(addr, len, value, TRUE)
      && NUMBER_from_string(NB_READ_BOTH | NB_READ_HEX_BIN | NB_LOCAL, addr, len, value))
    {
      if (len == 4 && strncasecmp(addr, "True", len) == 0)
      {
        value->type = T_BOOLEAN;
        value->_boolean.value = -1;
      }
      else if (len == 5 && strncasecmp(addr, "False", len) == 0)
      {
        value->type = T_BOOLEAN;
        value->_boolean.value = 0;
      }
      else
        value->type = T_NULL;
    }
}


PUBLIC void VALUE_class_read(CLASS *class, VALUE *value, char *addr, CTYPE ctype)
{
  if (ctype.id == T_OBJECT)
  {
    VALUE_read(value, addr, (ctype.value >= 0) ? (TYPE)class->load->class_ref[ctype.value] : T_OBJECT);
  }
  else if (ctype.id == T_ARRAY)
  {
    value->type = T_ARRAY;
    value->_array.desc = class->load->array[ctype.value];
    value->_array.addr = addr;
  }
  else
    VALUE_read(value, addr, (TYPE)ctype.id);
}


PUBLIC void VALUE_class_write(CLASS *class, VALUE *value, char *addr, CTYPE ctype)
{
  if (ctype.id == T_OBJECT)
  {
    VALUE_write(value, addr, (ctype.value >= 0) ? (TYPE)class->load->class_ref[ctype.value] : T_OBJECT);
  }
  else if (ctype.id == T_ARRAY)
  {
    ERROR_panic("Don't know how to copy arrays");
  }
  else
    VALUE_write(value, addr, (TYPE)ctype.id);
}


PUBLIC void VALUE_class_default(CLASS *class, VALUE *value, CTYPE ctype)
{
  if (ctype.id == T_OBJECT)
  {
    VALUE_default(value, (ctype.value >= 0) ? (TYPE)class->load->class_ref[ctype.value] : T_OBJECT);
  }
  else if (ctype.id == T_ARRAY)
  {
    value->type = T_ARRAY;
    value->_array.desc = class->load->array[ctype.value];
    ARRAY_new(&value->_array.addr, (ARRAY_DESC *)value->_array.desc);
  }
  else
    VALUE_default(value, (TYPE)ctype.id);
}
