/***************************************************************************

  CButton.cpp

  The Button and ToggleButton  class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/


#define __CBUTTON_CPP

#include "common.h"

#include <qapplication.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qsizepolicy.h>
#include <qiconset.h>

#include "gambas.h"

#include "CWidget.h"
#include "CPicture.h"
#include "CWindow.h"
#include "CButton.h"

/*#define DEBUG_CBUTTON*/

static long EVENT_Click, EVENT_ClickToggle, EVENT_ClickTool;


BEGIN_METHOD(CBUTTON_new, void *parent)

  MyPushButton *wid = new MyPushButton(CONTAINER(PARAM(parent)));

  QObject::connect(wid, SIGNAL(clicked()), &CButton::manager, SLOT(clicked()));

  CWIDGET_new(wid, (void *)_object, "Button");
  OBJECT(CBUTTON)->picture = NULL;

  wid->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
  wid->setAutoDefault(false);
  wid->show();

END_METHOD


BEGIN_METHOD(CTOGGLEBUTTON_new, void *parent)

  QPushButton *wid = new QPushButton(CONTAINER(PARAM(parent)));

  QObject::connect(wid, SIGNAL(clicked()), &CButton::manager, SLOT(clickedToggle()));

  CWIDGET_new(wid, (void *)_object, "ToggleButton");
  OBJECT(CBUTTON)->picture = NULL;

  wid->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
  wid->setAutoDefault(false);
  wid->setToggleButton(TRUE);
  wid->show();

END_METHOD


BEGIN_METHOD(CTOOLBUTTON_new, void *parent)

  QToolButton *wid = new QToolButton(CONTAINER(PARAM(parent)));

  QObject::connect(wid, SIGNAL(clicked()), &CButton::manager, SLOT(clickedTool()));

  CWIDGET_new(wid, (void *)_object, "ToolButton");
  OBJECT(CBUTTON)->picture = NULL;

  wid->show();

END_METHOD


BEGIN_METHOD_VOID(CBUTTON_free)

  PICTURE_set(&(OBJECT(CBUTTON)->picture), 0);
  CALL_METHOD_VOID(CWIDGET_delete);

END_METHOD


BEGIN_PROPERTY(CBUTTON_text)

  if (READ_PROPERTY)
    GB.ReturnString(QPUSHBUTTON(_object)->text(), 0);
  else
    QPUSHBUTTON(_object)->setText(GB.ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY


BEGIN_PROPERTY(CBUTTON_picture)

  QPushButton *wid = QPUSHBUTTON(_object);

  if (READ_PROPERTY)
    GB.ReturnObject(OBJECT(CBUTTON)->picture);
  else
    wid->setIconSet(QIconSet(PICTURE_set(&(OBJECT(CBUTTON)->picture), PROPERTY(CPICTURE *)), QIconSet::Small));

END_PROPERTY

/*
BEGIN_PROPERTY(CTOOLBUTTON_picture)

  QToolButton *wid = QTOOLBUTTON(_object);
  char **pict = (char **)&(OBJECT(CBUTTON)->picture);

  if (READ_PROPERTY)
    GB.ReturnString(*pict, 0);
  else
    wid->setIconSet(QIconSet(*PIXMAP_set_widget(PROPERTY(GB_STRING), pict), QIconSet::Small));

END_PROPERTY
*/

BEGIN_PROPERTY(CBUTTON_value)

  if (READ_PROPERTY)
    GB.ReturnBoolean(QPUSHBUTTON(_object)->isOn());
  else
    QPUSHBUTTON(_object)->setOn(PROPERTY(char));

END_PROPERTY


BEGIN_PROPERTY(CBUTTON_flat)

  if (READ_PROPERTY)
    GB.ReturnBoolean(QPUSHBUTTON(_object)->isFlat());
  else
    QPUSHBUTTON(_object)->setFlat(PROPERTY(char) != 0);

END_PROPERTY


BEGIN_PROPERTY(CBUTTON_toggle)

  if (READ_PROPERTY)
    GB.ReturnBoolean(WIDGET->isToggleButton());
  else
    WIDGET->setToggleButton(PROPERTY(char) != 0);

END_PROPERTY


BEGIN_PROPERTY(CBUTTON_default)

  MyMainWindow *top = (MyMainWindow *)WIDGET->topLevelWidget();

  if (READ_PROPERTY)
    GB.ReturnBoolean(top->defaultButton() == WIDGET);
  else
    top->setDefaultButton(WIDGET, PROPERTY(char) != 0);

END_PROPERTY


BEGIN_PROPERTY(CBUTTON_cancel)

  MyMainWindow *top = (MyMainWindow *)WIDGET->topLevelWidget();

  if (READ_PROPERTY)
    GB.ReturnBoolean(top->cancelButton() == WIDGET);
  else
    top->setCancelButton(WIDGET, PROPERTY(char) != 0);

END_PROPERTY


//BEGIN_METHOD_VOID(CBUTTON_init)

//  CWidget::setProperties(GB.FindClass("Button"), "Caption,Picture,Flat");

//END_METHOD


GB_DESC CButtonDesc[] =
{
  GB_DECLARE("Button", sizeof(CBUTTON)), GB_INHERITS("Control"),

  GB_METHOD("_new", NULL, CBUTTON_new, "Container;"),
  GB_METHOD("_free", NULL, CBUTTON_free, NULL),
  //GB_STATIC_METHOD("_init", NULL, CBUTTON_init, NULL),

  GB_PROPERTY("Text", "s", CBUTTON_text),
  GB_PROPERTY("Caption", "s", CBUTTON_text),
  GB_PROPERTY("Picture", "Picture", CBUTTON_picture),
  GB_PROPERTY("Flat", "b", CBUTTON_flat),
  //GB_PROPERTY("Toggle", "b", CBUTTON_toggle),
  GB_PROPERTY("Default", "b", CBUTTON_default),
  GB_PROPERTY("Cancel", "b", CBUTTON_cancel),
  //GB_PROPERTY("Value", "b", CBUTTON_value),

  GB_CONSTANT("_Properties", "s", CBUTTON_PROPERTIES),
  GB_CONSTANT("_DefaultEvent", "s", "Click"),

  GB_EVENT("Click", NULL, NULL, &EVENT_Click),

  GB_END_DECLARE
};


GB_DESC CToggleButtonDesc[] =
{
  GB_DECLARE("ToggleButton", sizeof(CBUTTON)), GB_INHERITS("Control"),

  GB_METHOD("_new", NULL, CTOGGLEBUTTON_new, "Container;"),
  GB_METHOD("_free", NULL, CBUTTON_free, NULL),

  GB_PROPERTY("Text", "s", CBUTTON_text),
  GB_PROPERTY("Caption", "s", CBUTTON_text),
  GB_PROPERTY("Picture", "Picture", CBUTTON_picture),
  GB_PROPERTY("Value", "b", CBUTTON_value),
  GB_PROPERTY("Flat", "b", CBUTTON_flat),

  GB_CONSTANT("_Properties", "s", CTOGGLEBUTTON_PROPERTIES),
  GB_CONSTANT("_DefaultEvent", "s", "Click"),

  GB_EVENT("Click", NULL, NULL, &EVENT_ClickToggle),

  GB_END_DECLARE
};

/* Class MyPushButton */

MyPushButton::MyPushButton(QWidget *parent) :
  QPushButton(parent)
{
}

MyPushButton::~MyPushButton()
{
  MyMainWindow *top = (MyMainWindow *)(this->topLevelWidget());

  top->setDefaultButton(this, false);
  top->setCancelButton(this, false);
}

QSize MyPushButton::sizeHint(void) const
{
  return QSize(width(), height());
}

/* Class CButton */

CButton CButton::manager;

void CButton::clicked(void)
{
  RAISE_EVENT(EVENT_Click, NULL);
}

void CButton::clickedToggle(void)
{
  RAISE_EVENT(EVENT_ClickToggle, NULL);
}

void CButton::clickedTool(void)
{
  RAISE_EVENT(EVENT_ClickTool, NULL);
}


/*
GB_DESC CToolButtonDesc[] =
{
  GB_DECLARE("ToolButton", sizeof(CBUTTON)), GB_INHERITS("Control"),

  GB_METHOD("_new", NULL, CTOOLBUTTON_new, "Container;"),
  GB_METHOD("_free", NULL, CBUTTON_delete, NULL),

  GB_PROPERTY("Text", "s", CBUTTON_text),
  GB_PROPERTY("Caption", "s", CBUTTON_text),
  GB_PROPERTY("Picture", "s", CTOOLBUTTON_picture),
  GB_PROPERTY("Value", "b", CBUTTON_value),
  GB_PROPERTY("Flat", "b", CBUTTON_flat),

  GB_EVENT("Click", NULL, NULL, &EVENT_ClickTool),

  GB_END_DECLARE
};
*/


/*!---------

\HELP
c-button
class

\IN
interface

\TITLE
Button

\DESC:fr
Bouton de commande.

\INHERITS
_c-control_

----------*/

/*!---------

\HELP
c-togglebutton
class

\IN
interface

\TITLE
ToggleButton

\DESC:fr
Bouton de commande  deux tats.

\INHERITS
_c-control_

----------*/

