/***************************************************************************

  CContainer.cpp

  The Container class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CCONTAINER_CPP

#include "common.h"

#include <qnamespace.h>
#include <qapplication.h>
#include <qmainwindow.h>

#include "gambas.h"

#include "CWidget.h"

#include "CContainer.h"

/*#define DEBUG*/


BEGIN_METHOD_VOID(CCONTAINER_next)

  QObjectList *list = (QObjectList *)THIS_CONTAINER->children();
  unsigned int index;
  CWIDGET *widget;

  for(;;)
  {
    index = ENUM(int);

    if (list == NULL || index >= list->count())
    {
      GB.StopEnum();
      return;
    }

    ENUM(int) = index + 1;

    widget = CWidget::getReal(list->at(index));
    if (widget)
    {
      GB.ReturnObject(widget);
      return;
    }
  }

END_METHOD


BEGIN_PROPERTY(CCONTAINER_count)

  QWidget *wid = THIS_CONTAINER;

  if (wid->children() == NULL)
    GB.ReturnInteger(0);
  else
    GB.ReturnInteger(wid->children()->count());

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_children)

  GB.ReturnObject(_object);

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_x)

  QPoint p(0, 0);

  GB.ReturnInteger(THIS_CONTAINER->mapTo(THIS, p).x());

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_y)

  QPoint p(0, 0);

  GB.ReturnInteger(THIS_CONTAINER->mapTo(THIS, p).y());

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_width)

  GB.ReturnInteger(THIS_CONTAINER->width());

END_PROPERTY


BEGIN_PROPERTY(CCONTAINER_height)

  GB.ReturnInteger(THIS_CONTAINER->height());

END_PROPERTY


GB_DESC CChildrenDesc[] =
{
  GB_DECLARE(".Container.Children", sizeof(CCONTAINER)), GB_VIRTUAL_CLASS(),

  GB_METHOD("_next", "Control", CCONTAINER_next, NULL),
  GB_PROPERTY_READ("Count", "i", CCONTAINER_count),

  GB_END_DECLARE
};


GB_DESC CContainerDesc[] =
{
  GB_DECLARE("Container", sizeof(CCONTAINER)), GB_INHERITS("Control"),

  GB_HOOK_NEW(NULL),

  GB_PROPERTY_READ("Children", ".Container.Children", CCONTAINER_children),

  GB_PROPERTY_READ("ClientX", "i", CCONTAINER_x),
  GB_PROPERTY_READ("ClientY", "i", CCONTAINER_y),
  GB_PROPERTY_READ("ClientW", "i", CCONTAINER_width),
  GB_PROPERTY_READ("ClientWidth", "i", CCONTAINER_width),
  GB_PROPERTY_READ("ClientH", "i", CCONTAINER_height),
  GB_PROPERTY_READ("ClientHeight", "i", CCONTAINER_height),

  GB_END_DECLARE
};



/*!---------

\HELP
c-container
class

\IN
interface

\TITLE
Container

\INHERITS
_c-control_

\DESC:fr
Cette classe est la racine de tous les contrles pouvant contenir d'autres
contrles.

\REMARK:fr
La classe <I>Container</I> ne peut tre instancie.

\SEE
_c-tabstrip_

----------*/

/*!---------

\HELP
c-container-children
function

\IN
c-container-p

\GROUP
_c-container_

\TITLE
Children

\TYPE
Property Collection

\DESC:fr
Retourne un objet virtuel permettant d'numrer les contrles contenus dans le <I>Container</I>.

\SYNTAX

<B>For Each</B> <I>contrle</I> <B>In</B> <I>objet</I>.<B>Children</B><BR>
...<BR>
<B>Next</B>

\ARG

<TR><TD><I>objet</I></TD><TD>Un objet de type <I>Container</I>.</TD><TR>
<TR><TD><I>contrle</I></TD><TD>Un contrle appartenant au conteneur.</TD><TR>

\SEE
_c-tabstrip_

----------*/

