/***************************************************************************

  CDialog.cpp

  The ListBox class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/


#define __CDIALOG_CPP

#include "common.h"

#include <qcolor.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qcolordialog.h>
#include <qfontdialog.h>

#include "gambas.h"
#include "CColor.h"

#include "CDialog.h"


static CFONT DIALOG_font;

static char *dialog_title = 0;
static char *dialog_filter = 0;
static char *dialog_path = 0;

static unsigned long dialog_color = 0;


BEGIN_METHOD_VOID(CDIALOG_init)

  DIALOG_font.klass = CLASS_Font;
  DIALOG_font.font = new QFont;

END_METHOD


BEGIN_METHOD_VOID(CDIALOG_exit)

  GB.FreeString(dialog_title);
  GB.FreeString(dialog_filter);
  GB.FreeString(dialog_path);

  delete DIALOG_font.font;

END_METHOD


BEGIN_PROPERTY(CDIALOG_title)

  if (READ_PROPERTY)
    GB.ReturnZeroString(dialog_title);
  else
    GB.StoreString(PROPERTY(GB_STRING), &dialog_title);

END_PROPERTY


BEGIN_PROPERTY(CDIALOG_filter)

  if (READ_PROPERTY)
    GB.ReturnZeroString(dialog_filter);
  else
    GB.StoreString(PROPERTY(GB_STRING), &dialog_filter);

END_PROPERTY


BEGIN_PROPERTY(CDIALOG_path)

  if (READ_PROPERTY)
    GB.ReturnZeroString(dialog_path);
  else
    GB.StoreString(PROPERTY(GB_STRING), &dialog_path);

END_PROPERTY


BEGIN_PROPERTY(CDIALOG_font)

  if (READ_PROPERTY)
    GB.ReturnObject(&DIALOG_font);
  else
    *(DIALOG_font.font) = CFONT_get_font(PROPERTY(CFONT *));

END_PROPERTY


BEGIN_PROPERTY(CDIALOG_color)

  if (READ_PROPERTY)
    GB.ReturnInteger((long)dialog_color);
  else
    dialog_color = PROPERTY(long);

END_PROPERTY


BEGIN_METHOD_VOID(CDIALOG_open_file)

  QString file;
  GB_STRING str;

  // le paramtre startWith est bugg !
  file = QFileDialog::getOpenFileName(QFileInfo(dialog_path).dirPath(), dialog_filter, 0, 0, dialog_title);

  if (file.isNull())
    GB.ReturnBoolean(true);
  else
  {
    (const char *)str.addr = file;
    str.len = file.length();
    GB.StoreString(str, &dialog_path);
    GB.ReturnBoolean(false);
  }

END_METHOD


BEGIN_METHOD_VOID(CDIALOG_save_file)

  QString file;
  GB_STRING str;

  // le paramtre startWith est bugg !
  file = QFileDialog::getSaveFileName(QFileInfo(dialog_path).dirPath(), dialog_filter, 0, 0, dialog_title);

  if (file.isNull())
    GB.ReturnBoolean(true);
  else
  {
    (const char *)str.addr = file;
    str.len = file.length();
    GB.StoreString(str, &dialog_path);
    GB.ReturnBoolean(false);
  }

END_METHOD


BEGIN_METHOD_VOID(CDIALOG_get_directory)

  QString file;
  GB_STRING str;

  file = QFileDialog::getExistingDirectory(dialog_path, 0, 0, dialog_title);

  if (file.isNull())
    GB.ReturnBoolean(true);
  else
  {
    (const char *)str.addr = file;
    str.len = file.length();
    GB.StoreString(str, &dialog_path);
    GB.ReturnBoolean(false);
  }

END_METHOD


BEGIN_METHOD_VOID(CDIALOG_get_color)

  QColor color;

  color = QColorDialog::getColor(dialog_color);

  if (!color.isValid())
    GB.ReturnBoolean(true);
  else
  {
    dialog_color = color.rgb() & 0xFFFFFF;
    GB.ReturnBoolean(false);
  }

END_METHOD


BEGIN_METHOD_VOID(CDIALOG_select_font)

  QFont qfont;
  bool ok;

  qfont = QFontDialog::getFont(&ok, *(DIALOG_font.font));

  if (!ok)
    GB.ReturnBoolean(true);
  else
  {
    *(DIALOG_font.font) = qfont;
    GB.ReturnBoolean(false);
  }

END_METHOD


GB_DESC CDialogDesc[] =
{
  GB_DECLARE("Dialog", 0), GB_VIRTUAL_CLASS(),

  GB_STATIC_METHOD("_init", NULL, CDIALOG_init, NULL),
  GB_STATIC_METHOD("_exit", NULL, CDIALOG_exit, NULL),

  GB_STATIC_METHOD("OpenFile", "b", CDIALOG_open_file, NULL),
  GB_STATIC_METHOD("SaveFile", "b", CDIALOG_save_file, NULL),
  GB_STATIC_METHOD("SelectDirectory", "b", CDIALOG_get_directory, NULL),
  GB_STATIC_METHOD("SelectColor", "b", CDIALOG_get_color, NULL),
  GB_STATIC_METHOD("SelectFont", "b", CDIALOG_select_font, NULL),

  GB_STATIC_PROPERTY("Title", "s", CDIALOG_title),
  GB_STATIC_PROPERTY("Path", "s", CDIALOG_path),
  GB_STATIC_PROPERTY("Filter", "s", CDIALOG_filter),
  GB_STATIC_PROPERTY("Color", "i", CDIALOG_color),
  GB_STATIC_PROPERTY("Font", "Font", CDIALOG_font),

  GB_END_DECLARE
};


