/***************************************************************************

  CGridView.cpp

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/


#define __CGRIDVIEW_CPP

#include "common.h"

#include <qapplication.h>
#include <qheader.h>

#include "gambas.h"

#include "CWidget.h"
#include "CPicture.h"
#include "CGridView.h"

static void *CLASS_GridItem;

//static long EVENT_Change;    /* selection change */
static long EVENT_Click;     /* simple click */
static long EVENT_Activate;  /* double click */
static long EVENT_Scroll;


/***************************************************************************

  class MyTableItem

***************************************************************************/

MyTableItem::MyTableItem(QTable *table)
: QTableItem(table, QTableItem::Never, 0)
{
  item.picture = NULL;
  item.item = this;
  item.ob.klass = (long)CLASS_GridItem;
  item.ob.ref = 1;
  align = AlignLeft;
}

MyTableItem::~MyTableItem()
{
  GB.Unref((void **)&item.picture);
}

void MyTableItem::setPicture(CPICTURE *pict)
{
  GB.StoreObject(pict, (void **)&item.picture);
  setPixmap(*pict->pixmap);
}

int MyTableItem::alignment() const
{
  return align | AlignVCenter;
}


/***************************************************************************

  class MyTable

***************************************************************************/

MyTable::MyTable(QWidget *parent) :
QTable(0, 0, parent)
{
}

void MyTable::paintFocus( QPainter *p, const QRect &r )
{
}

QSize MyTable::tableSize() const
{
  return QSize( columnPos( numCols() - 1 ) + columnWidth( numCols() - 1 ),
    rowPos( numRows() - 1 ) + rowHeight( numRows() - 1 ) );
}

void MyTable::columnWidthChanged( int col )
{
  updateContents( columnPos( col ), 0, contentsWidth(), contentsHeight() );
  QSize s(tableSize());
  int w = contentsWidth();
  resizeContents( s.width(), s.height() );


  if ( contentsWidth() < w )
	  repaintContents( s.width(), 0, w - s.width() + 1, contentsHeight(), TRUE );
  else
	  repaintContents( w, 0, s.width() - w + 1, contentsHeight(), FALSE );


  setLeftMargin(0); // updateGeometries();

  for ( int j = col; j < numCols(); ++j )
  {
  	for ( int i = 0; i < numRows(); ++i )
    {
	    QWidget *w = cellWidget( i, j );
	    if ( !w )
		    continue;
	    moveChild( w, columnPos( j ), rowPos( i ) );
	    w->resize( columnWidth( j ) - 1, rowHeight( i ) - 1 );
	  }
  }
}

void MyTable::rowHeightChanged( int row )
{
  updateContents( 0, rowPos( row ), contentsWidth(), contentsHeight() );
  QSize s( tableSize() );
  int h = contentsHeight();
  resizeContents( s.width(), s.height() );
  if ( contentsHeight() < h )
  	repaintContents( 0, contentsHeight(), contentsWidth(), h - s.height() + 1, TRUE );
  else
	  repaintContents( 0, h, contentsWidth(), s.height() - h + 1, FALSE );

  setLeftMargin(0); // updateGeometries();

  for ( int j = row; j < numRows(); ++j )
  {
	  for ( int i = 0; i < numCols(); ++i )
    {
	    QWidget *w = cellWidget( j, i );
	    if ( !w )
		    continue;
	    moveChild( w, columnPos( i ), rowPos( j ) );
	    w->resize( columnWidth( i ) - 1, rowHeight( j ) - 1 );
	  }
  }
}


/***************************************************************************

  GridView

***************************************************************************/

BEGIN_METHOD_VOID(CGRIDVIEW_init)

  CLASS_GridItem = GB.FindClass("Cell");

END_METHOD


BEGIN_METHOD(CGRIDVIEW_new, void *parent)

  MyTable *wid = new MyTable(CONTAINER(PARAM(parent)));

  //QObject::connect(wid, SIGNAL(currentChanged(int, int)), MANAGER, SLOT(changed()));
  QObject::connect(wid, SIGNAL(doubleClicked(int, int, int, const QPoint &)), MANAGER, SLOT(activated()));
  QObject::connect(wid, SIGNAL(clicked(int, int, int, const QPoint &)), MANAGER, SLOT(clicked()));
  //QObject::connect(wid, SIGNAL(contentsMoving(int, int)), MANAGER, SLOT(scrolled()));

  CWIDGET_new(wid, (void *)_object, "GridView");

  CONTROL->widget.background = QColorGroup::Base;
  CONTROL->row = -1;
  CONTROL->col = -1;

  wid->setSelectionMode(QTable::NoSelection);
  wid->setTopMargin(0);
  wid->verticalHeader()->hide();
  wid->setLeftMargin(0);
  wid->horizontalHeader()->hide();
  wid->show();

END_METHOD


BEGIN_PROPERTY(CGRIDVIEW_rows_columns)

  GB.ReturnObject(_object);

END_PROPERTY


BEGIN_PROPERTY(CGRIDVIEW_row)

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->currentRow());
  else
    THIS->setCurrentCell(PROPERTY(long), THIS->currentColumn());

END_PROPERTY


BEGIN_PROPERTY(CGRIDVIEW_column)

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->currentColumn());
  else
    THIS->setCurrentCell(THIS->currentRow(), PROPERTY(long));

END_PROPERTY


BEGIN_METHOD_VOID(CGRIDVIEW_clear)

  long rows = THIS->numRows();
  THIS->setNumRows(0);
  THIS->setNumRows(rows);

END_METHOD


BEGIN_METHOD(CGRIDVIEW_get, long row; long col)

  MyTableItem *item = CGridView::getItem(THIS, PARAM(row), PARAM(col), true);

  GB.ReturnObject(&(item->item));

END_METHOD


BEGIN_PROPERTY(CGRIDVIEW_current)

  MyTableItem *item = CGridView::getItem(THIS, THIS->currentRow(), THIS->currentColumn(), true);

  GB.ReturnObject(&(item->item));

END_PROPERTY


BEGIN_PROPERTY(CGRIDVIEW_grid)

  if (READ_PROPERTY)
    GB.ReturnBoolean(THIS->showGrid());
  else
    THIS->setShowGrid(PROPERTY(char));

END_PROPERTY


BEGIN_PROPERTY(CGRIDVIEW_scrollbars)

  long scroll;

  if (READ_PROPERTY)
  {
    scroll = 0;
    if (THIS->hScrollBarMode() == QScrollView::Auto)
      scroll += 1;
    if (THIS->vScrollBarMode() == QScrollView::Auto)
      scroll += 2;

    GB.ReturnInteger(scroll);
  }
  else
  {
    scroll = PROPERTY(long) & 3;
    THIS->setHScrollBarMode( (scroll & 1) ? QScrollView::Auto : QScrollView::AlwaysOff);
    THIS->setVScrollBarMode( (scroll & 2) ? QScrollView::Auto : QScrollView::AlwaysOff);
  }

END_PROPERTY


BEGIN_PROPERTY(CGRIDVIEW_client_width)

  GB.ReturnInteger(THIS->clipper()->width());

END_PROPERTY


BEGIN_PROPERTY(CGRIDVIEW_client_height)

  GB.ReturnInteger(THIS->clipper()->height());

END_PROPERTY


/***************************************************************************

  GridItem

***************************************************************************/

BEGIN_PROPERTY(CGRIDITEM_row)

  GB.ReturnInteger(ITEM->row());

END_PROPERTY


BEGIN_PROPERTY(CGRIDITEM_column)

  GB.ReturnInteger(ITEM->col());

END_PROPERTY


BEGIN_PROPERTY(CGRIDITEM_x)

  GB.ReturnInteger(ITEM->table()->columnPos(ITEM->col()) - ITEM->table()->contentsX());

END_PROPERTY


BEGIN_PROPERTY(CGRIDITEM_y)

  GB.ReturnInteger(ITEM->table()->rowPos(ITEM->row()) - ITEM->table()->contentsY());

END_PROPERTY


BEGIN_PROPERTY(CGRIDITEM_width)

  GB.ReturnInteger(ITEM->table()->columnWidth(ITEM->col()));

END_PROPERTY


BEGIN_PROPERTY(CGRIDITEM_height)

  GB.ReturnInteger(ITEM->table()->rowHeight(ITEM->row()));

END_PROPERTY


BEGIN_PROPERTY(CGRIDITEM_text)

  if (READ_PROPERTY)
    GB.ReturnZeroString(ITEM->text());
  else
  {
    ITEM->setText(PROPERTY(char *));
    ITEM->table()->updateCell(ITEM->row(), ITEM->col());
  }

END_PROPERTY


BEGIN_PROPERTY(CGRIDITEM_alignment)

  if (READ_PROPERTY)
    GB.ReturnInteger(ITEM->align);
  else
  {
    ITEM->align = PROPERTY(long);
    ITEM->table()->updateCell(ITEM->row(), ITEM->col());
  }

END_PROPERTY


BEGIN_PROPERTY(CGRIDITEM_picture)

  if (READ_PROPERTY)
    GB.ReturnObject(OBJECT(CGRIDITEM)->picture);
  else
    ITEM->setPicture(PROPERTY(CPICTURE *));

END_PROPERTY


BEGIN_METHOD_VOID(CGRIDITEM_ensure_visible)

  ITEM->table()->ensureCellVisible(THIS->currentRow(), THIS->currentColumn());

END_METHOD

BEGIN_METHOD_VOID(CGRIDITEM_clear)

  ITEM->table()->clearCell(THIS->currentRow(), THIS->currentColumn());

END_METHOD


/***************************************************************************

  GridRows

***************************************************************************/

BEGIN_METHOD(CGRIDROWS_get, long row)

  CGridView::checkRow(THIS, PARAM(row));
  CONTROL->row = PARAM(row);

  RETURN_SELF();

END_METHOD


BEGIN_PROPERTY(CGRIDROWS_count)

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->numRows());
  else
  {
    THIS->setNumRows(PROPERTY(long));
    CGridView::fillItems(THIS);
  }

END_PROPERTY


BEGIN_PROPERTY(CGRIDROWS_height)

  long row = CONTROL->row;

  if (row < 0)
    row = THIS->currentRow();

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->rowHeight(row));
  else
  {
    if (PROPERTY(long) < 0)
      THIS->adjustRow(row);
    else
      THIS->setRowHeight(row, PROPERTY(long));
  }

  CONTROL->row = -1;

END_PROPERTY


BEGIN_PROPERTY(CGRIDROWS_resizable)

  if (READ_PROPERTY)
    GB.ReturnBoolean(THIS->rowMovingEnabled());
  else
    THIS->setRowMovingEnabled(PROPERTY(char));

END_PROPERTY


/***************************************************************************

  GridCols

***************************************************************************/

BEGIN_METHOD(CGRIDCOLS_get, long col)

  CGridView::checkCol(THIS, PARAM(col));
  CONTROL->col = PARAM(col);

  RETURN_SELF();

END_METHOD


BEGIN_PROPERTY(CGRIDCOLS_count)

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->numCols());
  else
  {
    THIS->setNumCols(PROPERTY(long));
    CGridView::fillItems(THIS);
  }

END_PROPERTY


BEGIN_PROPERTY(CGRIDCOLS_width)

  long col = CONTROL->col;

  if (col < 0)
    col = THIS->currentColumn();

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->columnWidth(col));
  else
  {
    if (PROPERTY(long) < 0)
      THIS->adjustColumn(col);
    else
      THIS->setColumnWidth(col, PROPERTY(long));

  }

  CONTROL->col = -1;

END_PROPERTY


BEGIN_PROPERTY(CGRIDCOLS_resizable)

  if (READ_PROPERTY)
    GB.ReturnBoolean(THIS->columnMovingEnabled());
  else
    THIS->setColumnMovingEnabled(PROPERTY(char));

END_PROPERTY


/***************************************************************************

  Descriptions

***************************************************************************/

GB_DESC CGridItemDesc[] =
{
  GB_DECLARE("Cell", 0), GB_HOOK_NEW(NULL), GB_HOOK_FREE(NULL),

  GB_PROPERTY_READ("Row", "i", CGRIDITEM_row),
  GB_PROPERTY_READ("Column", "i", CGRIDITEM_column),
  GB_PROPERTY("Picture", "Picture", CGRIDITEM_picture),
  GB_PROPERTY("Text", "s", CGRIDITEM_text),
  GB_PROPERTY("Alignment", "i", CGRIDITEM_alignment),

  GB_PROPERTY_READ("X", "i", CGRIDITEM_x),
  GB_PROPERTY_READ("Y", "i", CGRIDITEM_y),
  GB_PROPERTY_READ("Left", "i", CGRIDITEM_x),
  GB_PROPERTY_READ("Top", "i", CGRIDITEM_y),
  GB_PROPERTY_READ("Width", "i", CGRIDITEM_width),
  GB_PROPERTY_READ("Height", "i", CGRIDITEM_height),
  GB_PROPERTY_READ("W", "i", CGRIDITEM_width),
  GB_PROPERTY_READ("H", "i", CGRIDITEM_height),

  GB_METHOD("Clear", NULL, CGRIDITEM_clear, NULL),
  GB_METHOD("EnsureVisible", NULL, CGRIDITEM_ensure_visible, NULL),

  GB_END_DECLARE
};


GB_DESC CGridRowsDesc[] =
{
  GB_DECLARE(".Grid.Rows", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("_get", ".Grid.Rows", CGRIDROWS_get, "(Row)i"),
  GB_PROPERTY("Count", "i", CGRIDROWS_count),
  GB_PROPERTY("Height", "i", CGRIDROWS_height),
  GB_PROPERTY("H", "i", CGRIDROWS_height),
  GB_PROPERTY("Resizable", "b", CGRIDROWS_resizable),

  GB_END_DECLARE
};


GB_DESC CGridColumnsDesc[] =
{
  GB_DECLARE(".Grid.Columns", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("_get", ".Grid.Columns", CGRIDCOLS_get, "(Column)i"),
  GB_PROPERTY("Count", "i", CGRIDCOLS_count),
  GB_PROPERTY("Width", "i", CGRIDCOLS_width),
  GB_PROPERTY("W", "i", CGRIDCOLS_width),
  GB_PROPERTY("Resizable", "b", CGRIDCOLS_resizable),

  GB_END_DECLARE
};


GB_DESC CGridViewDesc[] =
{
  GB_DECLARE("GridView", sizeof(CGRIDVIEW)), GB_INHERITS("Control"),

  GB_STATIC_METHOD("_init", NULL, CGRIDVIEW_init, NULL),
  GB_METHOD("_new", NULL, CGRIDVIEW_new, "Container;"),

  GB_PROPERTY("Rows", ".Grid.Rows", CGRIDVIEW_rows_columns),
  GB_PROPERTY("Columns", ".Grid.Columns", CGRIDVIEW_rows_columns),

  GB_METHOD("_get", "Cell", CGRIDVIEW_get, "(Row)i(Column)i"),

  GB_METHOD("Clear", NULL, CGRIDVIEW_clear, NULL),

  GB_PROPERTY("Row", "i", CGRIDVIEW_row),
  GB_PROPERTY("Column", "i", CGRIDVIEW_column),
  GB_PROPERTY_READ("Current", "Cell", CGRIDVIEW_current),
  GB_PROPERTY("Grid", "b", CGRIDVIEW_grid),
  GB_PROPERTY("ScrollBar", "i<Scroll>", CGRIDVIEW_scrollbars),
  GB_PROPERTY_READ("ClientWidth", "i", CGRIDVIEW_client_width),
  GB_PROPERTY_READ("ClientW", "i", CGRIDVIEW_client_width),
  GB_PROPERTY_READ("ClientHeight", "i", CGRIDVIEW_client_height),
  GB_PROPERTY_READ("ClientH", "i", CGRIDVIEW_client_height),

  GB_CONSTANT("_Properties", "s", CGRIDVIEW_PROPERTIES),
  GB_CONSTANT("_DefaultEvent", "s", "Click"),

  //GB_EVENT("Change", NULL, NULL, &EVENT_Change),
  GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
  GB_EVENT("Click", NULL, NULL, &EVENT_Click),
  //GB_EVENT("Scroll", NULL, NULL, &EVENT_Scroll),

  GB_END_DECLARE
};


/***************************************************************************

  class CGridView

***************************************************************************/

CGridView CGridView::manager;
bool CGridView::created;

void CGridView::checkRow(QTable *table, long row)
{
  if (row < 0 || row >= table->numRows())
    GB.Error("Bad row index %d", row);
}

void CGridView::checkCol(QTable *table, long col)
{
  if (col < 0 || col >= table->numCols())
    GB.Error("Bad column index %d", col);
}

void CGridView::check(QTable *table, long row, long col)
{
  checkRow(table, row);
  checkCol(table, col);
}

MyTableItem *CGridView::getItem(QTable *table, long row, long col, bool error)
{
  MyTableItem *item;

  if (error)
    check(table, row, col);

  //if (row < 0 || row >= table->numRows())
  //  return 0;
  //if (col < 0 || col >= table->numCols())
  //  return 0;

  item = (MyTableItem *)table->item(row, col);

  if (item == 0)
  {
    item = new MyTableItem(table);
    table->setItem(row, col, item);
    created = true;
  }
  else
    created = false;

  return item;
}

void CGridView::fillItems(QTable *table)
{
  long row, col;
  long cols;
  MyTableItem *item;

  row = table->numRows() - 1;
  cols = table->numCols();
  col =  cols - 1;

  if (col < 0)
    return;

  for(;;)
  {
    if (row < 0)
      return;

    if (col < 0)
      created = false;
    else
      item = getItem(table, row, col, false);

    if (!created)
    {
      if (col == (cols - 1))
        return;
      col = cols;
      row--;
    }
    /*
    else
      item->setText("Gambas");
    */

    col--;
  }
}

/*
void CGridView::changed(void)
{
  RAISE_EVENT(EVENT_Change, NULL);
}
*/

void CGridView::activated(void)
{
  RAISE_EVENT(EVENT_Activate, NULL);
}

void CGridView::clicked(void)
{
  RAISE_EVENT(EVENT_Click, NULL);
}

void CGridView::scrolled(void)
{
  /* ce signal est malheureusement dclench AVANT le scrolling
     et pas aprs => Cell.X et Cell.Y sont faux !
  */

  RAISE_EVENT(EVENT_Scroll, NULL);
}
