/***************************************************************************

  CImage.cpp

  The Image class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CIMAGE_CPP

#include "common.h"

#include <qapplication.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qframe.h>

#include "gambas.h"

#include "CPicture.h"
#include "CImage.h"

/*#define DEBUG_CBUTTON*/

#define ALIGN_MASK 0x3F


BEGIN_METHOD(CIMAGE_new, void *parent)

  QLabel *wid = new QLabel(CONTAINER(PARAM(parent)));

  QLABEL(_object) = wid;

  OBJECT(CIMAGE)->picture = NULL;

  CWIDGET_new(wid, (void *)_object, "Image");

  wid->setAlignment(Qt::AlignLeft | Qt::AlignTop);
  wid->show();

END_METHOD


BEGIN_METHOD_VOID(CIMAGE_free)

  PICTURE_set(&(OBJECT(CIMAGE)->picture), 0);

  CALL_METHOD_VOID(CWIDGET_delete);

END_METHOD


BEGIN_PROPERTY(CIMAGE_picture)

  QLabel *wid = QLABEL(_object);

  if (READ_PROPERTY)
    GB.ReturnObject(OBJECT(CIMAGE)->picture);
  else
  {
    wid->setPixmap(PICTURE_set(&(OBJECT(CIMAGE)->picture), PROPERTY(CPICTURE *)));

    /*
    if (wid->pixmap() != 0)
      if (!wid->hasScaledContents())
        wid->resize(wid->pixmap()->width(), wid->pixmap()->height());
    */
  }

END_PROPERTY


BEGIN_PROPERTY(CIMAGE_stretch)

  if (READ_PROPERTY)
    GB.ReturnBoolean(QLABEL(_object)->hasScaledContents());
  else
    QLABEL(_object)->setScaledContents(PROPERTY(char) != 0);

END_PROPERTY


/*
BEGIN_PROPERTY(CIMAGE_border)

  if (READ_PROPERTY)
    GB.ReturnBoolean(QLABEL(_object)->frameStyle() != QFrame::NoFrame);
  else
  {
    if (PROPERTY(char) != 0)
      QLABEL(_object)->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
    else
      QLABEL(_object)->setFrameStyle(QFrame::NoFrame);
  }

END_PROPERTY
*/

BEGIN_PROPERTY(CIMAGE_border)

  if (READ_PROPERTY)
    GB.ReturnInteger(WIDGET->frameStyle());
  else
    WIDGET->setFrameStyle(PROPERTY(long));

END_PROPERTY


BEGIN_PROPERTY(CIMAGE_alignment)

  if (READ_PROPERTY)
    GB.ReturnInteger(WIDGET->alignment() & ALIGN_MASK);
  else
    WIDGET->setAlignment((PROPERTY(int) & ALIGN_MASK) | Qt::WordBreak);

END_PROPERTY



GB_DESC CImageDesc[] =
{
  GB_DECLARE("Image", sizeof(CIMAGE)), GB_INHERITS("Control"),

  GB_METHOD("_new", NULL, CIMAGE_new, "Container;"),
  GB_METHOD("_free", NULL, CIMAGE_free, NULL),

  GB_PROPERTY("Picture", "Picture", CIMAGE_picture),
  GB_PROPERTY("Stretch", "b", CIMAGE_stretch),
  GB_PROPERTY("Border", "i<Border>", CIMAGE_border),
  GB_PROPERTY("Alignment", "i<Align>", CIMAGE_alignment),

  GB_CONSTANT("_Properties", "s", CIMAGE_PROPERTIES),

  GB_END_DECLARE
};


