/***************************************************************************

  CListBox.cpp

  The ListBox class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/


#define __CLISTBOX_CPP

#include "common.h"

#include <qapplication.h>
#include <qlistbox.h>

#include "gambas.h"

#include "CWidget.h"
#include "CPicture.h"
#include "CListBox.h"


static long EVENT_Select;    /* selection change */
static long EVENT_Click;     /* simple click */
static long EVENT_Activate;  /* double click */


BEGIN_METHOD(CLISTBOX_new, void *parent)

  QListBox *wid = new MyListBox(CONTAINER(PARAM(parent)));

  QObject::connect(wid, SIGNAL(selectionChanged()), &CListBox::manager, SLOT(selected()));
  QObject::connect(wid, SIGNAL(selected(int)), &CListBox::manager, SLOT(activated(int)));
  QObject::connect(wid, SIGNAL(highlighted(int)), &CListBox::manager, SLOT(clicked(int)));

  QLISTBOX(_object) = wid;

  CWIDGET_new(wid, (void *)_object, "ListBox");

  CONTROL->widget.background = QColorGroup::Base;

  wid->show();

END_METHOD

BEGIN_METHOD_VOID(CLISTBOX_clear)

  THIS->clear();

END_METHOD

BEGIN_METHOD(CLISTBOX_get, long index)

  GB.ReturnZeroString(THIS->text(PARAM(index)));

END_METHOD


BEGIN_METHOD(CLISTBOX_set, GB_STRING item; long index)

  long index = PARAM(index);
  long current;
  bool selected;

  if (index < 0 || index >= (long)THIS->count())
    return;

  selected = THIS->isSelected(PARAM(index));

  current = THIS->currentItem();
  THIS->changeItem(QString(GB.ToZeroString(PARAM(item))), index);
  THIS->setSelected(index, selected);

  THIS->setCurrentItem(current);

END_METHOD


BEGIN_METHOD(CLISTBOX_add, GB_STRING item; long pos)

  long pos = GB.IsMissing(2) ? -1 : PARAM(pos);

  //if (GB.IsMissing(2))
  THIS->insertItem(GB.ToZeroString(PARAM(item)), pos);
  //else
  //{
    //QLISTBOX(_object)->insertItem(PIXMAP_get(GB.ToZeroString(PARAM(picture))),
    //  GB.ToZeroString(PARAM(item)), pos);
  //}

  //QLISTBOX(_object)->updateGeometry();

END_METHOD


BEGIN_METHOD(CLISTBOX_remove, long pos)

  THIS->removeItem(PARAM(pos));

END_METHOD


BEGIN_METHOD_VOID(CLISTBOX_sort)

  THIS->sort();

END_METHOD


BEGIN_PROPERTY(CLISTBOX_count)

  GB.ReturnInteger(THIS->count());

END_PROPERTY


BEGIN_PROPERTY(CLISTBOX_index)

  long index;

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->currentItem());
  else
  {
    index = PROPERTY(long);
    if (index < 0 || index >= (long)THIS->count())
    {
      THIS->clearSelection();
      return;
    }

    //if (THIS->selectionMode() == QListBox::Multi || THIS->selectionMode() == QListBox::Extended)
    //{
      //THIS->clearSelection();
      //THIS->setSelected(index, true);
    //}

    THIS->setCurrentItem(index);
    THIS->ensureCurrentVisible();
  }

END_PROPERTY


BEGIN_PROPERTY(CLISTBOX_current)

  int current = THIS->currentItem();

  if (current < 0)
    GB.ReturnNull();
  else
    GB.ReturnZeroString(THIS->text(current));

END_PROPERTY


BEGIN_PROPERTY(CLISTBOX_mode)

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->selectionMode());
  else
    QLISTBOX(_object)->setSelectionMode((QListBox::SelectionMode)PROPERTY(long));

END_PROPERTY


BEGIN_PROPERTY(CLISTBOX_selected)

  RETURN_SELF();

END_PROPERTY


BEGIN_METHOD(CLISTBOX_selected_get, long index)

  GB.ReturnBoolean(THIS->isSelected(PARAM(index)));

END_METHOD


BEGIN_METHOD(CLISTBOX_selected_set, char selected; long index)

  THIS->setSelected(PARAM(index), PARAM(selected) != 0);

END_METHOD



GB_DESC CListBoxSelectedDesc[] =
{
  GB_DECLARE(".ListBox.Selected", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("_get", "b", CLISTBOX_selected_get, "(Index)i"),
  GB_METHOD("_put", NULL, CLISTBOX_selected_set, "(Selected)b(Index)i"),

  GB_END_DECLARE
};


GB_DESC CListBoxDesc[] =
{
  GB_DECLARE("ListBox", sizeof(CLISTBOX)), GB_INHERITS("Control"),

  GB_CONSTANT("None", "i", QListBox::NoSelection),
  GB_CONSTANT("Single", "i", QListBox::Single),
  GB_CONSTANT("Multi", "i", QListBox::Multi),
  GB_CONSTANT("Extended", "i", QListBox::Extended),

  GB_METHOD("_new", NULL, CLISTBOX_new, "Container;"),
  GB_METHOD("_get", "s", CLISTBOX_get, "(Index)i"),
  GB_METHOD("_put", NULL, CLISTBOX_set, "(Text)s(Index)i"),

  GB_METHOD("Clear", NULL, CLISTBOX_clear, NULL),
  GB_METHOD("Add", NULL, CLISTBOX_add, "(Text)s[(After)i]"),
  GB_METHOD("Sort", NULL, CLISTBOX_sort, NULL),
  GB_METHOD("Remove", NULL, CLISTBOX_remove, "(Index)i"),

  /*
  GB_METHOD("Find", "i", CLISTBOX_find, 1, 1, "s"),
  GB_METHOD("Select", NULL, CLISTBOX_select, 1, 1, "i"),
  GB_METHOD("Invert", NULL, CLISTBOX_invert, 0, 0, NULL),
  */

  GB_PROPERTY_READ("Count", "i", CLISTBOX_count),
  GB_PROPERTY_READ("Current", "s", CLISTBOX_current),
  GB_PROPERTY("Index", "i", CLISTBOX_index),

  GB_PROPERTY("Mode", "i<ListBox,None,Single,Multi,Extended>", CLISTBOX_mode),

  GB_PROPERTY_READ("Selected", ".ListBox.Selected", CLISTBOX_selected),

  GB_CONSTANT("_Properties", "s", CLISTBOX_PROPERTIES),
  GB_CONSTANT("_DefaultEvent", "s", "Click"),

  GB_EVENT("Select", NULL, NULL, &EVENT_Select),
  GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
  GB_EVENT("Click", NULL, NULL, &EVENT_Click),

  GB_END_DECLARE
};


MyListBox::MyListBox(QWidget *parent) :
  QListBox(parent)
{
  //clearWFlags(Qt::WRepaintNoErase | Qt::WNorthWestGravity);
}

void MyListBox::resizeEvent( QResizeEvent *e )
{
  //qDebug("MyListBox::resizeEvent");
  QScrollView::resizeEvent(e);
  triggerUpdate(true);
  doLayout();
	viewport()->repaint( FALSE );
  ensureCurrentVisible();
}

/* Classe CListBox */

CListBox CListBox::manager;

void CListBox::selected(void)
{
  RAISE_EVENT(EVENT_Select, NULL);
}

void CListBox::activated(int index)
{
  RAISE_EVENT(EVENT_Activate, NULL);
}

void CListBox::clicked(int index)
{
  RAISE_EVENT(EVENT_Click, NULL);
}

