/***************************************************************************

  CMenu.cpp

  The Menu class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CMENU_CPP

#include "common.h"

#include <qapplication.h>
#include <qmainwindow.h>
#include <qmenubar.h>
#include <qaccel.h>
#include <qcursor.h>

#include "gambas.h"

#include "CWidget.h"
#include "CWindow.h"
#include "CMenu.h"

//#define DEBUG_MENU

PRIVATE void *CLASS_Menu;
PRIVATE void *CLASS_Window;

PRIVATE long EVENT_Click, EVENT_Show;

PRIVATE long menu_id = 0;


PRIVATE void set_text(CMENU *item, char *text)
{
#ifdef DEBUG_MENU
  qDebug("set_text: item->text = '%s' text = '%s'", item->text, text);
#endif

  if (item->text != NULL)
  {
    GB.FreeString(item->text);
    item->text = NULL;
  }

  if (text == NULL)
    return;

  item->text = GB.CopyString(text, 0);
}


PRIVATE void hide_menu(CMENU *item)
{
  if (!CWIDGET_test_flag(item, WF_VISIBLE))
    return;

#ifdef DEBUG_MENU
  qDebug("hide_menu: item = %p (%d)", item, item->id);
#endif

  //index = item->container->indexOf(item->id);
  //if (index >= 0)
  item->container->removeItem(item->id);

  if (CMENU_is_top(item))
  {
    if (item->container->count() == 0)
      ((QMenuBar *)item->container)->hide();
  }

  CWIDGET_clear_flag(item, WF_VISIBLE);
}



PRIVATE void show_menu(CMENU *item)
{
  CMENU *parent;
  int pos;
  bool change;


  if (CWIDGET_test_flag(item, WF_VISIBLE))
    return;

#ifdef DEBUG_MENU
  qDebug("show_menu: item = %p (%d) parent = %p (%d)", item, item->id, item->parent, item->parent ? item->parent->id : 0);
#endif

  parent = item->parent;

  if (parent)
  {
    if (QPOPUPMENU(parent) == 0)
    {
#ifdef DEBUG_MENU
      qDebug("show_menu: add_popup");
#endif

      QPopupMenu *popup = new QPopupMenu(item->toplevel);

      change = CMENU_is_visible(parent);

      if (change)
        hide_menu(parent);

      QPOPUPMENU(parent) = popup;

      //parent->dict = new QIntDict<CMENU>;

      QObject::connect(popup, SIGNAL(activated(int)), &CMenu::manager, SLOT(activated(int)));
      QObject::connect(popup, SIGNAL(aboutToShow()), &CMenu::manager, SLOT(shown()));
      QObject::connect(popup, SIGNAL(destroyed()), &CMenu::manager, SLOT(destroy()));

      // un popupmenu est rfrenc deux fois !
      // Et il ne faut pas effacer le tag !
      //qDebug("*** show_menu %p (Popup)", parent);
      CWIDGET_new(popup, (void *)parent, "Menu", true, true);

      if (change)
        show_menu(parent);
    }

    item->container = QPOPUPMENU(parent);
  }

  for (pos = 0; pos < (int)item->container->count(); pos++)
  {
    if (item->container->idAt(pos) >= item->id)
      break;
  }

  if (item->text == NULL || *item->text == 0)
  {
#ifdef DEBUG_MENU
    qDebug("show_menu: insertSeparator(%d)", pos);
#endif
    //item->container->insertSeparator(pos);
    item->container->insertItem((QWidget *)0, item->id, pos);
  }
  else if (QPOPUPMENU(item) != 0)
  {
#ifdef DEBUG_MENU
    qDebug("show_menu: insertItem('%s', %p, %d, %d)", item->text, QPOPUPMENU(item), item->id, pos);
#endif
    item->container->insertItem(item->text, QPOPUPMENU(item), item->id, pos);
  }
  else
  {
#ifdef DEBUG_MENU
    qDebug("show_menu: insertItem('%s', %d, %d)", item->text, item->id, pos);
#endif
    item->container->insertItem(item->text, item->id, pos);
  }

  if (CMENU_is_top(item))
    ((QMenuBar *)item->container)->show();


  //if (pos < 0)
  //  item->pos = item->container->indexOf(item->id);

  item->pos = pos;
  item->container->setAccel(item->accel, item->id);

#ifdef DEBUG_MENU
  qDebug("show_menu: pos = %d", pos);
#endif

  CWIDGET_set_flag(item, WF_VISIBLE);
}


PRIVATE void delete_menu(CMENU *item)
{
  hide_menu(item);
  if (QPOPUPMENU(item))
  {
    delete QPOPUPMENU(item);
    QPOPUPMENU(item) = 0;
  }
}


BEGIN_METHOD_VOID(CMENU_init)

  CLASS_Menu = GB.FindClass("Menu");
  CLASS_Window = GB.FindClass("Window");

END_METHOD


BEGIN_METHOD(CMENU_new, void *parent)

  CMENU *item = OBJECT(CMENU);
  void *parent = PARAM(parent);
  QMainWindow *qwindow;
  QList<CMENU> **list;

  //printf("CMENU_new %p\n", _object);

  if (GB.Is(parent, CLASS_Window))
    qwindow = QMAINWINDOW(parent);
  else if (GB.Is(parent, CLASS_Menu))
    qwindow = (QMainWindow *)((CMENU *)parent)->toplevel;
  else
    GB.Error("Type mismatch. The parent control of a Menu must be a Window or another Menu.");

  GB.CheckObject(parent);

  QPOPUPMENU(item) = 0;
  item->widget.tag.type = GB_T_NULL;
  item->children = NULL;
  item->text = NULL;
  item->picture = NULL;
  item->toplevel = qwindow;
  item->accel = 0;

  menu_id++;
  item->id = menu_id;

  CWIDGET_clear_flag(item, WF_VISIBLE);

  if (GB.Is(parent, CLASS_Menu))
  {
    CMENU *pmenu = (CMENU *)parent;

    item->container = NULL; /* plus tard */
    item->parent = pmenu;

    list = &(pmenu->children);

    GB.Ref(parent);
  }
  else
  {
    item->container = qwindow->menuBar();
    item->parent = NULL;

    list = &(((CWINDOW *)parent)->menu);
  }

  CMenu::dict.insert(item->id, item);

  if (*list == NULL)
    *list = new QList<CMENU>;

  (*list)->append(item);

#ifdef DEBUG_MENU
  qDebug("CMENU_new: item = %p (%d) parent = %p (%d)", item, item->id, item->parent, item->parent ? item->parent->id : 0);
#endif

  //qDebug("*** CMENU_new %p", _object);
  GB.Ref(_object);

  show_menu(item);

END_METHOD


BEGIN_METHOD_VOID(CMENU_free)

  CMENU *item = OBJECT(CMENU);

#ifdef DEBUG_MENU
  qDebug("CMENU_free: item = %p '%s' (%d) parent = %p (%d)", item, item->text, item->id, item->parent, item->parent ? item->parent->id : 0);
#endif

  set_text(item, NULL);

  GB.Store(GB_T_VARIANT, NULL, &(OBJECT(CWIDGET)->tag));

  CMenu::dict.remove(item->id);

  if (item->parent)
  {
    item->parent->children->removeRef(item);
    GB.Unref((void **)&item->parent);
  }
  else
  {
    //printf("item = %p  item->toplevel = %p\n", item, item->toplevel);
    //fflush(NULL);
    CWINDOW *parent = (CWINDOW *)CWidget::getReal(item->toplevel);
    if (parent)
    {
      parent->menu->removeRef(item);
      //GB.Unref((void **)&parent);
    }
  }

  if (item->children)
  {
    delete item->children;
    item->children = NULL;
  }

  //qDebug("< CMENU_free");

  //if (item->parent != NULL)
  //  if (item->parent->dict != NULL)
  //    item->parent->dict->remove(item->id);

  //if (item->children != NULL)
  //{
  //  delete item->dict;
  //  item->dict = NULL;
  //}

END_METHOD


BEGIN_PROPERTY(CMENU_text)

  CMENU *item = OBJECT(CMENU);

  if (READ_PROPERTY)
  {
    GB.ReturnZeroString(item->text);
  }
  else
  {
    bool change = CMENU_is_visible(item);

    if (change)
      hide_menu(item);

    set_text(item, GB.ToZeroString(PROPERTY(GB_STRING)));

    if (change)
      show_menu(item);

    //change_menu_item(item, GB.ToZeroString(PROPERTY(GB_STRING)));
  }

END_PROPERTY


BEGIN_PROPERTY(CMENUITEM_enabled)

  CMENU *item = OBJECT(CMENU);

  if (item->parent == 0)
  {
    if (READ_PROPERTY)
      GB.ReturnBoolean(((QMenuBar *)item->container)->isItemEnabled(item->id));
    else
      ((QMenuBar *)item->container)->setItemEnabled(item->id, PROPERTY(char) != 0);
  }
  else
  {
    if (READ_PROPERTY)
      GB.ReturnBoolean(((QPopupMenu *)item->container)->isItemEnabled(item->id));
    else
      ((QPopupMenu *)item->container)->setItemEnabled(item->id, PROPERTY(char) != 0);
  }

END_PROPERTY


BEGIN_PROPERTY(CMENUITEM_checked)

  CMENU *item = OBJECT(CMENU);

  if (item->parent == 0)
  {
    if (READ_PROPERTY)
      //GB.ReturnBoolean(((QMenuBar *)item->container)->isItemChecked(item->id));
      GB.ReturnBoolean(0);
    //else
      //((QMenuBar *)item->container)->setItemChecked(item->id, PROPERTY(char) != 0);
  }
  else
  {
    if (READ_PROPERTY)
      GB.ReturnBoolean(((QPopupMenu *)item->container)->isItemChecked(item->id));
    else
      ((QPopupMenu *)item->container)->setItemChecked(item->id, PROPERTY(char) != 0);
  }

END_PROPERTY


BEGIN_PROPERTY(CMENU_shortcut)

  CMENU *item = OBJECT(CMENU);
  QPopupMenu *parent;

  if (CMENU_is_popup(item))
  {
    if (READ_PROPERTY)
      GB.ReturnNull();

    return;
  }

  parent = (QPopupMenu *)item->container;

  if (READ_PROPERTY)
  {
    if (item->accel == 0)
      GB.ReturnNull();
    else
      GB.ReturnCopyString(QAccel::keyToString(item->accel), 0);
  }
  else
  {
    QString s(GB.ToZeroString(PROPERTY(GB_STRING)));
    item->accel = QAccel::stringToKey(s);
    parent->setAccel(item->accel, item->id);
  }

END_PROPERTY


BEGIN_PROPERTY(CMENU_visible)

  CMENU *item = OBJECT(CMENU);

  if (READ_PROPERTY)
    GB.ReturnBoolean(CWIDGET_test_flag(item, WF_VISIBLE));
  else
  {
    if (PROPERTY(char))
      show_menu(item);
    else
      hide_menu(item);
  }

END_PROPERTY


BEGIN_METHOD_VOID(CMENU_delete)

  CMENU *item = OBJECT(CMENU);

  delete_menu(item);

END_METHOD


BEGIN_PROPERTY(CMENU_count)

  CMENU *item = OBJECT(CMENU);

  if (CMENU_is_popup(item))
    GB.ReturnInteger(item->children->count());
  else
    GB.ReturnInteger(0);

END_PROPERTY


BEGIN_METHOD_VOID(CMENU_next)

  CMENU *item = OBJECT(CMENU);
  unsigned int index;

  if (item->children == NULL)
  {
    GB.StopEnum();
    return;
  }

  index = ENUM(int);

  if (index >= item->children->count())
  {
    GB.StopEnum();
    return;
  }

  GB.ReturnObject(item->children->at(index));

  ENUM(int) = index + 1;

END_PROPERTY


BEGIN_METHOD_VOID(CMENU_clear)

  CMENU *item = OBJECT(CMENU);
  CMENU *child;

  if (CMENU_is_popup(item))
  {
    QListIterator<CMENU> it(*(item->children));

    while ((child = it.current()))
    {
      ++it;
      delete_menu(child);
    }
  }

END_PROPERTY


BEGIN_METHOD_VOID(CMENU_popup)

  CMENU *item = OBJECT(CMENU);

  if (CMENU_is_popup(item))
  {
    QPopupMenu *popup = QPOPUPMENU(item);

    if (popup)
      popup->exec(QCursor::pos());
  }

END_METHOD



/*
BEGIN_METHOD_VOID(CMENUITEM_remove)

  CMENUITEM *item = OBJECT(CMENUITEM);
  QPopupMenu *qmenu = QPOPUPMENU(item->menu);

  qmenu->removeItem(item->id);
  GB.Detach(_object);

END_METHOD
*/



/*
BEGIN_METHOD(CMENU_add, GB_STRING caption; GB_STRING key; GB_STRING before)

  long before = -1;
  long id;
  char *key = GB.ToZeroString(PARAM(key));

  if (!GB.IsMissing(3))
  {
    before = id_from_key(OBJECT(CMENU), PARAM(before));
    before = QPOPUPMENU(_object)->indexOf(before);
  }

  id = QPOPUPMENU(_object)->insertItem(GB.ToZeroString(PARAM(caption)), -1, before);

  if (OBJECT(CMENU)->keyToIndex->find(key) != 0)
    GB.Error("Key already used");

  OBJECT(CMENU)->keyToIndex->insert(key, (void *)id);

  GB.ReturnInteger(QPOPUPMENU(_object)->indexOf(id));

END_METHOD


BEGIN_METHOD(CMENU_add_separator, GB_STRING before)

  long before = -1;
  long id;

  if (!GB.IsMissing(1))
  {
    before = id_from_key(OBJECT(CMENU), PARAM(before));
    before = QPOPUPMENU(_object)->indexOf(before);
  }

  id = QPOPUPMENU(_object)->insertSeparator(before);

  GB.ReturnInteger(QPOPUPMENU(_object)->indexOf(id));

END_METHOD


BEGIN_METHOD(CMENU_add_menu, GB_STRING caption; CMENU *sub_menu; GB_STRING before)

  long before = -1;
  long id;
  CMENU *sub_menu = PARAM(sub_menu);

  if (!GB.IsMissing(3))
  {
    before = id_from_key(OBJECT(CMENU), PARAM(before));
    before = QPOPUPMENU(_object)->indexOf(before);
  }

  id = QPOPUPMENU(_object)->insertItem(GB.ToZeroString(PARAM(caption)), QPOPUPMENU(sub_menu), -1, before);

  GB.ReturnInteger(QPOPUPMENU(_object)->indexOf(id));

END_METHOD


BEGIN_METHOD(CMENU_get, GB_STRING key)

  OBJECT(CMENU)->id = id_from_key(OBJECT(CMENU), PARAM(key));
  GB.ReturnObject(OBJECT(void));

END_METHOD
*/

/*
BEGIN_PROPERTY(CMENU_item_index)

  long id = OBJECT(CMENU)->id;
  QPopupMenu *menu = QPOPUPMENU(_object);

  GB.ReturnInteger(menu->indexOf(id));

END_PROPERTY
*/

/*
GB_DESC CMenuItemDesc[] =
{
  GB_DECLARE("MenuItem", sizeof(CMENUITEM)),

  GB_METHOD("_new", NULL, CMENUITEM_new, "Menu"),
  GB_METHOD("_free", NULL, CMENUITEM_free, NULL),

  GB_PROPERTY("Caption", "s", CMENUITEM_text),
  GB_PROPERTY("Text", "s", CMENUITEM_text),
  GB_PROPERTY("Enabled", "b", CMENUITEM_enabled),
  GB_PROPERTY("Checked", "b", CMENUITEM_checked),
  //GB_PROPERTY("Picture", "Picture", CMENUITEM_picture),
  //GB_PROPERTY("Menu", "Menu", CMENUITEM_menu),

  //GB_PROPERTY_READ("Index", "i", CMENU_item_index),

  GB_METHOD("Delete", NULL, CMENUITEM_remove, NULL),

  GB_EVENT("Click", NULL, NULL, &EVENT_Click),

  GB_END_DECLARE
};
*/

BEGIN_PROPERTY(CMENU_children)

  RETURN_SELF();

END_PROPERTY


GB_DESC CMenuChildrenDesc[] =
{
  GB_DECLARE(".Menu.Children", sizeof(CMENU)), GB_VIRTUAL_CLASS(),

  GB_METHOD("_next", "Menu", CMENU_next, NULL),
  GB_METHOD("Clear", NULL, CMENU_clear, NULL),
  GB_PROPERTY_READ("Count", "i", CMENU_count),

  GB_END_DECLARE
};


GB_DESC CMenuDesc[] =
{
  GB_DECLARE("Menu", sizeof(CMENU)), //GB_HOOK_CHECK(CWIDGET_check),

  GB_STATIC_METHOD("_init", NULL, CMENU_init, NULL),
  GB_METHOD("_new", NULL, CMENU_new, "o"),
  GB_METHOD("_free", NULL, CMENU_free, NULL),
  //GB_METHOD("_get", "MenuItem", CMENU_get, "s"),

  //GB_PROPERTY("Name", "s", CWIDGET_name),

  //GB_PROPERTY_READ("Count", "i", CMENU_count),

  //GB_PROPERTY_READ("Parent", "Control", CWIDGET_parent),

  GB_PROPERTY("Caption", "s", CMENU_text),
  GB_PROPERTY("Text", "s", CMENU_text),
  GB_PROPERTY("Enabled", "b", CMENUITEM_enabled),
  GB_PROPERTY("Checked", "b", CMENUITEM_checked),
  GB_PROPERTY("Tag", "v", CWIDGET_tag),
  //GB_PROPERTY("Picture", "Picture", CMENUITEM_picture),
  GB_PROPERTY("Shortcut", "s", CMENU_shortcut),
  GB_PROPERTY("Visible", "b", CMENU_visible),

  GB_PROPERTY_READ("Children", ".Menu.Children", CMENU_children),
  //GB_PROPERTY_READ("Index", "i", CMENU_item_index),

  GB_CONSTANT("_Properties", "s", CMENU_PROPERTIES),

  GB_METHOD("Popup", NULL, CMENU_popup, NULL),
  GB_METHOD("Delete", NULL, CMENU_delete, NULL),

  GB_EVENT("Destroy", NULL, NULL, NULL), // Must be first
  GB_EVENT("Click", NULL, NULL, &EVENT_Click),
  GB_EVENT("Show", NULL, NULL, &EVENT_Show),

  GB_END_DECLARE
};



/* CMenu class */

CMenu CMenu::manager;
CMenuDict CMenu::dict;

void CMenu::activated(int id)
{
  CMENU *menu = CMenu::dict[id];

  GB.Raise(menu, EVENT_Click, NULL);
}


void CMenu::shown(void)
{
  RAISE_EVENT(EVENT_Show, NULL);
}


void CMenu::unrefChildren(CMenuList *list)
{
  CMENU *child;

  //printf("> CMenu::destroy: %p\n", menu);

  QListIterator<CMENU> it(*list);

  while ((child = it.current()))
  {
    ++it;

    /* ne drfrencer que les enfant simples. */
    if (child->children == 0)
    {
      GB.Detach(child);
      //qDebug("*** CMenu::destroy %p (child)", child);
      GB.Unref((void **)&child);
    }
  }
}


void CMenu::destroy(void)
{
  GET_SENDER(ob);
  CMENU *menu = (CMENU *)ob;

  //qDebug("*** CMenu::destroy %p", menu);

  unrefChildren(menu->children);

  GB.Unref((void **)&menu);

  //menu->dict = dict;

  //printf("< CMenu::destroy: %p\n", menu);
}

