/***************************************************************************

  CMouse.cpp

  The Mouse and Cursor classes

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CMOUSE_CPP

#include "common.h"

#include <qapplication.h>
#include <qpixmap.h>
#include <qcursor.h>
#include <qnamespace.h>

#include "gambas.h"

#include "CPicture.h"
#include "CMouse.h"


//static CCURSOR PredefinedCursor[LastCursor + 1] = { { { 0, 0 }, NULL, NULL } };
//static long MouseClassID;

BEGIN_METHOD(CCURSOR_new, CPICTURE *picture; long x; long y)

  //OBJECT(CCURSOR)->name = NULL;
  OBJECT(CCURSOR)->cursor = NULL;

  if (GB.IsMissing(2)) PARAM(x) = -1;
  if (GB.IsMissing(3)) PARAM(y) = -1;

  //OBJECT(CCURSOR)->name = GB.CopyString(PARAM(path).addr, PARAM(path).len);
  OBJECT(CCURSOR)->cursor = new QCursor(*(PARAM(picture)->pixmap), PARAM(x), PARAM(y));

END_METHOD


BEGIN_METHOD_VOID(CCURSOR_delete)

  //GB.FreeString(OBJECT(CCURSOR)->name);
  if (OBJECT(CCURSOR)->cursor != NULL)
    delete OBJECT(CCURSOR)->cursor;

END_METHOD


// BEGIN_METHOD(CCURSOR_get, long shape)
//
//   long shape = PARAM(shape);
//   CCURSOR *p;
//
//   if (shape < 0 || shape > LastCursor)
//     GB.ReturnObject(NULL);
//
//   p = &PredefinedCursor[shape];
//   if (p->ob.klass == 0)
//   {
//     p->ob.klass = MouseClassID;
//     p->cursor = new QCursor(shape);
//     GB.Ref(p);
//   }
//
//   GB.ReturnObject(p);
//
// END_METHOD


BEGIN_PROPERTY(CMOUSE_x)

  GB.ReturnInteger(QCursor::pos().x());

END_PROPERTY

BEGIN_PROPERTY(CMOUSE_y)

  GB.ReturnInteger(QCursor::pos().y());

END_PROPERTY

BEGIN_METHOD(CMOUSE_move, long x; long y)

  QCursor::setPos(PARAM(x), PARAM(y));

END_PROPERTY


GB_DESC CMouseDesc[] =
{
  GB_DECLARE("Mouse", sizeof(CCURSOR)),

  GB_STATIC_PROPERTY_READ("X", "i", CMOUSE_x),
  GB_STATIC_PROPERTY_READ("Y", "i", CMOUSE_y),
  GB_STATIC_METHOD("Move", NULL, CMOUSE_move, "(X)i(Y)i"),

  GB_CONSTANT("Default", "i", CMOUSE_DEFAULT),
  GB_CONSTANT("Blank", "i", BlankCursor),
  GB_CONSTANT("Arrow", "i", ArrowCursor),
  GB_CONSTANT("Cross", "i", CrossCursor),
  GB_CONSTANT("Wait", "i", WaitCursor),
  GB_CONSTANT("Text", "i", IbeamCursor),
  GB_CONSTANT("SizeAll", "i", SizeAllCursor),
  GB_CONSTANT("SizeH", "i", SizeHorCursor),
  GB_CONSTANT("SizeV", "i", SizeVerCursor),
  GB_CONSTANT("SizeN", "i", SizeVerCursor),
  GB_CONSTANT("SizeS", "i", SizeVerCursor),
  GB_CONSTANT("SizeW", "i", SizeHorCursor),
  GB_CONSTANT("SizeE", "i", SizeHorCursor),
  GB_CONSTANT("SizeNW", "i", SizeFDiagCursor),
  GB_CONSTANT("SizeSE", "i", SizeFDiagCursor),
  GB_CONSTANT("SizeNE", "i", SizeBDiagCursor),
  GB_CONSTANT("SizeSW", "i", SizeBDiagCursor),
  GB_CONSTANT("SizeNWSE", "i", SizeFDiagCursor),
  GB_CONSTANT("SizeNESW", "i", SizeBDiagCursor),
  GB_CONSTANT("SplitH", "i", SplitHCursor),
  GB_CONSTANT("SplitV", "i", SplitVCursor),
  GB_CONSTANT("Pointing", "i", PointingHandCursor),

  GB_CONSTANT("Left", "i", Qt::LeftButton),
  GB_CONSTANT("Right", "i", Qt::RightButton),
  GB_CONSTANT("Middle", "i", Qt::MidButton),
  GB_CONSTANT("Shift", "i", Qt::ShiftButton),
  GB_CONSTANT("Control", "i", Qt::ControlButton),
  GB_CONSTANT("Alt", "i", Qt::AltButton),

  GB_METHOD("_new", NULL, CCURSOR_new, "(Picture)Picture;[(X)i(Y)i]"),
  GB_METHOD("_free", NULL, CCURSOR_delete, NULL),

  //GB_METHOD("$GET", "Cursor", CCURSOR_get, "i"),

  //GB_PROPERTY_READ("Name", "s", CCURSOR_name),

  GB_END_DECLARE
};

