/***************************************************************************

  CPicture.cpp

  The Picture class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CPICTURE_CPP

#include "common.h"

#include <qpixmap.h>
#include <qnamespace.h>
#include <qdict.h>

#include "gambas.h"

#include "CPicture.h"

static QDict<CPICTURE> dict;


MyPixmap::MyPixmap(CPICTURE *picture)
: QPixmap()
{
  this->picture = picture;
  picture->pixmap = this;
  //GB.Ref(picture);
  //qDebug("MyPixmap %p", this);
}


MyPixmap::~MyPixmap()
{
  //qDebug("~MyPixmap %p", this);
  //GB.Unref(picture);
}


QPixmap PICTURE_set(CPICTURE **store, CPICTURE *picture)
{
  GB.StoreObject(picture, (void **)store);

  if (picture)
    return *(picture->pixmap);
  else
    return 0;
}


CPICTURE *get_picture(char *key)
{
  CPICTURE *pict;
  char *addr;
  long len;

  pict = dict[key];
  if (!pict)
  {
    GB.New((void **)&pict, "Picture", 0, 0);
    GB.Ref(pict);

    GB.LoadFile(key, &addr, &len);
    pict->pixmap->loadFromData((const uchar *)addr, (uint)len);
    GB.ReleaseFile(&addr, len);

    dict.insert(key, pict);
  }

  return pict;
}


BEGIN_METHOD(CPICTURE_get, GB_STRING path)

  char *key = GB.ToZeroString(PARAM(path));

  GB.ReturnObject(get_picture(key));

END_METHOD


BEGIN_METHOD_VOID(CPICTURE_exit)

  QDictIterator<CPICTURE> it(dict);
  CPICTURE *pict;

  while (it.current())
  {
    //delete it.current()->pixmap;
    pict = it.current();
    GB.Unref((void **)&pict);
    ++it;
  }

END_METHOD


BEGIN_METHOD_VOID(CPICTURE_new)

  new MyPixmap(OBJECT(CPICTURE));
  //qDebug("CPICTURE_new: pixmap = %p", OBJECT(CPICTURE)->pixmap);

END_METHOD


BEGIN_METHOD_VOID(CPICTURE_free)

  //if (OBJECT(CPICTURE)->pixmap != NULL)
  //{
  //  OBJECT(CPICTURE)->pixmap = NULL;
  //}

END_METHOD


BEGIN_PROPERTY(CPICTURE_width)

  GB.ReturnInteger(OBJECT(CPICTURE)->pixmap->width());

END_PROPERTY


BEGIN_PROPERTY(CPICTURE_height)

  GB.ReturnInteger(OBJECT(CPICTURE)->pixmap->height());

END_PROPERTY


BEGIN_METHOD(CPICTURE_load, GB_STRING path)

  char *path = GB.ToZeroString(PARAM(path));

  char *addr;
  long len;

  GB.LoadFile(path, &addr, &len);
  OBJECT(CPICTURE)->pixmap->loadFromData((const uchar *)addr, (uint)len);
  GB.ReleaseFile(&addr, len);

END_METHOD



GB_DESC CPictureDesc[] =
{
  GB_DECLARE("Picture", sizeof(CPICTURE)),

  GB_STATIC_METHOD("_exit", NULL, CPICTURE_exit, NULL),

  GB_METHOD("_new", NULL, CPICTURE_new, NULL),
  GB_METHOD("_free", NULL, CPICTURE_free, NULL),

  GB_STATIC_METHOD("_get", "Picture", CPICTURE_get, "s"),

  GB_PROPERTY_READ("Width", "i", CPICTURE_width),
  GB_PROPERTY_READ("Height", "i", CPICTURE_height),
  GB_METHOD("Load", NULL, CPICTURE_load, "(Path)s"),
  //GB_METHOD("Save", NULL, CPICTURE_load, "(Path)s(Format)s"),

  GB_END_DECLARE
};

