/***************************************************************************

  CRadioButton.cpp

  The RadioButton class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CRADIOBUTTON_CPP

#include "common.h"

#include <qapplication.h>
#include <qradiobutton.h>
#include <qobjcoll.h>

#include "gambas.h"

#include "CRadioButton.h"

static long EVENT_Click;


BEGIN_METHOD(CRADIOBUTTON_new, void *parent)

  QRadioButton *wid = new QRadioButton(CONTAINER(PARAM(parent)));

  QObject::connect(wid, SIGNAL(clicked()), &CRadioButton::manager, SLOT(clicked()));

  QRADIOBUTTON(_object) = wid;

  CWIDGET_new(wid, (void *)_object, "RadioButton");

  wid->show();

END_METHOD


BEGIN_PROPERTY(CRADIOBUTTON_text)

  if (READ_PROPERTY)
    GB.ReturnString(QRADIOBUTTON(_object)->text(), 0);
  else
    QRADIOBUTTON(_object)->setText(GB.ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY


BEGIN_PROPERTY(CRADIOBUTTON_value)

  if (READ_PROPERTY)
    GB.ReturnBoolean(QRADIOBUTTON(_object)->isChecked());
  else
    QRADIOBUTTON(_object)->setChecked(PROPERTY(char));

END_PROPERTY


/* Class CCheckBox */

CRadioButton CRadioButton::manager;

void CRadioButton::clicked(void)
{
  QRadioButton *wid = (QRadioButton *)sender();
  QObject *parent = wid->parent();

  QObjectList *list = parent->queryList("QRadioButton", NULL, false, false);
  QObjectListIt it(*list);
  QRadioButton *obj;

  while ((obj = (QRadioButton *)(it.current())) != NULL)
  {
    ++it;
    if (obj != wid)
      obj->setChecked(false);
  }

  delete list;

  if (!wid->isChecked())
    wid->setChecked(true);
  else
    RAISE_EVENT(EVENT_Click, NULL);
}

GB_DESC CRadioButtonDesc[] =
{
  GB_DECLARE("RadioButton", sizeof(CRADIOBUTTON)), GB_INHERITS("Control"),

  GB_METHOD("_new", NULL, CRADIOBUTTON_new, "Container;"),

  GB_PROPERTY("Text", "s", CRADIOBUTTON_text),
  GB_PROPERTY("Caption", "s", CRADIOBUTTON_text),

  GB_PROPERTY("Value", "b", CRADIOBUTTON_value),

  GB_CONSTANT("_Properties", "s", CRADIOBUTTON_PROPERTIES),
  GB_CONSTANT("_DefaultEvent", "s", "Click"),

  GB_EVENT("Click", NULL, NULL, &EVENT_Click),

  GB_END_DECLARE
};


