/***************************************************************************

  CTabStrip.cpp

  The TabStrip class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CTABSTRIP_CPP

#include "common.h"

#include <qapplication.h>
#include <qframe.h>
#include <qtabwidget.h>

#include "gambas.h"

#include "CPicture.h"
#include "CTabStrip.h"


static long EVENT_Click;


/***************************************************************************

  TabStrip

***************************************************************************/


PRIVATE void set_tab_count(void *_object, long new_count)
{
  long count = THIS->stack->count();
  QFrame *page;
  long i;
  long index;
  QString label;

  if (new_count < 1 || new_count > 32)
  {
    GB.Error(GB_ERR_ARG);
    return;
  }

  if (new_count == count)
    return;

  if (new_count > count)
  {
    THIS->stack->resize(new_count);

    for (i = count; i < new_count; i++)
    {
      page = new QFrame(WIDGET);
      CWidget::installFilter(page);

      label.sprintf("Tab %ld", i);
      WIDGET->insertTab(page, label);
      THIS->stack->insert(i, page);
    }

    index = new_count - 1;
  }
  else
  {
    index = WIDGET->currentPageIndex();
    //same = (id == PARAM(index));

    for (i = new_count; i < count; i++)
    {
      if (THIS->stack->at(i)->children() != NULL)
      {
        GB.Error("Tab is not empty");
        return;
      }
    }

    for (i = new_count; i < count; i++)
    {
      if (i == index)
        index = 0;

      page = THIS->stack->at(i);
      WIDGET->removePage(page);
      delete page;
    }

    THIS->stack->resize(new_count);
  }

  THIS->container = THIS->stack->at(index);
  WIDGET->showPage(THIS->container);
}


BEGIN_METHOD(CTABSTRIP_new, void *parent)

  QTabWidget *wid = new QTabWidget(CONTAINER(PARAM(parent)));

  CWIDGET_new(wid, (void *)_object, "TabStrip");

  OBJECT(CTABSTRIP)->container = NULL;
  OBJECT(CTABSTRIP)->index = -1;
  OBJECT(CTABSTRIP)->stack = new QVector<QFrame>;

  QObject::connect(wid, SIGNAL(currentChanged(QWidget *)), &CTabStrip::manager, SLOT(currentChanged(QWidget *)));

  wid->show();

  set_tab_count(THIS, 1);

END_METHOD


BEGIN_METHOD_VOID(CTABSTRIP_free)

  delete OBJECT(CTABSTRIP)->stack;

  //CALL_METHOD_VOID(CWIDGET_delete);

END_METHOD


BEGIN_PROPERTY(CTABSTRIP_tabs)

  if (READ_PROPERTY)
    GB.ReturnInteger(THIS->stack->count());
  else
    set_tab_count(THIS, PROPERTY(long));

END_PROPERTY


PRIVATE bool check_index(CTABSTRIP *_object, long index)
{
  if (index < 0 || index >= (long)THIS->stack->count())
  {
    GB.Error("Bad index");
    return true;
  }
  else
    return false;
}

BEGIN_PROPERTY(CTABSTRIP_current)

  if (READ_PROPERTY)
    GB.ReturnInteger(WIDGET->currentPageIndex());
  else
  {
    long index = PROPERTY(long);

    if (check_index(THIS, index))
      return;

    if (index == WIDGET->currentPageIndex())
      return;

    WIDGET->showPage(THIS->stack->at(index));
  }

END_PROPERTY


BEGIN_PROPERTY(CTABSTRIP_text)

  if (READ_PROPERTY)
    GB.ReturnZeroString(WIDGET->tabLabel(WIDGET->currentPage()));
  else
    WIDGET->changeTab(WIDGET->currentPage(), GB.ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY


BEGIN_METHOD(CTABSTRIP_get, long index)

  long index = PARAM(index);

  if (check_index(THIS, index))
    return;

  THIS->index = index;
  RETURN_SELF();

END_METHOD


/***************************************************************************

  .Tab

***************************************************************************/

PRIVATE QFrame *get_page(CTABSTRIP *_object)
{
  int index = THIS->index;

  if (index < 0)
    return (QFrame *)WIDGET->currentPage();

  THIS->index = (-1);
  return THIS->stack->at(index);
}


BEGIN_PROPERTY(CTAB_text)

  QFrame *page = get_page(THIS);

  if (READ_PROPERTY)
    GB.ReturnZeroString(WIDGET->tabLabel(page));
  else
    WIDGET->changeTab(page, GB.ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY


BEGIN_PROPERTY(CTAB_enabled)

  QFrame *page = get_page(THIS);

  if (READ_PROPERTY)
    GB.ReturnBoolean(WIDGET->isTabEnabled(page));
  else
    WIDGET->setTabEnabled(page, PROPERTY(char) != 0);

END_PROPERTY


BEGIN_METHOD_VOID(CTAB_next)

  QFrame *page = THIS->stack->at(THIS->index);

  QObjectList *list = (QObjectList *)page->children();
  unsigned int index;
  CWIDGET *widget;

  for(;;)
  {
    index = ENUM(int);

    if (list == NULL || index >= list->count())
    {
      GB.StopEnum();
      return;
    }

    ENUM(int) = index + 1;

    widget = CWidget::getReal(list->at(index));
    if (widget)
    {
      GB.ReturnObject(widget);
      return;
    }
  }

END_METHOD


BEGIN_PROPERTY(CTAB_count)

  QFrame *page = THIS->stack->at(THIS->index);

  if (page->children() == NULL)
    GB.ReturnInteger(0);
  else
    GB.ReturnInteger(page->children()->count());

END_PROPERTY


BEGIN_PROPERTY(CTAB_children)

  RETURN_SELF();

END_PROPERTY



/***************************************************************************

  Descriptions

***************************************************************************/

GB_DESC CTabChildrenDesc[] =
{
  GB_DECLARE(".Tab.Children", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("_next", "Control", CTAB_next, NULL),
  GB_PROPERTY_READ("Count", "i", CTAB_count),

  GB_END_DECLARE
};


GB_DESC CTabDesc[] =
{
  GB_DECLARE(".Tab", 0), GB_VIRTUAL_CLASS(),

  GB_PROPERTY("Text", "s", CTAB_text),
  GB_PROPERTY("Caption", "s", CTAB_text),
  GB_PROPERTY("Enabled", "b", CTAB_enabled),
  GB_PROPERTY_READ("Children", ".Tab.Children", CTAB_children),
  //GB_PROPERTY_WRITE("Picture", "s", CTABSTRIP_picture),

  GB_END_DECLARE
};


GB_DESC CTabStripDesc[] =
{
  GB_DECLARE("TabStrip", sizeof(CTABSTRIP)), GB_INHERITS("Container"),

  GB_METHOD("_new", NULL, CTABSTRIP_new, "Container;"),
  GB_METHOD("_free", NULL, CTABSTRIP_free, NULL),

  GB_PROPERTY("Count", "i", CTABSTRIP_tabs),
  GB_PROPERTY("Text", "s", CTABSTRIP_text),
  GB_PROPERTY("Caption", "s", CTABSTRIP_text),
  GB_PROPERTY("Current", "i", CTABSTRIP_current),
  GB_PROPERTY("Index", "i", CTABSTRIP_current),

  GB_METHOD("_get", ".Tab", CTABSTRIP_get, "(Index)i"),

  GB_CONSTANT("_Properties", "s", CTABSTRIP_PROPERTIES),
  GB_CONSTANT("_DefaultEvent", "s", "Click"),

  GB_EVENT("Click", NULL, NULL, &EVENT_Click),

  GB_END_DECLARE
};


/***************************************************************************

  Class CTabStrip

***************************************************************************/


CTabStrip CTabStrip::manager;

void CTabStrip::currentChanged(QWidget *wid)
{
  GET_SENDER(ob);
  ((CTABSTRIP *)ob)->container = wid;

  if (wid->isVisible())
    RAISE_EVENT(EVENT_Click, NULL);
}


/*!---------

\HELP
c-tabstrip
class

\IN
interface

\TITLE
TabStrip

\DESC:fr
Panneau  onglets.

\REMARK:fr
Attention ! Seuls les contrles contenus dans l'onglet courant sont visibles
par la proprit _c-container-children_

\INHERITS
_c-container_ _c-control_

\SEE
_c-container_ _c-container-children_

----------*/

/*!---------

\HELP
c-tabstrip-tabs
function

\IN
c-tabstrip-p

\GROUP
_c-tabstrip_

\TITLE
Tabs

\TYPE
Property Integer

\DESC:fr
Dfinit ou retourne le nombre d'onglets contenus dans un panneau  onglets.

\SYNTAX

<I>nombre</I> = <I>objet</I>.<B>Tabs</B><BR><BR>
<I>objet</I>.<B>Tabs</B> = <I>nombre</I>

\ARG

<TR><TD><I>objet</I></TD><TD>Un objet de type <I>TabStrip</I>.</TD><TR>
<TR><TD><I>nombre</I></TD><TD>Le nombre d'onglets.</TD><TR>

\SEE
_c-tabstrip_

----------*/

/*!---------

\HELP
c-tabstrip-caption
function

\IN
c-tabstrip-p

\GROUP
_c-tabstrip_

\TITLE
Caption

\TYPE
Property String

\DESC:fr
Dfinit ou retourne le libell de l'onglet courant.

\SYNTAX

<I>libell</I> = <I>objet</I>.<B>Caption</B><BR><BR>
<I>objet</I>.<B>Caption</B> = <I>libell</I>

\ARG

<TR><TD><I>objet</I></TD><TD>Un objet de type <I>TabStrip</I>.</TD><TR>
<TR><TD><I>libell</I></TD><TD>Un texte quelconque.</TD><TR>

\SAME
Text

\SEE
_c-tabstrip_

----------*/

/*!---------

\HELP
c-tabstrip-current
function

\IN
c-tabstrip-p

\GROUP
_c-tabstrip_

\TITLE
Current

\TYPE
Property Integer

\DESC:fr
Dfinit ou retourne l'index de l'onglet courant.

\SYNTAX

<I>index</I> = <I>objet</I>.<B>Current</B><BR><BR>
<I>objet</I>.<B>Current</B> = <I>index</I>

\ARG

<TR><TD><I>objet</I></TD><TD>Un objet de type <I>TabStrip</I>.</TD><TR>
<TR><TD><I>index</I></TD><TD>Le numro de l'onglet courant.</TD><TR>

\SEE
_c-tabstrip_

----------*/

/*!---------

\HELP
c-tabstrip-click
function

\IN
c-tabstrip-e

\GROUP
_c-tabstrip_

\TITLE
Click

\TYPE
Event

\DESC:fr
Cet vnement se dclenche lors d'un changement de l'onglet courant.

\SYNTAX

<B>Procedure TabStrip\_Click()</B>

\SEE
_c-tabstrip_

----------*/

