/***************************************************************************

  CWidget.cpp

  The Control class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CWIDGET_CPP

#include "gambas.h"

#include <stdio.h>

#include "CWidget.h"
#include "CFont.h"
#include "CMouse.h"
#include "CWindow.h"

#include <qapplication.h>
#include <qobjcoll.h>
#include <qpalette.h>
#include <qtooltip.h>
#include <qpushbutton.h>


/* Control class */

void CWIDGET_new(QWidget *w, void *object, char *klass = NULL, bool no_filter = false, bool no_tag = false)
{
  //char name[6];
  //static int n = 0;

  CWidget::add(w, object, no_filter);

  ((CWIDGET *)object)->widget = w;
  ((CWIDGET *)object)->font.klass = CLASS_Font;
  ((CWIDGET *)object)->font.font = CFONT_WIDGET;
  ((CWIDGET *)object)->background = QColorGroup::Background;
  ((CWIDGET *)object)->foreground = QColorGroup::Foreground;

  if (!no_tag)
    ((CWIDGET *)object)->tag.type = GB_T_NULL;

  //n++;
  //sprintf(name, "#%d", n);

  //w->setName(name);
  //CWidget::setName((CWIDGET *)object, (const char *)name);
}


int CWIDGET_check(void *object)
{
  return QWIDGET(object) == NULL;
}


BEGIN_PROPERTY(CWIDGET_x)

  if (READ_PROPERTY)
    GB.ReturnInteger(QWIDGET(_object)->pos().x());
  else
    QWIDGET(_object)->move(PROPERTY(long), QWIDGET(_object)->pos().y());

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_screen_x)

  GB.ReturnInteger(QWIDGET(_object)->mapToGlobal(QPoint(0, 0)).x());

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_y)

  if (READ_PROPERTY)
    GB.ReturnInteger(QWIDGET(_object)->pos().y());
  else
    QWIDGET(_object)->move(QWIDGET(_object)->pos().x(), PROPERTY(long));

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_screen_y)

  GB.ReturnInteger(QWIDGET(_object)->mapToGlobal(QPoint(0, 0)).y());

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_w)

  if (READ_PROPERTY)
    GB.ReturnInteger(QWIDGET(_object)->size().width());
  else
    QWIDGET(_object)->resize(PROPERTY(long), QWIDGET(_object)->size().height());

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_h)

  if (READ_PROPERTY)
    GB.ReturnInteger(QWIDGET(_object)->size().height());
  else
    QWIDGET(_object)->resize(QWIDGET(_object)->size().width(), PROPERTY(long));

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_name)

  if (READ_PROPERTY)
    GB.ReturnString(QWIDGET(_object)->name(), 0);
  else
    CWidget::setName(OBJECT(CWIDGET), GB.ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_font)

  if (READ_PROPERTY)
  {
    if (THIS->ownFont())
      GB.ReturnObject(&(OBJECT(CWIDGET)->font));
    else
      GB.ReturnNull();
  }
  else
  {
    if (PROPERTY(CFONT *) == 0)
      THIS->unsetFont();
    else
      THIS->setFont(CFONT_get_font(PROPERTY(CFONT *)));
  }

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_design)

  if (READ_PROPERTY)
  {
    GB.ReturnBoolean(CWIDGET_test_flag(_object, WF_DESIGN));
    return;
  }

  if (PROPERTY(char))
  {
    CWIDGET_set_flag(_object, WF_DESIGN);
    QWIDGET(_object)->setFocusPolicy(QWidget::NoFocus);
    //CWidget::installFilter(QWIDGET(_object));
  }
  //else
  //{
  //  CWIDGET_clear_flag(_object, WF_DESIGN);
  //  //CWidget::removeFilter(QWIDGET(_object));
  //}

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_visible)

  if (READ_PROPERTY)
    GB.ReturnBoolean(!QWIDGET(_object)->isHidden());
  else
  {
    if (PROPERTY(char))
      QWIDGET(_object)->show();
    else
      QWIDGET(_object)->hide();
  }

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_enabled)

  if (READ_PROPERTY)
    GB.ReturnBoolean(QWIDGET(_object)->isEnabled());
  else
    QWIDGET(_object)->setEnabled(PROPERTY(char));

END_PROPERTY


BEGIN_METHOD(CWIDGET_move, long x; long y; long w; long h)

  long w;
  long h;

  QWIDGET(_object)->move(PARAM(x), PARAM(y));

  if (GB.IsMissing(3))
    w = QWIDGET(_object)->width();
  else
    w = PARAM(w);

  if (GB.IsMissing(4))
    h = QWIDGET(_object)->height();
  else
    h = PARAM(h);

  QWIDGET(_object)->resize(w, h);

END_METHOD


BEGIN_METHOD(CWIDGET_resize, long w; long h)

  QWIDGET(_object)->resize(PARAM(w), PARAM(h));

END_METHOD


BEGIN_METHOD_VOID(CWIDGET_delete)

  if (QWIDGET(_object) != NULL)
    delete QWIDGET(_object);

END_METHOD


BEGIN_METHOD_VOID(CWIDGET_show)

  QWIDGET(_object)->show();

END_METHOD


BEGIN_METHOD_VOID(CWIDGET_hide)

  QWIDGET(_object)->hide();

END_METHOD


BEGIN_METHOD_VOID(CWIDGET_raise)

  QWIDGET(_object)->raise();

END_METHOD


BEGIN_METHOD_VOID(CWIDGET_lower)

  QWIDGET(_object)->lower();

END_METHOD


BEGIN_METHOD_VOID(CWIDGET_set_focus)

  QWIDGET(_object)->setFocus();

END_METHOD


BEGIN_PROPERTY(CWIDGET_tag)

  if (READ_PROPERTY)
    GB.ReturnPtr(GB_T_VARIANT, &OBJECT(CWIDGET)->tag);
  else
  {
    GB.Store(GB_T_VARIANT, (void *)&PROPERTY(GB_VARIANT), (void *)&(OBJECT(CWIDGET)->tag));
    //printf("Set Tag %p : %i\n", _object, OBJECT(CWIDGET)->tag.type);
  }

END_METHOD


BEGIN_PROPERTY(CWIDGET_mouse)

  QWidget *wid = QWIDGET(_object);

  if (READ_PROPERTY)
  {
    if (wid->ownCursor())
      GB.ReturnInteger(wid->cursor().shape());
    else
      GB.ReturnInteger(CMOUSE_DEFAULT);
  }
  else
  {
    if (PROPERTY(long) == CMOUSE_DEFAULT)
      wid->unsetCursor();
    else
      wid->setCursor(PROPERTY(long));
  }

END_METHOD


BEGIN_PROPERTY(CWIDGET_background)

  QWidget *wid = QWIDGET(_object);
  QPalette pal(wid->palette());
  QColorGroup::ColorRole role = (QColorGroup::ColorRole)OBJECT(CWIDGET)->background;

  //qDebug("bm = %d (%d %d)", wid->backgroundMode(), QWidget::PaletteButton, QWidget::PaletteBase);

  if (READ_PROPERTY)
    GB.ReturnInteger(pal.color(QPalette::Active, role).rgb() & 0xFFFFFF);
  else
  {
    pal.setColor(role, QColor((QRgb)PROPERTY(long)));
    wid->setPalette(pal);
  }

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_foreground)

  QWidget *wid = QWIDGET(_object);

  if (READ_PROPERTY)
  {
    GB.ReturnInteger((long)(wid->foregroundColor().rgb() & 0xFFFFFF));
    return;
  }
  else
  {
    QPalette pal(QWIDGET(_object)->palette());

    pal.setColor(QColorGroup::Foreground, QColor((QRgb)PROPERTY(long)));
    pal.setColor(QColorGroup::Text, QColor((QRgb)PROPERTY(long)));
    wid->setPalette(pal);
  }

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_parent)

  QWidget *parent = QWIDGET(_object)->parentWidget();

  if (QWIDGET(_object)->isTopLevel())
    GB.ReturnObject(NULL);
  else
    GB.ReturnObject(CWidget::get(parent));

END_PROPERTY


BEGIN_PROPERTY(CWIDGET_tooltip)

  if (READ_PROPERTY)
    GB.ReturnNull();
  else
    QToolTip::add(QWIDGET(_object), GB.ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY

/*
BEGIN_PROPERTY(CWIDGET_properties)

  GB.ReturnZeroString(CWidget::getProperties(GB.GetClass(_object)));

END_PROPERTY
*/

GB_DESC CWidgetDesc[] =
{
  GB_DECLARE("Control", sizeof(CWIDGET)),

  GB_HOOK_NEW(NULL),
  GB_HOOK_CHECK(CWIDGET_check),

  GB_METHOD("_free", NULL, CWIDGET_delete, NULL),

  GB_METHOD("Move", NULL, CWIDGET_move, "(X)i(Y)i[(Width)i(Height)i]"),
  GB_METHOD("Resize", NULL, CWIDGET_resize, "(Width)i(Height)i"),
  GB_METHOD("Delete", NULL, CWIDGET_delete, NULL),
  GB_METHOD("Show", NULL, CWIDGET_show, NULL),
  GB_METHOD("Hide", NULL, CWIDGET_hide, NULL),
  GB_METHOD("Raise", NULL, CWIDGET_raise, NULL),
  GB_METHOD("Lower", NULL, CWIDGET_lower, NULL),
  GB_METHOD("SetFocus", NULL, CWIDGET_set_focus, NULL),

  GB_PROPERTY("X", "i", CWIDGET_x),
  GB_PROPERTY("Y", "i", CWIDGET_y),
  GB_PROPERTY_READ("ScreenX", "i", CWIDGET_screen_x),
  GB_PROPERTY_READ("ScreenY", "i", CWIDGET_screen_y),
  GB_PROPERTY("W", "i", CWIDGET_w),
  GB_PROPERTY("H", "i", CWIDGET_h),
  GB_PROPERTY("Left", "i", CWIDGET_x),
  GB_PROPERTY("Top", "i", CWIDGET_y),
  GB_PROPERTY("Width", "i", CWIDGET_w),
  GB_PROPERTY("Height", "i", CWIDGET_h),

  //GB_PROPERTY("Name", "s", CWIDGET_name),

  GB_PROPERTY("Visible", "b", CWIDGET_visible),
  GB_PROPERTY("Enabled", "b", CWIDGET_enabled),

  GB_PROPERTY("Font", "Font", CWIDGET_font),
  GB_PROPERTY("Background", "i", CWIDGET_background),
  GB_PROPERTY("Foreground", "i", CWIDGET_foreground),
  GB_PROPERTY("BackColor", "i", CWIDGET_background),
  GB_PROPERTY("ForeColor", "i", CWIDGET_foreground),

  GB_PROPERTY("Design", "b", CWIDGET_design),
  GB_PROPERTY("Tag", "v", CWIDGET_tag),
  GB_PROPERTY("Mouse", "i" MOUSE_CONSTANTS, CWIDGET_mouse),
  GB_PROPERTY("ToolTip", "s", CWIDGET_tooltip),
  GB_PROPERTY_READ("Parent", "Control", CWIDGET_parent),

  GB_EVENT("Destroy", NULL, NULL, &EVENT_Destroy),
  GB_EVENT("Enter", NULL, NULL, &EVENT_Enter),
  GB_EVENT("GotFocus", NULL, NULL, &EVENT_GotFocus),
  GB_EVENT("LostFocus", NULL, NULL, &EVENT_LostFocus),
  GB_EVENT("KeyPress", "b", "(Ascii)s(Code)i(State)i", &EVENT_KeyPress),
  GB_EVENT("KeyRelease", NULL, "(Ascii)s(Code)i(State)i", &EVENT_KeyRelease),
  GB_EVENT("Leave", NULL, NULL, &EVENT_Leave),
  GB_EVENT("MouseDown", NULL, "(X)i(Y)i(Button)i", &EVENT_MouseDown),
  GB_EVENT("MouseMove", NULL, "(X)i(Y)i(Button)i", &EVENT_MouseMove),
  GB_EVENT("MouseUp", NULL, "(X)i(Y)i(Button)i", &EVENT_MouseUp),
  GB_EVENT("DblClick", NULL, NULL, &EVENT_DblClick),
  //GB_EVENT("Move", NULL, NULL, &EVENT_Move),
  //GB_EVENT("Resize", NULL, NULL, &EVENT_Resize),
  //GB_EVENT("Show", "b", NULL, &EVENT_Show),
  //GB_EVENT("Hide", "b", NULL, &EVENT_Hide),

  //GB_STATIC_PROPERTY_READ(".Properties", "s", CWIDGET_properties),
  GB_CONSTANT("_DefaultEvent", "s", "MouseDown"),
  GB_CONSTANT("_Properties", "s", CWIDGET_PROPERTIES),

  GB_END_DECLARE
};



/* Classe CWidget */

CWidget CWidget::manager;
QPtrDict<CWIDGET> CWidget::dict;
bool CWidget::real;
CWIDGET *CWidget::enter = NULL;
//QPtrDict<char> CWidget::propDict;

void CWidget::installFilter(QObject *o)
{
  QObjectList *children = (QObjectList *)(o->children());
  QObject *child;

  o->installEventFilter(&manager);

  if (!children)
    return;

  child = children->first();
  while (child)
  {
    child->installEventFilter(&manager);
    CWidget::installFilter(child);
    child = children->next();
  }
}

void CWidget::removeFilter(QObject *o)
{
  QObjectList *children = (QObjectList *)(o->children());
  QObject *child;

  o->removeEventFilter(&manager);

  if (!children)
    return;

  child = children->first();
  while (child)
  {
    //child->installEventFilter(&manager);
    CWidget::removeFilter(child);
    child = children->next();
  }
}


void CWidget::add(QObject *o, void *object, bool no_filter)
{
  //if (!no_filter)
  QObject::connect(o, SIGNAL(destroyed()), &manager, SLOT(destroy()));

  dict.insert(o, (CWIDGET *)object);

  if (!no_filter)
    CWidget::installFilter(o);

  GB.Ref(object);
}


CWIDGET *CWidget::get(QObject *o)
{
  CWIDGET *ob;

  real = true;

  do
  {
    ob = dict[o];
    if (ob != NULL)
      return ob;

    o = o->parent();
    real = false;
  }
  while (o != NULL);

  return NULL;
}

CWIDGET *CWidget::getReal(QObject *o)
{
  return dict[o];
}


QWidget *CWidget::getContainerWidget(CCONTAINER *object)
{
  if (object == NULL || object->container == NULL)
    GB.Error("Null container");

  //qDebug("Container = %p", object->container);

  return (object->container);
}


void CWidget::setName(CWIDGET *object, const char *name)
{
  QWidget *w = QWIDGET(object);
  CTOPLEVEL *top = (CTOPLEVEL *)CWidget::get(w->topLevelWidget());

  if (QWIDGET(top) == w)
    return;

  if (w->name() != NULL)
  {
    /*qDebug("- %s", w->name());*/
    top->dict->remove(w->name());
  }

  if (name != NULL)
  {
    top->dict->insert((const char *)name, object);
    w->setName(name);
    /*qDebug("+ %s", w->name());*/
  }
}

void CWidget::destroy()
{
  QWidget *w = (QWidget *)sender();
  CWIDGET *ob = CWidget::get(w);
  //MyMainWindow *top = (MyMainWindow *)w->topLevelWidget();

  //qDebug("> DESTROY %p widget = %p top = %p", ob, ob->widget, top);

  if (ob == NULL)
    return;

  //w->blockSignals(TRUE);
  //w->disconnect();

  // Oblige les widgets n'hritant pas de Control  possder cet vnement en premier !
  GB.Raise(ob, EVENT_Destroy, NULL);

  if (enter == ob)
    enter = NULL;

  //printf("#1\n");

  /*
  if (w->inherits("QPushButton"))
  {
    qDebug("Inherits QPushButton");
    top->setDefaultButton((QPushButton *)w, false);
    top->setCancelButton((QPushButton *)w, false);
  }
  */

  GB.Detach(ob);

  //printf("#2\n");

  //CWidget::setName(ob, NULL);
  dict.remove(w);
  QWIDGET(ob) = NULL;

  //printf("#3\n");

  GB.Store(GB_T_VARIANT, NULL, &ob->tag);

  //printf("#4\n");

  //qDebug("CWidget::destroy %p\n", ob);
  GB.Unref((void **)&ob);

  //printf("< DESTROY %p\n", ob);

  //printf("destroy OK\n");
}


bool CWidget::eventFilter(QObject *o, QEvent *e)
{
  bool ret = false;
  int ev;
  void *param;
  CWIDGET *ob;
  int type = e->type();
  int drop;
  bool form = false;
  QWidget *w;
  QPoint p;

  struct {
    long x;
    long y;
    long state;
    }
  mouse_param;

  struct {
    GB_STRING ascii;
    long key;
    long state;
    }
  key_param;

  //printf("EVENT widget = %p\n", o);

  ob = CWidget::get(o);

  if (ob && CWIDGET_test_flag(ob, WF_DESIGN))
  {
    if ((type == QEvent::MouseButtonPress)
        || (type == QEvent::MouseButtonRelease)
        || (type == QEvent::MouseButtonDblClick)
        || (type == QEvent::MouseMove)
        || (type == QEvent::KeyPress)
        || (type == QEvent::KeyRelease)
        || (type == QEvent::Enter)
        || (type == QEvent::Leave)
        || (type == QEvent::FocusIn)
        || (type == QEvent::FocusOut))
    {
      ret = true;
    }
  }

  if (!ret)
  {
    ret = QObject::eventFilter(o, e);    // standard event processing

    if (ret)
      return ret;
  }

  if (ob == NULL)
    return ret;


  //qDebug("! eventFilter %s (real %s)", o->name(), ob->widget->name());

  param = NULL;
  ev = (-1);

  switch (type)
  {
    case QEvent::MouseButtonPress: ev = EVENT_MouseDown; break;
    case QEvent::MouseButtonRelease: ev = EVENT_MouseUp; break;
    case QEvent::MouseMove: ev = EVENT_MouseMove; break;
    case QEvent::MouseButtonDblClick: ev = EVENT_DblClick; break;
  }

  if (ev >= 0)
  {
    if (!GB.CanRaise(ob, ev))
      return ret;
  }

  ev = (-1);

  switch (type)
  {
    case QEvent::Enter:

      if (ob != enter)
      {
        if (enter != NULL)
          GB.Raise(enter, EVENT_Leave, NULL);

        enter = ob;
        ev = EVENT_Enter;
      }

      break;

    case QEvent::Leave:

      break;

    case QEvent::MouseButtonPress:
    case QEvent::MouseButtonRelease:
    case QEvent::MouseMove:

      if (type == QEvent::MouseButtonPress)
      {
        mouse_param.state = ((QMouseEvent *)e)->stateAfter();
        ev = EVENT_MouseDown;
      }
      else
      {
        mouse_param.state = ((QMouseEvent *)e)->state();
        ev = (type == QEvent::MouseButtonRelease) ? EVENT_MouseUp : EVENT_MouseMove;
      }

      p.setX(((QMouseEvent *)e)->x());
      p.setY(((QMouseEvent *)e)->y());

      w = QWIDGET(ob);
      if (w != o)
        p = ((QWidget *)o)->mapTo(w, p);

      mouse_param.x = p.x();
      mouse_param.y = p.y();

      //if (type == QEvent::MouseButtonPress)
      //  qDebug("Mouse: x = %d y = %d", mouse_param.x, mouse_param.y);

      param = &mouse_param;

      break;

    case QEvent::KeyPress:
    case QEvent::KeyRelease:

      ev = (type == QEvent::KeyPress) ? EVENT_KeyPress : EVENT_KeyRelease;

      key_param.ascii.addr = (char *)((QKeyEvent *)e)->text().latin1();
      key_param.ascii.len = strlen(key_param.ascii.addr);
      key_param.key = ((QKeyEvent *)e)->key();
      key_param.state = ((QKeyEvent *)e)->state();

      param = &key_param;

      form = true;

      break;

    case QEvent::MouseButtonDblClick:

      ev = EVENT_DblClick;

      param = NULL;

      break;


    //case QEvent::Show:

      //if (((QShowEvent *)e)->spontaneous())
      //  ev = EVENT_Show;
      //break;

    //case QEvent::Hide:

      //if (((QHideEvent *)e)->spontaneous())
      //  ev = EVENT_Hide;
      //break;

    case QEvent::FocusIn:
      if (ob->widget == o)
        ev = EVENT_GotFocus;
      break;

    case QEvent::FocusOut:
      if (ob->widget == o)
        ev = EVENT_LostFocus;
      break;

    default:
      break;
  }

  if (ev >= 0)
  {
    CWIDGET *top = CWidget::get(((QWidget *)o)->topLevelWidget());

    /*qDebug("e->type() = %d", e->type());*/
    drop = GB.Raise(ob, ev, param);
    //if (drop)
    //  qDebug("DROP !");

    if ((ev == EVENT_KeyPress) && drop)
      return true;

    if (form && top)
    {
      if (top != ob)
      {
        drop = GB.Raise(top, ev, param);
        //if (drop) qDebug("DROP !");
        if (drop)
          return true;
      }

      if (ev == EVENT_KeyPress && CWINDOW_Main && top != (CWIDGET *)CWINDOW_Main)
      {
        QKeyEvent *k = (QKeyEvent *)e;
        QObject *o = QMAINWINDOW(CWINDOW_Main);

        if (o)
        {
          //qDebug("%p post Accel to %p", o, CWINDOW_Main);
          qApp->postEvent(o,
            new QKeyEvent(QEvent::Accel, k->key(), k->ascii(), k->state(), k->text(), k->isAutoRepeat(), k->count()));
        }
      }
    }
  }

  return ret;
}

/*
void CWidget::setProperties(const void *klass, const char *prop)
{
  propDict.insert(klass, prop);
}

const char *CWidget::getProperties(const void *klass)
{
  return propDict[klass];
}
*/

