/***************************************************************************

  CWidget.h

  The Control class

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __CWIDGET_H
#define __CWIDGET_H

#include "common.h"
#include "main.h"

#include <qobject.h>
#include <qwidget.h>
#include <qptrdict.h>
#include <qasciidict.h>
#include <qpixmap.h>

#include "gb.qt.h"

#include "CFont.h"

#ifndef __CWIDGET_CPP

extern GB_DESC CWidgetDesc[];

#else

#define THIS (((CWIDGET *)_object)->widget)

#endif

typedef
  struct {
    GB_OBJECT ob;
    QWidget *widget;
    unsigned short flag;
    char background;
    char foreground;
    GB_VARIANT tag;
    CFONT font;
    }
  CWIDGET;

typedef
  struct {
    CWIDGET widget;
    QWidget *container;
    }
  CCONTAINER;

typedef
  struct {
    CCONTAINER widget;
    QAsciiDict<void> *dict;
    }
  CTOPLEVEL;

enum {
  WF_DESIGN     = 0x01,
  WF_FIRST      = 0x02,
  WF_MENUBAR    = 0x04,
  WF_PERSISTENT = 0x08,
  WF_IN_CLOSE   = 0x10,
  WF_FIXED      = 0x20,
  WF_IN_RESIZE  = 0x40,
  WF_NO_EVENT   = 0x80,

  WF_LAST       = 0x80
  };

#define QWIDGET(object) (((CWIDGET *)object)->widget)
#define CONTAINER(_ob) CWidget::getContainerWidget((CCONTAINER *)_ob)

/*
GB_FUNC_METHOD CWIDGET_move, CWIDGET_resize, CWIDGET_delete, CWIDGET_show, CWIDGET_hide,
               CWIDGET_raise, CWIDGET_lower;

GB_FUNC_PROPERTY CWIDGET_x, CWIDGET_y, CWIDGET_w, CWIDGET_h,
                 CWIDGET_name, CWIDGET_design, CWIDGET_visible, CWIDGET_enabled,
                 CWIDGET_font, CWIDGET_tag, CWIDGET_mouse,
                 CWIDGET_background, CWIDGET_foreground, CWIDGET_parent;
*/

DECLARE_METHOD(CWIDGET_delete);
DECLARE_PROPERTY(CWIDGET_tag);

#define CWIDGET_set_flag(wid, f) (((CWIDGET *)wid)->flag |= f)
#define CWIDGET_clear_flag(wid, f) (((CWIDGET *)wid)->flag &= ~f)
#define CWIDGET_test_flag(wid, f) ((((CWIDGET *)wid)->flag & f) != 0)

#define GET_SENDER(_ob)   CWIDGET *_ob = CWidget::get((QObject *)sender())

#define CWIDGET_PROPERTIES \
  "X,Y,Width,Height,Visible,Enabled,Font,Background,Foreground,Tag,Mouse"

#define RAISE_EVENT(_event, _param) \
{ \
  GET_SENDER(ob); \
\
  if (ob == NULL) \
    return; \
\
  GB.Raise(ob, _event, _param); \
}

#define ENUM(type)  (*((type *)GB.GetEnum()))


void CWIDGET_new(QWidget *w, void *object, char *klass = NULL,
                 bool no_filter = false, bool no_tag = false);

int CWIDGET_check(void *object);

#ifndef __CWIDGET_CPP
extern
#endif
long
//EVENT_Move,
//EVENT_Resize,
EVENT_MouseDown,
EVENT_MouseUp,
EVENT_MouseMove,
EVENT_DblClick,
EVENT_KeyPress,
EVENT_KeyRelease,
EVENT_Enter,
EVENT_Leave,
EVENT_GotFocus,
EVENT_LostFocus,
EVENT_Destroy;
//EVENT_Show,
//EVENT_Hide;


class CWidget : public QObject
{
  Q_OBJECT

public:

  static void add(QObject *, void *, bool no_filter);
  static CWIDGET *get(QObject *);
  static CWIDGET *getReal(QObject *);

  static QWidget *getContainerWidget(CCONTAINER *object);

  static void setName(CWIDGET *, const char *);
  static void installFilter(QObject *);
  static void removeFilter(QObject *);

  static const char *getProperties(const void *klass);
  static void setProperties(const void *klass, const char *prop);

public slots:

  void destroy(void);

protected:

  bool eventFilter(QObject *, QEvent *);

private:

  static bool real;

  static CWidget manager;
  static CWIDGET *enter;
  static QPtrDict<CWIDGET> dict;
  //static QPtrDict<char> propDict;
};

#endif
