/***************************************************************************

  CWindow.cpp

  The Window and Form classes

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define _CWINDOW_CPP

#include "common.h"

#include <qnamespace.h>
#include <qapplication.h>
#include <qmenubar.h>
#include <qframe.h>
#include <qabstractlayout.h>
#include <qsizepolicy.h>
#include <qtoolbar.h>
#include <qworkspace.h>

#include "gambas.h"

#include "CWidget.h"
#include "CMenu.h"
#include "CWindow.h"

#include <qkeycode.h>

/*#define DEBUG*/

static long EVENT_Close;
static long EVENT_Activate;
static long EVENT_Deactivate;
static long EVENT_Move;
static long EVENT_Resize;

CWINDOW *CWINDOW_Main = NULL;


BEGIN_METHOD_VOID(CWINDOW_new)

  //MyMainWindow *parent;

  //if (GB.IsMissing(1))
  //  parent = NULL;
  //else
  //  parent = QMAINWINDOW(PARAM(parent));

  //MyMainWindow *win = new MyMainWindow(window_main ? QMAINWINDOW(window_main) : NULL);
  MyMainWindow *win = new MyMainWindow(0);
  QWidget *container = new QFrame(win);
  //QWidget *container = new QWorkspace(win);

  THIS = win;
  OBJECT(CWINDOW)->dict = new QAsciiDict<void>;
  OBJECT(CWINDOW)->menu = NULL;
  OBJECT(CWINDOW)->ret = 0;
  OBJECT(CWINDOW)->icon = NULL;
  OBJECT(CWINDOW)->border = MyMainWindow::BorderResizable;

  CWIDGET_new(win, (void *)_object, NULL);

  OBJECT(CWINDOW)->container = container;
  win->setCentralWidget(container);

  win->setOpaqueMoving(true);
  win->installEventFilter(&CWindow::manager);

  CWindow::dict.insert(_object, OBJECT(const CWINDOW));

  if (CWINDOW_Main == 0)
  {
    CWINDOW_Main = OBJECT(CWINDOW);
    //qDebug("CWINDOW_Main = %p", CWINDOW_Main);
  }

END_METHOD


BEGIN_METHOD_VOID(CFORM_new)

  GB.Attach(_object, _object, "Form");

END_METHOD


BEGIN_METHOD_VOID(CWINDOW_free)

  CWINDOW *window = OBJECT(CWINDOW);

  delete window->dict;
  if (window->menu)
    delete window->menu;

  PICTURE_set(&(window->icon), 0);

  CALL_METHOD_VOID(CWIDGET_delete);

END_METHOD


BEGIN_METHOD_VOID(CWINDOW_next)

  CWINDOW *next;
  QPtrDictIterator<CWINDOW> *iter = ENUM(QPtrDictIterator<CWINDOW> *);

  if (iter == NULL)
  {
    iter = new QPtrDictIterator<CWINDOW>(CWindow::dict);
    ENUM(QPtrDictIterator<CWINDOW> *) = iter;
  }

  next = iter->current();

  if (next == NULL)
  {
    delete iter;
    //ENUM(QPtrDictIterator<CWINDOW>) = NULL;
    GB.StopEnum();
    return;
  }

  ++(*iter);
  GB.ReturnObject(next);

END_METHOD


BEGIN_PROPERTY(CWINDOW_count)

  GB.ReturnInteger(CWindow::dict.count());

END_PROPERTY


BEGIN_METHOD(CWINDOW_get, long id)

  QWidget *wid = QWidget::find(PARAM(id));

  //qDebug("id = %d wid = %p", PARAM(id), wid);

  if (wid != 0 && wid->isTopLevel())
  {
    //qDebug("-> %p", CWidget::getReal(wid));
    GB.ReturnObject(CWidget::getReal(wid));
  }
  else
  {
    //qDebug("-> %p", 0);
    GB.ReturnObject(0);
  }

END_METHOD


BEGIN_METHOD(CWINDOW_close, long ret)

  bool closed;
  //bool flag;

  if (CWIDGET_test_flag(_object, WF_IN_CLOSE))
  {
    GB.Return(GB_T_BOOLEAN, FALSE);
    return;
  }

  //qDebug("CWINDOW_close");

  //flag = CWIDGET_test_flag(_object, WF_PERSISTENT);

  //if (flag)
  //  CWIDGET_clear_flag(_object, WF_PERSISTENT);

  closed = THIS->close();

  if (closed && !GB.IsMissing(1))
    CONTROL->ret = PARAM(ret);

  //if (flag)
  //  CWIDGET_set_flag(_object, WF_PERSISTENT);

  GB.Return(GB_T_BOOLEAN, (char)!closed);

END_METHOD


BEGIN_METHOD_VOID(CWINDOW_show)

  THIS->show();

END_METHOD


BEGIN_METHOD_VOID(CWINDOW_show_modal)

  CONTROL->ret = 0;
  THIS->showModal();
  GB.ReturnInteger(CONTROL->ret);

END_METHOD


BEGIN_PROPERTY(CWINDOW_persistent)

  /*
  if (READ_PROPERTY)
    GB.ReturnBoolean(THIS->isPersistent());
  else
    THIS->setPersistent(PROPERTY(char) != 0);
  */

  if (READ_PROPERTY)
    GB.ReturnBoolean(CWIDGET_test_flag(CONTROL, WF_PERSISTENT));
  else
  {
    if (PROPERTY(char))
      CWIDGET_set_flag(CONTROL, WF_PERSISTENT);
    else
      CWIDGET_clear_flag(CONTROL, WF_PERSISTENT);
  }

END_PROPERTY


BEGIN_PROPERTY(CWINDOW_text)

  if (READ_PROPERTY)
    GB.ReturnZeroString(THIS->caption());
  else
    THIS->setCaption(GB.ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY


BEGIN_PROPERTY(CWINDOW_menu)

  RETURN_SELF();

END_PROPERTY


BEGIN_PROPERTY(CWINDOW_menu_count)

  CWINDOW *window = OBJECT(CWINDOW);

  if (window->menu)
    GB.ReturnInteger(window->menu->count());
  else
    GB.ReturnInteger(0);

END_PROPERTY


BEGIN_METHOD_VOID(CWINDOW_menu_next)

  CWINDOW *window = OBJECT(CWINDOW);
  unsigned int index;

  if (window->menu == NULL)
  {
    GB.StopEnum();
    return;
  }

  index = ENUM(int);

  if (index >= window->menu->count())
  {
    GB.StopEnum();
    return;
  }

  GB.ReturnObject(window->menu->at(index));

  ENUM(int) = index + 1;

END_PROPERTY




/*

BEGIN_METHOD(CWINDOW_toolbar_add, CTOOLBAR *toolbar)

  QMainWindow *wid = THIS;
  QToolBar *tb = QTOOLBAR(PARAM(toolbar));

  qDebug("ToolBar H = %d", tb->height());
  wid->addToolBar(tb);
  qDebug("ToolBar H = %d", tb->height());

END_METHOD


BEGIN_METHOD(CWINDOW_toolbar_remove, CTOOLBAR *toolbar)

  QMainWindow *wid = THIS;

  wid->removeToolBar(QTOOLBAR(PARAM(toolbar)));

END_METHOD
*/


BEGIN_PROPERTY(CWINDOW_id)

  GB.ReturnInteger((long)THIS->winId());

END_PROPERTY


BEGIN_PROPERTY(CWINDOW_border)

  int border;

  if (READ_PROPERTY)
    GB.ReturnInteger(CONTROL->border);
  else
  {
    border = PROPERTY(long);
    if (border >= MyMainWindow::BorderNone && border <= MyMainWindow::BorderResizable)
    {
      THIS->setBorder(border);
      CONTROL->border = border;
    }
  }

END_PROPERTY


BEGIN_PROPERTY(CWINDOW_icon)

  if (READ_PROPERTY)
    GB.ReturnObject(CONTROL->icon);
  else
    THIS->setIcon(PICTURE_set(&(CONTROL->icon), PROPERTY(CPICTURE *)));

END_PROPERTY


BEGIN_PROPERTY(CWINDOW_state)

  if (READ_PROPERTY)
    GB.ReturnObject(CONTROL->icon);
  else
    THIS->setIcon(PICTURE_set(&(CONTROL->icon), PROPERTY(CPICTURE *)));

END_PROPERTY


BEGIN_METHOD_VOID(CWINDOW_center)

  THIS->move(
    (qApp->desktop()->width() - THIS->width()) / 2,
    (qApp->desktop()->height() - THIS->height()) / 2
    );

END_METHOD


/*
BEGIN_PROPERTY(CWINDOW_design)

  if (READ_PROPERTY)
  {
    GB.ReturnBoolean(CWIDGET_test_flag(_object, WF_WIN_DESIGN));
    return;
  }

  if (PROPERTY(char))
  {
    CWIDGET_set_flag(_object, WF_WIN_DESIGN);

  }
  else
  {
    CWIDGET_clear_flag(_object, WF_WIN_DESIGN);
  }

END_PROPERTY
*/

/*
BEGIN_PROPERTY(CWINDOW_workspace)

  CWINDOW *win = OBJECT(CWINDOW);

  if (READ_PROPERTY)
    GB.ReturnBoolean(win->container->isA("QWorkspace"));
  else
  {
    if (win->container->children() != 0)
    {
      GB.Error("Property is read-only");
      return;
    }

    delete win->container;

    if (PROPERTY(char))
      win->container = new QWorkspace(THIS);
    else
      win->container = new QFrame(THIS);
  }

END_PROPERTY
*/

GB_DESC CWindowMenusDesc[] =
{
  GB_DECLARE(".Window.Menus", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("_next", "Menu", CWINDOW_menu_next, NULL),
  GB_PROPERTY_READ("Count", "i", CWINDOW_menu_count),

  GB_END_DECLARE
};

/*
GB_DESC CWindowToolBarsDesc[] =
{
  GB_DECLARE(".Window.ToolBars", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("Add", NULL, CWINDOW_toolbar_add, "'Toolbar'Toolbar;"),
  GB_METHOD("Remove", NULL, CWINDOW_toolbar_remove, "'Toolbar'Toolbar;"),

  GB_END_DECLARE
};
*/


GB_DESC CWindowDesc[] =
{
  GB_DECLARE("Window", sizeof(CWINDOW)), GB_INHERITS("Container"),

  GB_CONSTANT("Fixed", "i", 1),
  GB_CONSTANT("Resizable", "i", 2),

  GB_CONSTANT("Normal", "i", 0),
  GB_CONSTANT("Minimized", "i", 1),
  GB_CONSTANT("Maximized", "i", 2),

  GB_METHOD("_new", NULL, CWINDOW_new, NULL),
  GB_METHOD("_free", NULL, CWINDOW_free, NULL),

  GB_METHOD("Close", "b", CWINDOW_close, "[(Return)i]"),
  GB_METHOD("Show", NULL, CWINDOW_show, NULL),
  GB_METHOD("ShowModal", "i", CWINDOW_show_modal, NULL),
  GB_METHOD("Center", NULL, CWINDOW_center, NULL),

  GB_PROPERTY("Persistent", "b", CWINDOW_persistent),
  GB_PROPERTY("Text", "s", CWINDOW_text),
  GB_PROPERTY("Title", "s", CWINDOW_text),
  GB_PROPERTY("Caption", "s", CWINDOW_text),
  GB_PROPERTY("Icon", "Picture", CWINDOW_icon),
  GB_PROPERTY("Border", "i<Window,Fixed,Resizable>", CWINDOW_border), // None ne marche pas !
  GB_PROPERTY("State", "i<Window,Normal,Minimized,Maximized>", CWINDOW_state),

  GB_PROPERTY_READ("Menus", ".Window.Menus", CWINDOW_menu),

  GB_PROPERTY_READ("Id", "i", CWINDOW_id),

  GB_CONSTANT("_Properties", "s", CWINDOW_PROPERTIES),

  GB_EVENT("Close", "b", NULL, &EVENT_Close),
  GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
  GB_EVENT("Deactivate", NULL, NULL, &EVENT_Deactivate),
  GB_EVENT("Move", NULL, NULL, &EVENT_Move),
  GB_EVENT("Resize", NULL, NULL, &EVENT_Resize),

  GB_END_DECLARE
};


GB_DESC CWindowsDesc[] =
{
  GB_DECLARE("Windows", 0), GB_VIRTUAL_CLASS(),

  GB_STATIC_METHOD("_next", "Window", CWINDOW_next, NULL),
  GB_STATIC_METHOD("_get", "Window", CWINDOW_get, "'Id'i"),
  GB_STATIC_PROPERTY_READ("Count", "i", CWINDOW_count),

  GB_END_DECLARE
};


GB_DESC CFormDesc[] =
{
  GB_DECLARE("Form", sizeof(CFORM)), GB_INHERITS("Window"),

  GB_METHOD("_new", NULL, CFORM_new, NULL),

  GB_END_DECLARE
};

/* Class MyMainWindow */

MyMainWindow::MyMainWindow(MyMainWindow *copy = 0) :
  QMainWindow::QMainWindow(0, 0, Qt::WType_TopLevel | (copy ? copy->getWFlags() : 0)) // | Qt::WDestructiveClose)
{
  db = 0;
  cb = 0;
  shown = false;
  //qDebug("MainWindow %p", this);
}


MyMainWindow::~MyMainWindow()
{
  CWINDOW *ob = (CWINDOW *)CWidget::get(this);

  //qDebug("~MyMainWindow %p", this);

  if (ob == NULL)
  {
    qWarning("~MyMainWindow: ob == NULL");
    return;
  }

  GB.Detach(ob);

  if (ob->menu)
    CMenu::unrefChildren(ob->menu);

  CWindow::dict.remove(ob);

  //if (ob == window_main)
  //  window_main = NULL;

  if (CWindow::dict.count() == 0)
    qApp->quit();

  //qDebug("~MyMainWindow %p (end)", this);
}


void MyMainWindow::setCancelButton(QPushButton *pb, bool on)
{
  if (on)
    cb = pb;
  else if (pb == cb)
    cb = 0;
}

void MyMainWindow::setDefaultButton(QPushButton *pb, bool on)
{
  //qDebug("setDefaultButton: %s %s", pb->text().latin1(), on ? "true" : "false");

  if (on)
  {
    if (db)
      db->setDefault(false);

    db = pb;
    db->setDefault(true);
  }
  else
  {
    if (pb == db)
    {
      db->setDefault(false);
      db = 0;
    }
  }
}


void MyMainWindow::show(void)
{
  CWIDGET *ob = CWidget::get(this);

  //qDebug(">> Show %d %d %d %d", x(), y(), width(), height());
  QMainWindow::show();
  raise();
  setActiveWindow();

  shown = true;

  GB.Raise(ob, EVENT_Resize, NULL);
  GB.Raise(ob, EVENT_Move, NULL);

  //qDebug("<< Show %d %d %d %d", x(), y(), width(), height());
  //CWIDGET_set_flag(CWidget::get(this), WF_IGNORE_MOVE);
}

void MyMainWindow::showModal(void)
{
  WFlags flags = getWFlags();
  CWIDGET *ob = CWidget::get(this);
  bool persistent = CWIDGET_test_flag(ob, WF_PERSISTENT);

  QPoint p((qApp->desktop()->width() - width()) / 2, (qApp->desktop()->height() - height()) / 2);

  setWFlags(Qt::WType_Modal | Qt::WStyle_Customize | Qt::WStyle_Dialog | Qt::WStyle_DialogBorder);
  clearWFlags(Qt::WStyle_Minimize);
  clearWFlags(Qt::WStyle_SysMenu);

  reparent(parentWidget(), getWFlags(), p);

  show();

  setWState(Qt::WState_Modal);
  qApp->enter_loop();

  if (persistent)
  {
    setWFlags(flags);
    clearWState(Qt::WState_Modal);
  }
}


void MyMainWindow::hide(void)
{
  //qDebug("hide");

  QMainWindow::hide();

  if (testWFlags(Qt::WType_Modal))
  {
    //qDebug("exit_loop");
    qApp->exit_loop();
  }
}

/*
QSize MyMainWindow::sizeHint() const
{
  //if (fixed)
  return QSize(width(), height());

  //return QFrame::sizeHint();
}
*/

void MyMainWindow::setBorder(int border)
{
  QWidget *cont = centralWidget();

  if (border == BorderNone)
  {
    clearWFlags(Qt::WStyle_NormalBorder);
    clearWFlags(Qt::WStyle_DialogBorder);
    setWFlags(Qt::WStyle_NoBorderEx);
    reparent(parentWidget(), getWFlags(), pos());
    return;
  }
  else
  {
    clearWFlags(Qt::WStyle_NoBorderEx);
    if (testWState(Qt::WState_Modal))
      setWFlags(Qt::WStyle_DialogBorder);
    else
      setWFlags(Qt::WStyle_NormalBorder);
  }

  if (border == BorderFixed)
  {
    layout()->activate();
    cont->setFixedSize(cont->width(), cont->height());
    layout()->setResizeMode(QLayout::Fixed);
  }
  else
  {
    //cont->setMinimumSize(0, 0);
    //cont->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
    cont->setMinimumSize(0, 0);
    cont->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
    layout()->setResizeMode(QLayout::Minimum);
    setMinimumSize(0, 0);
    setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
    //layout()->activate();
    //qDebug("%d %d", cont->minimumWidth(), cont->maximumWidth());
  }
}

/*
bool MyMainWindow::isFixed(void)
{
  QWidget *cont = centralWidget();

  return (!(cont->minimumWidth() == 0 && cont->minimumHeight() == 0));
}
*/

void MyMainWindow::paintUnclip(bool on)
{
  if (on)
    setWFlags(Qt::WPaintUnclipped);
  else
    clearWFlags(Qt::WPaintUnclipped);
}


void MyMainWindow::resizeEvent(QResizeEvent *e)
{
  QMainWindow::resizeEvent(e);

  //qDebug("Resize %d %d / %d %d", e->size().width(), e->size().height(), e->oldSize().width(), e->oldSize().height());

  //if (e->size() == e->oldSize())
  //{
  //  qDebug("...Ignore");
  //  return;
  //}

  if (shown)
    GB.Raise(CWidget::get(this), EVENT_Resize, NULL);
}


void MyMainWindow::moveEvent(QMoveEvent *e)
{
  CWIDGET *ob = CWidget::getReal(this);

  QMainWindow::moveEvent(e);

  //qDebug("Move (pos %d %d) (oldPos %d %d)", e->pos().x(), e->pos().y(), e->oldPos().x(), e->oldPos().y());
  //qDebug("     (geom %d %d) (fgeom %d %d)", geometry().x(), geometry().y(), frameGeometry().x(), frameGeometry().y());
  //qDebug("     Visible = %s Hidden = %s", (isVisible() ? "Yes" : "No"), (isHidden() ? "Yes" : "No"));
  //qDebug("     Flags = 0x%s State = 0x%s", QString::number(getWFlags(), 16).latin1(), QString::number(getWState(), 16).latin1());

  //if (CWIDGET_test_flag(ob, WF_IGNORE_MOVE))
  if (!testWFlags(Qt::WStyle_NoBorderEx))
  {
    if (geometry().x() == frameGeometry().x() && geometry().y() == frameGeometry().y())
    {
      //qDebug("...Ignore");
      return;
    }
  }

  if (shown)
    GB.Raise(ob, EVENT_Move, NULL);
}


void MyMainWindow::keyPressEvent(QKeyEvent *e)
{
  QPushButton *test = 0;

  if (e->state() == 0 || ( e->state() & Keypad && e->key() == Key_Enter ))
  {
    switch (e->key())
    {
      case Key_Enter:
      case Key_Return:

        test = defaultButton();
        break;

      case Key_Escape:

        test = cancelButton();
        break;
    }

    if (test)
    {
      QObjectList *list = queryList( "QPushButton" );
      QObjectListIt it( *list );
      QPushButton *pb;

      while ((pb = (QPushButton*)it.current()))
      {
        if (pb == test)
        {
          delete list;
          if (pb->isVisible() && pb->isEnabled())
            pb->animateClick();
          return;
        }
        ++it;
      }

      delete list;
  	}
  }

  e->ignore();
}


void MyMainWindow::closeEvent(QCloseEvent *e)
{
  CWIDGET *ob = CWidget::get(this);
  bool cancel;

  //qDebug("closeEvent");

  e->ignore();

  if (ob == NULL)
  {
    qWarning("closeEvent: ob == NULL");
    return;
  }

  if (!CWIDGET_test_flag(ob, WF_PERSISTENT))
  {
    CWIDGET_set_flag(ob, WF_IN_CLOSE);
    cancel = GB.Raise(ob, EVENT_Close, NULL);
    CWIDGET_clear_flag(ob, WF_IN_CLOSE);

    if (cancel)
      return;

    if (CWINDOW_Main == (CWINDOW *)ob)
    {
      CWINDOW_Main = 0;
      //qDebug("CWINDOW_Main = %p", CWINDOW_Main);
    }

    //qDebug("post delete %p", this);
    qApp->postEvent(this, new QEvent(QEvent::User));
  }

  //qDebug("Accept !");
  e->accept();
}



bool MyMainWindow::isPersistent(void)
{
  return !testWFlags(WDestructiveClose);
}


void MyMainWindow::setPersistent(bool pers)
{
  if (!pers)
    setWFlags(WDestructiveClose);
  else
    clearWFlags(WDestructiveClose);
}


/* Classe CWindow */

CWindow CWindow::manager;
int CWindow::count = 0;
QPtrDict<CWINDOW> CWindow::dict;

bool CWindow::eventFilter(QObject *o, QEvent *e)
{
  CWIDGET *ob;
  //bool cancel;

  ob = CWidget::get(o);
  if (ob != NULL)
  {
    /*
    if ((e->type() == QEvent::Close)
        && (!CWIDGET_test_flag(ob, WF_IN_CLOSE)))
    {
      CWIDGET_set_flag(ob, WF_IN_CLOSE);

      cancel = GB.Raise(ob, EVENT_Close, NULL);
      if (!cancel)
        ((MyMainWindow *)o)->hide();

      if (CWIDGET_test_flag(ob, WF_PERSISTENT))
        cancel = TRUE;

      if (!cancel)
      {
        GB.Detach(ob);

        if (((CWINDOW *)ob)->menu)
          CMenu::unrefChildren(((CWINDOW *)ob)->menu);

        CWindow::dict.remove(ob);

        if (ob == (CWIDGET *)window_main)
          window_main = NULL;
      }

      CWIDGET_clear_flag(ob, WF_IN_CLOSE);

      return cancel;
    }
    else*/ if (e->type() == QEvent::WindowActivate)
    {
      GB.Raise(ob, EVENT_Activate, NULL);
    }
    else if (e->type() == QEvent::WindowDeactivate)
    {
      GB.Raise(ob, EVENT_Deactivate, NULL);
    }
    else if (e->type() == QEvent::User)
    {
      //qDebug("posted delete %p !", o);
      delete o;
      return true;
    }
  }

  return QObject::eventFilter(o, e);    // standard event processing
}
