/***************************************************************************

  CEditor.cpp

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/


#define __CEDITOR_CPP

#include "common.h"
#include "main.h"

#include <qobject.h>
#include <qpalette.h>
#include "qeditor.h"

#include "CEditor.h"

static long EVENT_Change;    /* text has changed */
//static long EVENT_Click;     /* simple click */
//static long EVENT_Activate;  /* double click */
//static long EVENT_Scroll;


PRIVATE void look_pos(QEditor *wid, int *line, int *col)
{
  if (*line == -1)
    *line = wid->numLines();

  if (*col == -1)
    *col = wid->lineLength(*line);
}

BEGIN_METHOD(CEDITOR_new, void *parent)

  QEditor *wid = new QEditor(QT.GetContainer(PARAM(parent)));

  QObject::connect(wid, SIGNAL(textChanged()), &CEditor::manager, SLOT(changed()));

  QT.InitWidget(wid, _object);
  QT.SetBackgroundRole(_object, QColorGroup::Base);

  wid->show();

END_METHOD


BEGIN_PROPERTY(CEDITOR_text)

  if (READ_PROPERTY)
    GB.ReturnCopyString(THIS->text(), 0);
  else
  {
    THIS->setText(GB.ToZeroString(PROPERTY(GB_STRING)));
    THIS->setEdited(false);
  }

END_PROPERTY


BEGIN_PROPERTY(CEDITOR_line)

  int line, col;

  WIDGET->cursorPosition(&line, &col);

  if (READ_PROPERTY)
    GB.ReturnInteger(line);
  else
  {
    line = PROPERTY(long);
    look_pos(WIDGET, &line, &col);

    WIDGET->setCursorPosition(line, col);
  }

END_PROPERTY


BEGIN_PROPERTY(CEDITOR_column)

  int line, col;

  WIDGET->getCursorPosition(&line, &col);

  if (READ_PROPERTY)
    GB.ReturnInteger(col);
  else
  {
    col = PROPERTY(long);
    look_pos(WIDGET, &line, &col);

    WIDGET->setCursorPosition(line, col);
  }

END_PROPERTY


BEGIN_PROPERTY(CEDITOR_pos)

  int line, col;

  if (READ_PROPERTY)
  {
    WIDGET->getCursorPosition(&line, &col);
    GB.ReturnInteger(WIDGET->toPos(line, col));
  }
  else
  {
    WIDGET->fromPos(PROPERTY(long), &line, &col);
    WIDGET->setCursorPosition(line, col);
  }

END_PROPERTY


BEGIN_PROPERTY(CEDITOR_lines)

  RETURN_SELF();

END_PROPERTY


BEGIN_PROPERTY(CEDITOR_line_count)

  GB.ReturnInteger((long)THIS->numLines());

END_PROPERTY


BEGIN_METHOD(CEDITOR_line_get, long line)

  long line = PARAM(line);

  if (line < 0 || line >= THIS->numLines())
    GB.ReturnNull();
  else
    GB.ReturnCopyString(THIS->textLine(line).latin1(), 0);

END_METHOD


BEGIN_METHOD(CEDITOR_line_get_flag, long line; long flag)

  long line = PARAM(line);

  if (line >= 0 && line < THIS->numLines())
    GB.ReturnBoolean(THIS->lineType(line) & (1 << PARAM(flag)));
  else
    GB.ReturnBoolean(false);

END_METHOD


BEGIN_METHOD(CEDITOR_line_set_flag, long line; long flag; char value)

  long line = PARAM(line);

  if (line >= 0 && line < THIS->numLines())
  {
    long flag = THIS->lineType(line);
    if (PARAM(value))
      THIS->setLineType(line, flag | (1 << PARAM(flag)) );
    else
      THIS->setLineType(line, flag & ~(1 << PARAM(flag)) );
  }

END_METHOD


BEGIN_METHOD_VOID(CEDITOR_clear)

  WIDGET->clear();

END_METHOD


BEGIN_METHOD(CEDITOR_insert, GB_STRING text)

  WIDGET->insert(GB.ToZeroString(PARAM(text)));

END_METHOD


/***************************************************************************

  .GambasEditor.Selection

***************************************************************************/

BEGIN_PROPERTY(CEDITOR_sel_text)

  if (READ_PROPERTY)
    GB.ReturnCopyString(WIDGET->markedText(), 0);
  else
    WIDGET->insert(GB.ToZeroString(PROPERTY(GB_STRING)));

END_PROPERTY


BEGIN_PROPERTY(CEDITOR_sel_length)

  long start, length;

  WIDGET->getSelection(&start, &length);
  GB.ReturnInteger(length);

END_PROPERTY


BEGIN_PROPERTY(CEDITOR_sel_start)

  long start, length;

  WIDGET->getSelection(&start, &length);
  GB.ReturnInteger(start);

END_PROPERTY


BEGIN_METHOD_VOID(CEDITOR_sel_clear)

  WIDGET->deselect();

END_METHOD


BEGIN_METHOD_VOID(CEDITOR_sel_all)

  WIDGET->selectAll();

END_METHOD


BEGIN_METHOD(CEDITOR_sel_select, long start; long length)

  WIDGET->setSelection(PARAM(start), PARAM(length));

END_METHOD


BEGIN_METHOD(CEDITOR_to_pos, long line; long column)

  GB.ReturnInteger(WIDGET->toPos(PARAM(line), PARAM(column)));

END_METHOD



BEGIN_METHOD(CEDITOR_color_get, long index)

  int index = PARAM(index);

  if (index < 0 || index >= NUM_COLORS)
  {
    GB.Error("Bad index");
    return;
  }

  GB.ReturnInteger(WIDGET->getColor(index).rgb() & 0xFFFFFF);

END_METHOD


BEGIN_METHOD(CEDITOR_color_set, long color; long index)

  int index = PARAM(index);

  if (index < 0 || index >= NUM_COLORS)
  {
    GB.Error("Bad index");
    return;
  }

  WIDGET->setColor(index, QColor((QRgb)PARAM(color)));

END_METHOD


GB_DESC CEditorSelectionDesc[] =
{
  GB_DECLARE(".GambasEditor.Selection", 0), GB_VIRTUAL_CLASS(),

  GB_PROPERTY("Text", "s", CEDITOR_sel_text),
  GB_PROPERTY_READ("Length", "i", CEDITOR_sel_length),
  GB_PROPERTY_READ("Start", "i", CEDITOR_sel_start),

  GB_METHOD("Clear", NULL, CEDITOR_sel_clear, NULL),
  GB_METHOD("Select", NULL, CEDITOR_sel_select, "(Start)i(Length)i"),
  GB_METHOD("SelectAll", NULL, CEDITOR_sel_all, NULL),

  GB_END_DECLARE
};



GB_DESC CEditorLinesDesc[] =
{
  GB_DECLARE(".GambasEditor.Lines", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("_get", "s", CEDITOR_line_get, "(Line)i"),
  GB_METHOD("SetFlag", NULL, CEDITOR_line_set_flag, "(Line)i(Flag)i(Value)b"),
  GB_METHOD("GetFlag", "b", CEDITOR_line_get_flag, "(Line)i(Flag)i"),
  GB_PROPERTY_READ("Count", "i", CEDITOR_line_count),

  GB_END_DECLARE
};


GB_DESC CEditorColorsDesc[] =
{
  GB_DECLARE(".GambasEditor.Colors", 0), GB_VIRTUAL_CLASS(),

  GB_METHOD("_get", "i", CEDITOR_color_get, "(Style)i"),
  GB_METHOD("_set", NULL, CEDITOR_color_set, "(Color)i(Style)i"),

  GB_CONSTANT("Background", "i", QEditorRow::Background),
  GB_CONSTANT("Normal", "i", QEditorRow::Normal),
  GB_CONSTANT("Keyword", "i", QEditorRow::Keyword),
  GB_CONSTANT("Function", "i", QEditorRow::Subr),
  GB_CONSTANT("Operator", "i", QEditorRow::Operator),
  GB_CONSTANT("Symbol", "i", QEditorRow::Symbol),
  GB_CONSTANT("Number", "i", QEditorRow::Number),
  GB_CONSTANT("String", "i", QEditorRow::String),
  GB_CONSTANT("Comment", "i", QEditorRow::Commentary),
  GB_CONSTANT("Breakpoint", "i", QEditorRow::Breakpoint),
  GB_CONSTANT("Current", "i", QEditorRow::Current),

  GB_END_DECLARE
};



GB_DESC CEditorDesc[] =
{
  GB_DECLARE("GambasEditor", sizeof(CEditor)), GB_INHERITS("Control"),

  GB_CONSTANT("Current", "i", 0),
  GB_CONSTANT("Breakpoint", "i", 1),

  GB_METHOD("_new", NULL, CEDITOR_new, "Container;"),

  GB_PROPERTY("Text", "s", CEDITOR_text),

  GB_PROPERTY("Line", "i", CEDITOR_line),
  GB_PROPERTY("Column", "i", CEDITOR_column),
  GB_PROPERTY("Pos", "i", CEDITOR_pos),

  GB_METHOD("Clear", NULL, CEDITOR_clear, NULL),
  GB_METHOD("Insert", NULL, CEDITOR_insert, "s"),

  GB_METHOD("ToPos", "i", CEDITOR_to_pos, "(Line)i(Column)i"),

  GB_PROPERTY_READ("Lines", ".GambasEditor.Lines", CEDITOR_lines),
  GB_PROPERTY_READ("Selection", ".GambasEditor.Selection", CEDITOR_lines),
  GB_PROPERTY_READ("Colors", ".GambasEditor.Colors", CEDITOR_lines),

  GB_EVENT("Change", NULL, NULL, &EVENT_Change),

  GB_END_DECLARE
};


CEditor CEditor::manager;

void CEditor::changed(void)
{
  GB.Raise(QT.GetObject((QWidget *)sender()), EVENT_Change, NULL);
}

/*
void CGridView::activated(void)
{
  RAISE_EVENT(EVENT_Activate, NULL);
}

void CGridView::clicked(void)
{
  RAISE_EVENT(EVENT_Click, NULL);
}

void CGridView::scrolled(void)
{
  RAISE_EVENT(EVENT_Scroll, NULL);
}
*/
