/***************************************************************************

  main.cpp

  The interface between the QT plug-in and the Gambas interpreter

  (c) 2000-2001 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __MAIN_C

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "gambas.h"

#include <qapplication.h>
#include <qobject.h>
#include <qmessagebox.h>
#include <qclipboard.h>

#include "gb.qt.h"

#include "CFont.h"
#include "CScreen.h"
#include "CWidget.h"
#include "CWindow.h"
#include "CButton.h"
#include "CContainer.h"
#include "CLabel.h"
#include "CListBox.h"
#include "CTextBox.h"
#include "CTextArea.h"
#include "CImage.h"
#include "CMenu.h"
#include "CPanel.h"
#include "CMouse.h"
#include "CColor.h"
#include "CConst.h"
#include "CCheckBox.h"
#include "CFrame.h"
#include "CRadioButton.h"
#include "CTreeView.h"
#include "CGridView.h"
#include "CTabStrip.h"
#include "CDialog.h"
#include "CPicture.h"
#include "CClipboard.h"
#include "CDraw.h"
#include "CTimer.h"
#include "CWatch.h"

#include "main.h"

/*#define DEBUG*/


static void my_main(int *argc, char **argv)
{
  new QApplication(*argc, argv);
  //QObject::connect(qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));
}


static int my_loop()
{
  int ret;

  if (CWindow::dict.count() == 0)
    return 0;

  ret = qApp->exec();

  //qApp->clipboard()->clear();

  return ret;
}


static void my_wait(long duration)
{
  qApp->processEvents(duration);
}


static int my_message(GAMBAS_MESSAGE *msg)
{
  QMessageBox::Icon icon;
  int button[3];
  int i;

  switch (msg->icon)
  {
    case GB_MSG_INFORMATION:
      icon = QMessageBox::Information;
      break;
    case GB_MSG_WARNING:
      icon = QMessageBox::Warning;
      break;
    case GB_MSG_CRITICAL:
      icon = QMessageBox::Critical;
      break;
    default:
      icon = QMessageBox::Information;
  }

  button[0] = button[1] = button[2] = QMessageBox::NoButton;

  switch (msg->type)
  {
    case GB_MSG_USER_DEFINED:
      for (i = 0; i < 3; i++)
        button[i] = msg->button[i] ? (i + 1) : QMessageBox::NoButton;
      break;

    case GB_MSG_OK_ONLY:
      button[0] = QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape;
      break;

    case GB_MSG_OK_CANCEL:
      button[0] = QMessageBox::Ok | QMessageBox::Default;
      button[1] = QMessageBox::Cancel | QMessageBox::Escape;
      break;

    case GB_MSG_YES_NO:
      button[0] = QMessageBox::Yes | QMessageBox::Default;
      button[1] = QMessageBox::No | QMessageBox::Escape;
      break;

    case GB_MSG_YES_NO_CANCEL:
      button[0] = QMessageBox::Yes | QMessageBox::Default;
      button[1] = QMessageBox::No;
      button[2] = QMessageBox::Cancel | QMessageBox::Escape;
      break;
  }

  QMessageBox mb(msg->title, msg->text, icon, button[0], button[1], button[2]);

  if (msg->type == GB_MSG_USER_DEFINED)
  {
    for (i = 0; i < 3; i++)
    {
      if (msg->button[i] != NULL)
      {
        qDebug(msg->button[i]);
        mb.setButtonText(i + 1, msg->button[i]);
      }
    }
  }

  switch (mb.exec())
  {
    case QMessageBox::Yes:
      return GB_MSG_YES;
    case QMessageBox::No:
      return GB_MSG_NO;
    case QMessageBox::Ok:
      return GB_MSG_OK;
    case QMessageBox::Cancel:
      return GB_MSG_CANCEL;
    case QMessageBox::Abort:
      return GB_MSG_ABORT;
    case QMessageBox::Retry:
      return GB_MSG_RETRY;
    case QMessageBox::Ignore:
      return GB_MSG_IGNORE;
    default:
      return 0;
  }
}

static void my_watch(int fd, int type, void *callback, long param)
{
  CWatch::watch(fd, type, (GB_WATCH_CALLBACK)callback, param);
}


static void QT_InitWidget(QWidget *widget, void *object)
{
  CWIDGET_new(widget, object);
}

static void *QT_GetObject(QWidget *widget)
{
  return CWidget::get((QObject *)widget);
}

static QWidget *QT_GetContainer(void *object)
{
  return CONTAINER(object);
}

static void QT_SetBackgroundRole(void *object, int role)
{
  ((CWIDGET *)object)->background = role;
}

/*
static void QT_SetProperties(void *klass, char *prop)
{
  CWidget::setProperties(klass, prop);
}
*/

extern "C" {

GB_INTERFACE GB;

void *GB_QT_1[] = {

  (void *)1,

  (void *)QT_InitWidget,
  (void *)QT_GetObject,
  (void *)QT_GetContainer,
  (void *)QT_SetBackgroundRole,

  NULL
  };

int GB_INIT(void)
{
  GB.Hook(GB_HOOK_MAIN, (void *)my_main);
  GB.Hook(GB_HOOK_LOOP, (void *)my_loop);
  GB.Hook(GB_HOOK_WAIT, (void *)my_wait);
  GB.Hook(GB_HOOK_MESSAGE, (void *)my_message);
  GB.Hook(GB_HOOK_WATCH, (void *)my_watch);

  GB.Declare(

    CBorderDesc, CColorDesc, CAlignDesc, CMouseDesc, CScrollDesc, CKeyDesc,
    CLineDesc, CFillDesc,
    CPictureDesc, CClipboardDesc, CFontDesc, CScreenDesc,
    CWidgetDesc, CChildrenDesc,
    CContainerDesc,
    CMenuChildrenDesc, CMenuDesc,
    CLabelDesc, CImageDesc,
    CButtonDesc, CToggleButtonDesc,
    CCheckBoxDesc, CRadioButtonDesc,
    CTextBoxSelectionDesc, CTextBoxDesc, CComboBoxDesc,
    CTextAreaSelectionDesc, CTextAreaLinesDesc, CTextAreaDesc,
    CListBoxSelectedDesc, CListBoxDesc,
    CTreeViewItemDesc, CTreeViewDesc, CListViewDesc,
    CGridItemDesc, CGridRowsDesc, CGridColumnsDesc, CGridViewDesc,
    CFrameDesc, CPanelDesc,
    CTabChildrenDesc, CTabDesc, CTabStripDesc,
    CWindowMenusDesc, CWindowDesc, CWindowsDesc, CFormDesc,
    CDialogDesc,
    CDrawDesc,
    CTimerDesc,

    NULL);

  return TRUE;
}

void GB_EXIT()
{
  delete qApp;
}

}


