/***************************************************************************

  common.h

  Common useful macros

  Copyright (C) 2000 Benot Minisini <gambas@freesurf.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#ifndef __COMMON_H
#define __COMMON_H

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

#if !defined(__cplusplus)

enum
{
  FALSE = 0,
  TRUE = 1
};

#endif

typedef
  char boolean;

typedef
  unsigned char byte;

#define PUBLIC
#define PRIVATE static
#define INLINE __inline__
#define EXTERN extern
#define PACKED __attribute__((packed))
#define NORETURN __attribute__((noreturn))
#define CONST __attribute__((const))

#define _(x) x

#define strclr(s) (*(s) = 0)

#define Clear(s) (memset(s, 0, sizeof(*s)))

#ifndef offsetof
#define offsetof(_type, _arg) ((size_t)&(((_type *)0)->_arg))
#endif

#define stradd(d, s) \
({ \
  char *_d = (d); \
  char *_s = (s); \
  \
  for(;;) \
  { \
    if ((*_d = *_s) == 0) \
      break; \
    \
    _d++; \
    _s++; \
  } \
  \
  _d; \
})

/*
__inline__ long Max(long a, long b)
{
  return (a > b) ? a : b;
}

__inline__ long Min(long a, long b)
{
  return (a > b) ? b : a;
}
*/

#define Max(a, b) ({ int _a = (a), _b = (b); _a > _b ? _a : _b; })
#define Min(a, b) ({ int _a = (a), _b = (b); _a < _b ? _a : _b; })
#define MinMax(v, a, b) ({ int _v = (v), _a = (a), _b = (b); _v < _a ? _a : (_v > _b ? _b : _v); })


/*
__inline__ long MinMax(long val, long a, long b)
{
  if (val < a)
    return a;
  else if (val > b)
    return b;
  else
    return val;
}
*/


#endif /* _COMMON_H */
